/* Copyright (C) 2000 Cyrano SA

   This file is part of OpenSTA

   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.

   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html

   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc   CyrStRepository
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrStHelper_inl
#define _CyrStHelper_inl

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <solutions.h>

///////////////////////////////////////////////////////////////////////////////
// @func    This function converts a Microsoft DATE type into a value that can
//          be used by CTTime class.
// @rdesc   The total number of seconds since CTTime's base date/time.
// @parm    The DATE to convert.
inline long DateToSeconds( DATE dt )
{
	// In typical Microsoft fashion their time is different to UTC.
	// The base date is 30/12/1899 00:00:00 and UTC's base date is 
	// 01/01/1970 00:00:00.  Need to take a 70 years and two days off the value.
	
	// Calculate number of days, hours, minutes and seconds in seconds.
	dt = dt * 60.0 * 60.0 * 24.0;
	// remove difference in seconds between both dates
	dt -= 2177452800.0;
	return (long)dt;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method retrieves the default directory for storage of
//          OpenSTARep projects.  This value is stored in the system registry.
//          If the value has not been set then the default is the temp 
//          directory specified by Windows.
// @rdesc   The directory name.
inline string GetRootDirectory()
{
	HKEY hRegKey;
	DWORD length = MAX_PATH;
	char* buf = new char[length];

	// Default directory is temp dir
	GetTempPath( length, buf );
	string ret( buf );

	// Get handle to registry entry
	long error = RegOpenKeyEx( HKEY_CURRENT_USER, CURRENT_PROJECT_KEY, 0, 
							   KEY_QUERY_VALUE, &hRegKey );
	if ( error == ERROR_SUCCESS )
	{
		// Get value of registry entry
		RegQueryValueEx( hRegKey, "Path", 0, NULL, 
						 (unsigned char*)buf, &length );
		ret = buf;
        RegCloseKey( hRegKey );
	}

	// Tidy memory
	delete [] buf;

    // Make sure directory has a '\' character on the end
    if ( ret[ret.length()-1] != '\\' )
        ret += '\\';

	return ret;
}

#endif
