/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4514)
#endif  // WIN32

#define RW_MULTI_THREAD

//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifdef  WIN32
#  ifndef _WINSOCKAPI_
#    pragma warning(disable:4201)
#    include <winsock.h>
#    pragma warning(default:4201)
#  endif    // _WINSOCKAPI_
#  include <string.h>
#endif  // WIN32

#ifdef SUN
#include <stdio.h>
#  include <pthread.h>
#  include <netdb.h>
#  include <errno.h>
#endif
#ifdef  DGUX
#  ifdef    RW_MULTI_THREAD
#    ifndef _POSIX4A_DRAFT6_SOURCE
#      define   _POSIX4A_DRAFT6_SOURCE
#    endif  // _POSIX4A_DRAFT6_SOURCE
#    include <pthread.h>
#  endif    // RW_MULTI_THREAD
#  include <string.h>
#  include <netdb.h>
#endif  // DGUX

#ifdef  AIX
#  ifdef    RW_MULTI_THREAD
#    include <pthread.h>
#  endif    // RW_MULTI_THREAD
#  include <string.h>
#  include <netdb.h>
#endif  // AIX

#ifdef  HPUX
#  ifdef    RW_MULTI_THREAD
#    include <pthread.h>
#  endif    // RW_MULTI_THREAD
#  include <string.h>
#  include <netdb.h>
#endif  // HPUX

#ifndef __MTSAFE__
#  include "mtsafe.hpp"
#endif  // __MTSAFE__

#include "xalloc.hpp"


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CTMTSafe //
///////////////////////////////////////////////////////////////////////////////
// Class used to acces to a multi-thread safe version of the standard routines.

//====================================================================< Types >
#ifdef  RW_MULTI_THREAD
#  ifdef DGUX
struct dataStdCRT
{
    char*   zWork ;
} ;
static pthread_once_t   onceStdCRT = PTHREAD_ONCE_INIT ;
static pthread_key_t    keyStdCRT ;
void destructorStdCRT (struct dataStdCRT* pDataStdCRT)
{
    if (pDataStdCRT != NULL)
    {
        pDataStdCRT->zWork = NULL ;
        delete pDataStdCRT ;
        pDataStdCRT = NULL ;
    }
} // destructorStdCRT
void initRoutineStdCRT (void)
{
    pthread_key_create (&keyStdCRT, destructorStdCRT) ;
} // initRoutineStdCRT
#  endif    // DGUX
#  ifdef SUN
struct hostent_thread_data
{
    struct hostent      hostEnt ;
    char * hostBuffer ;
} ;
struct servent_thread_data
{
    struct servent      servEnt ;
    char * servBuffer ;
} ;
struct dataStdCRT
{
    char*                       zWork ;
    char*                       zErrMsg ;
    struct hostent_thread_data* pHostEntThreadData ;
    struct servent_thread_data* pServEntThreadData ;
} ;
static pthread_once_t   onceStdCRT = PTHREAD_ONCE_INIT ;
static pthread_key_t    keyStdCRT ;
void destructorStdCRT (struct dataStdCRT* pDataStdCRT)
{
    if (pDataStdCRT != NULL)
    {
        pDataStdCRT->zWork = NULL ;
        if (pDataStdCRT->zErrMsg)
        {
            delete [] pDataStdCRT->zErrMsg ;
            pDataStdCRT->zErrMsg = NULL ;
        }
        if (pDataStdCRT->pHostEntThreadData)
        {
            delete pDataStdCRT->pHostEntThreadData ;
            pDataStdCRT->pHostEntThreadData = NULL ;
        }
        if (pDataStdCRT->pServEntThreadData)
        {
            delete pDataStdCRT->pServEntThreadData ;
            pDataStdCRT->pServEntThreadData = NULL ;
        }
        delete pDataStdCRT ;
        pDataStdCRT = NULL ;
    }
} // destructorStdCRT
void initRoutineStdCRT (void)
{
    pthread_key_create (&keyStdCRT, (void(*)(void*))destructorStdCRT) ;
} // initRoutineStdCRT
#  endif    // SUN
#  ifdef AIX
struct hostent_thread_data
{
    struct hostent      hostEnt ;
    struct hostent_data hostEntData ;
} ;
struct servent_thread_data
{
    struct servent      servEnt ;
    struct servent_data servEntData ;
} ;
struct dataStdCRT
{
    char*                       zWork ;
    char*                       zErrMsg ;
    struct hostent_thread_data* pHostEntThreadData ;
    struct servent_thread_data* pServEntThreadData ;
} ;
static pthread_once_t   onceStdCRT = PTHREAD_ONCE_INIT ;
static pthread_key_t    keyStdCRT ;
void destructorStdCRT (struct dataStdCRT* pDataStdCRT)
{
    if (pDataStdCRT != NULL)
    {
        pDataStdCRT->zWork = NULL ;
        if (pDataStdCRT->zErrMsg)
        {
            delete [] pDataStdCRT->zErrMsg ;
            pDataStdCRT->zErrMsg = NULL ;
        }
        if (pDataStdCRT->pHostEntThreadData)
        {
            delete pDataStdCRT->pHostEntThreadData ;
            pDataStdCRT->pHostEntThreadData = NULL ;
        }
        if (pDataStdCRT->pServEntThreadData)
        {
            delete pDataStdCRT->pServEntThreadData ;
            pDataStdCRT->pServEntThreadData = NULL ;
        }
        delete pDataStdCRT ;
        pDataStdCRT = NULL ;
    }
} // destructorStdCRT
void initRoutineStdCRT (void)
{
    pthread_key_create (&keyStdCRT, destructorStdCRT) ;
} // initRoutineStdCRT
#  endif    // AIX
#  ifdef HPUX
struct hostent_thread_data
{
    struct hostent      hostEnt ;
    struct hostent_data hostEntData ;
} ;
struct servent_thread_data
{
    struct servent      servEnt ;
    struct servent_data servEntData ;
} ;
struct dataStdCRT
{
    char*                       zWork ;
    char*                       zErrMsg ;
    struct hostent_thread_data* pHostEntThreadData ;
    struct servent_thread_data* pServEntThreadData ;
} ;
static pthread_once_t   onceStdCRT = pthread_once_init ;
static pthread_key_t    keyStdCRT ;
void destructorStdCRT (struct dataStdCRT* pDataStdCRT)
{
    if (pDataStdCRT != NULL)
    {
        pDataStdCRT->zWork = NULL ;
        if (pDataStdCRT->zErrMsg)
        {
            delete [] pDataStdCRT->zErrMsg ;
            pDataStdCRT->zErrMsg = NULL ;
        }
        if (pDataStdCRT->pHostEntThreadData)
        {
            delete pDataStdCRT->pHostEntThreadData ;
            pDataStdCRT->pHostEntThreadData = NULL ;
        }
        if (pDataStdCRT->pServEntThreadData)
        {
            delete pDataStdCRT->pServEntThreadData ;
            pDataStdCRT->pServEntThreadData = NULL ;
        }
        delete pDataStdCRT ;
        pDataStdCRT = NULL ;
    }
} // destructorStdCRT
void initRoutineStdCRT (void)
{
    pthread_keycreate (&keyStdCRT, (pthread_destructor_t) destructorStdCRT) ;
} // initRoutineStdCRT
#  endif    // HPUX
#endif  // RW_MULTI_THREAD

//========================================================< Standard routines >

//-----------------------------------------------------------------------------
// CTMTSafe::strerror
//-----------------------------------------------------------------------------
char* CTMTSafe::strerror (int iErrNum)
{
#ifdef  RW_MULTI_THREAD
#  ifdef    WIN32
    return ::strerror (iErrNum) ;
#  endif    // WIN32
#  ifdef    DGUX
    return ::strerror (iErrNum) ;
#  endif    // DGUX
#  ifdef    SUN
    return ::strerror (iErrNum) ;
#  endif    // DGUX
#  ifdef    AIX
    char*   zErrMsg ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = (struct dataStdCRT*) pthread_getspecific (keyStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->zErrMsg == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    zErrMsg = pDataStdCRT->zErrMsg = new char [255] ;
                }
                catch (CTxalloc&)
                {
                    zErrMsg = NULL ;
                }
            }
        }
        else
        {   // Error from pDataStdCRT.
            zErrMsg = NULL ;
        }
        if (zErrMsg)
        {
            if (strerror_r (iErrNum, zErrMsg, 255) == -1)
            {   // Error from strerror_r ().
                zErrMsg = NULL ;
            }
        }
    }
    else
    {   // Error from pthread_once ().
        zErrMsg = NULL ;
    }

    return zErrMsg ;
#  endif    // AIX
#  ifdef    HPUX
    char*   zErrMsg ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = NULL ;
        pthread_getspecific (keyStdCRT, (pthread_addr_t*) &pDataStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->zErrMsg == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    zErrMsg = pDataStdCRT->zErrMsg = new char [255] ;
                }
                catch (CTxalloc&)
                {
                    zErrMsg = NULL ;
                }
            }
        }
        else
        {   // Error from pDataStdCRT.
            zErrMsg = NULL ;
        }
        if (zErrMsg)
        {
            if (strerror_r (iErrNum, zErrMsg, 255) == -1)
            {   // Error from strerror_r ().
                zErrMsg = NULL ;
            }
        }
    }
    else
    {   // Error from pthread_once ().
        zErrMsg = NULL ;
    }

    return zErrMsg ;
#  endif    // HPUX
#else
    return ::strerror (iErrNum) ;
#endif  // RW_MULTI_THREAD
} // CTMTSafe::strerror

//-----------------------------------------------------------------------------
// CTMTSafe::strtok
//-----------------------------------------------------------------------------

char* CTMTSafe::strtok (char* zString1, char* zString2)
{
#ifdef  RW_MULTI_THREAD
#  ifdef    WIN32
    return ::strtok (zString1, zString2) ;
#  endif    // WIN32
#  ifdef    DGUX
    char*   zToken ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = NULL ;
        pthread_getspecific (keyStdCRT, (pthread_addr_t*) &pDataStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            zToken = strtok_r (zString1, zString2, &pDataStdCRT->zWork) ;
        }
    }
    else
    {   // Error from pthread_once ().
        zToken = NULL ;
    }

    return zToken ;
#  endif    // DGUX
#  ifdef    SUN
    char*   zToken ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = (struct dataStdCRT*) pthread_getspecific (keyStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            zToken = strtok_r (zString1, zString2, &pDataStdCRT->zWork) ;
        }
    }
    else
    {   // Error from pthread_once ().
        zToken = NULL ;
    }

    return zToken ;
#  endif    // SUN
#  ifdef    AIX
    char*   zToken ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = (struct dataStdCRT*) pthread_getspecific (keyStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            zToken = strtok_r (zString1, zString2, &pDataStdCRT->zWork) ;
        }
    }
    else
    {   // Error from pthread_once ().
        zToken = NULL ;
    }

    return zToken ;
#  endif    // AIX
#  ifdef    HPUX
    char*   zToken ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = NULL ;
        pthread_getspecific (keyStdCRT, (pthread_addr_t*) &pDataStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            zToken = strtok_r (zString1, zString2, &pDataStdCRT->zWork) ;
        }
    }
    else
    {   // Error from pthread_once ().
        zToken = NULL ;
    }

    return zToken ;
#  endif    // HPUX
#else
    return ::strtok (zString1, zString2) ;
#endif  // RW_MULTI_THREAD
} // strtok

//-----------------------------------------------------------------------------
// CTMTSafe::gethostbyname
//-----------------------------------------------------------------------------

struct hostent* CTMTSafe::gethostbyname (const char* zName)
{
#ifdef RW_MULTI_THREAD
#  ifdef    WIN32
    return ::gethostbyname (zName) ;
#  endif    // WIN32
#  ifdef    DGUX
    return ::gethostbyname (zName) ;
#  endif    // DGUX
#  ifdef    SUNSIMPLE
    	struct hostent* pHostEnt ;
	pHostEnt = gethostbyname (zName) ;
	return pHostEnt;
#  endif    // DGUX
#  ifdef SUN
    struct hostent* pHostEnt ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = (struct dataStdCRT*) pthread_getspecific (keyStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->pHostEntThreadData == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    pDataStdCRT->pHostEntThreadData = new struct hostent_thread_data ;
                    if (pDataStdCRT->pHostEntThreadData)
                    {
                    	memset (pDataStdCRT->pHostEntThreadData, 0, sizeof (struct hostent_thread_data) ) ;
                    }
		    pDataStdCRT->pHostEntThreadData->hostBuffer = new char[4096] ;
                }
                catch (CTxalloc&)
                {
                    pDataStdCRT->pHostEntThreadData = NULL ;
                }
            }
            if (pDataStdCRT->pHostEntThreadData)
            {
		int iNb = 1 ;
		int iError = 0 ;
                while (gethostbyname_r (zName, &pDataStdCRT->pHostEntThreadData->hostEnt, pDataStdCRT->pHostEntThreadData->hostBuffer, iNb * 4096, &iError) == NULL)
                {   
		    if (errno == ERANGE)
		    {
		   	delete [] pDataStdCRT->pHostEntThreadData->hostBuffer ;
		    	iNb ++ ;
		    	pDataStdCRT->pHostEntThreadData->hostBuffer = new char[iNb * 4096] ;
		    } 
		    else
		    {
			return NULL ;
		    }
               }
                pHostEnt = &pDataStdCRT->pHostEntThreadData->hostEnt ;
            }
            else
            {   // Error from pDataStdCRT->pHostEntThreadData.
                pHostEnt = NULL ;
            }
        }
        else
        {   // Error from pDataStdCRT.
            pHostEnt = NULL ;
        }
    }
    else
    {   // Error from pthread_once ().
        pHostEnt = NULL ;
    }

    return pHostEnt ;
#  endif    // SUN
#  ifdef AIX
    struct hostent* pHostEnt ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = (struct dataStdCRT*) pthread_getspecific (keyStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->pHostEntThreadData == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    pDataStdCRT->pHostEntThreadData = new struct hostent_thread_data ;
                }
                catch (CTxalloc&)
                {
                    pDataStdCRT->pHostEntThreadData = NULL ;
                }
                if (pDataStdCRT->pHostEntThreadData)
                {
                    memset (pDataStdCRT->pHostEntThreadData, 0, sizeof (struct hostent_thread_data) ) ;
                }
            }
            if (pDataStdCRT->pHostEntThreadData)
            {
                if (gethostbyname_r (zName, &pDataStdCRT->pHostEntThreadData->hostEnt, &pDataStdCRT->pHostEntThreadData->hostEntData) == -1)
                {   // Error from gethostbyname_r ().
                    pHostEnt = NULL ;
                }
                else
                {
                    pHostEnt = &pDataStdCRT->pHostEntThreadData->hostEnt ;
                }
            }
            else
            {   // Error from pDataStdCRT->pHostEntThreadData.
                pHostEnt = NULL ;
            }
        }
        else
        {   // Error from pDataStdCRT.
            pHostEnt = NULL ;
        }
    }
    else
    {   // Error from pthread_once ().
        pHostEnt = NULL ;
    }

    return pHostEnt ;
#  endif    // AIX
#  ifdef    HPUX
    struct hostent* pHostEnt ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = NULL ;
        pthread_getspecific (keyStdCRT, (pthread_addr_t*) &pDataStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->pHostEntThreadData == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    pDataStdCRT->pHostEntThreadData = new struct hostent_thread_data ;
                }
                catch (CTxalloc&)
                {
                    pDataStdCRT->pHostEntThreadData = NULL ;
                }
                if (pDataStdCRT->pHostEntThreadData)
                {
                    memset (pDataStdCRT->pHostEntThreadData, 0, sizeof (struct hostent_thread_data) ) ;
                }
            }
            if (pDataStdCRT->pHostEntThreadData)
            {
                if (gethostbyname_r (zName, &pDataStdCRT->pHostEntThreadData->hostEnt, &pDataStdCRT->pHostEntThreadData->hostEntData) == -1)
                {   // Error from gethostbyname_r ().
                    pHostEnt = NULL ;
                }
                else
                {
                    pHostEnt = &pDataStdCRT->pHostEntThreadData->hostEnt ;
                }
            }
            else
            {   // Error from pDataStdCRT->pHostEntThreadData.
                pHostEnt = NULL ;
            }
        }
        else
        {   // Error from pDataStdCRT.
            pHostEnt = NULL ;
        }
    }
    else
    {   // Error from pthread_once ().
        pHostEnt = NULL ;
    }

    return pHostEnt ;
#  endif    // HPUX
#else
    return ::gethostbyname (zName) ;
#endif
} // gethostbyname

//-----------------------------------------------------------------------------
// CTMTSafe::gethostbyaddr
//-----------------------------------------------------------------------------

struct hostent* CTMTSafe::gethostbyaddr (const char* zAddr, int iLen, int iType)
{
#ifdef  RW_MULTI_THREAD
#  ifdef    WIN32
    return ::gethostbyaddr (zAddr, iLen, iType) ;
#  endif    // WIN32
#  ifdef    DGUX
    return ::gethostbyaddr (zAddr, iLen, iType) ;
#  endif    // DGUX
#  ifdef    SUN
    struct hostent* pHostEnt ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = (struct dataStdCRT*) pthread_getspecific (keyStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->pHostEntThreadData == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    pDataStdCRT->pHostEntThreadData = new struct hostent_thread_data ;
                    if (pDataStdCRT->pHostEntThreadData)
                    {
                    	memset (pDataStdCRT->pHostEntThreadData, 0, sizeof (struct hostent_thread_data) ) ;
                    }
		    pDataStdCRT->pHostEntThreadData->hostBuffer = new char[4096] ;
                }
                catch (CTxalloc&)
                {
                    pDataStdCRT->pHostEntThreadData = NULL ;
                }
            }
            if (pDataStdCRT->pHostEntThreadData)
            {
		int iNb = 1 ;
		int iError = 0 ;
                while (gethostbyaddr_r (zAddr, iLen, iType, &pDataStdCRT->pHostEntThreadData->hostEnt, pDataStdCRT->pHostEntThreadData->hostBuffer, iNb * 4096, &iError) == NULL)
                {   
		    if (errno == ERANGE)
		    {
		   	delete [] pDataStdCRT->pHostEntThreadData->hostBuffer ;
		    	iNb ++ ;
		    	pDataStdCRT->pHostEntThreadData->hostBuffer = new char[iNb * 4096] ;
		    } 
		    else
			return NULL ;
                }
                pHostEnt = &pDataStdCRT->pHostEntThreadData->hostEnt ;
            }
            else
            {   // Error from pDataStdCRT->pHostEntThreadData.
                pHostEnt = NULL ;
            }
        }
        else
        {   // Error from pDataStdCRT.
            pHostEnt = NULL ;
        }
    }
    else
    {   // Error from pthread_once ().
        pHostEnt = NULL ;
    }

    return pHostEnt ;
#  endif    // SUN
#  ifdef    AIX
    struct hostent* pHostEnt ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = (struct dataStdCRT*) pthread_getspecific (keyStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->pHostEntThreadData == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    pDataStdCRT->pHostEntThreadData = new struct hostent_thread_data ;
                }
                catch (CTxalloc&)
                {
                    pDataStdCRT->pHostEntThreadData = NULL ;
                }
                if (pDataStdCRT->pHostEntThreadData)
                {
                    memset (pDataStdCRT->pHostEntThreadData, 0, sizeof (struct hostent_thread_data) ) ;
                }
            }
            if (pDataStdCRT->pHostEntThreadData)
            {
                if (gethostbyaddr_r (zAddr, iLen, iType, &pDataStdCRT->pHostEntThreadData->hostEnt, &pDataStdCRT->pHostEntThreadData->hostEntData) == -1)
                {   // Error from gethostbyaddr_r ().
                    pHostEnt = NULL ;
                }
                else
                {
                    pHostEnt = &pDataStdCRT->pHostEntThreadData->hostEnt ;
                }
            }
            else
            {   // Error from pDataStdCRT->pHostEntThreadData.
                pHostEnt = NULL ;
            }
        }
        else
        {   // Error from pDataStdCRT.
            pHostEnt = NULL ;
        }
    }
    else
    {   // Error from pthread_once ().
        pHostEnt = NULL ;
    }

    return pHostEnt ;
#  endif    // AIX
#  ifdef    HPUX
    struct hostent* pHostEnt ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = NULL ;
        pthread_getspecific (keyStdCRT, (pthread_addr_t*) &pDataStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->pHostEntThreadData == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    pDataStdCRT->pHostEntThreadData = new struct hostent_thread_data ;
                }
                catch (CTxalloc&)
                {
                    pDataStdCRT->pHostEntThreadData = NULL ;
                }
                if (pDataStdCRT->pHostEntThreadData)
                {
                    memset (pDataStdCRT->pHostEntThreadData, 0, sizeof (struct hostent_thread_data) ) ;
                }
            }
            if (pDataStdCRT->pHostEntThreadData)
            {
                if (gethostbyaddr_r (zAddr, iLen, iType, &pDataStdCRT->pHostEntThreadData->hostEnt, &pDataStdCRT->pHostEntThreadData->hostEntData) == -1)
                {   // Error from gethostbyaddr_r ().
                    pHostEnt = NULL ;
                }
                else
                {
                    pHostEnt = &pDataStdCRT->pHostEntThreadData->hostEnt ;
                }
            }
            else
            {   // Error from pDataStdCRT->pHostEntThreadData.
                pHostEnt = NULL ;
            }
        }
        else
        {   // Error from pDataStdCRT.
            pHostEnt = NULL ;
        }
    }
    else
    {   // Error from pthread_once ().
        pHostEnt = NULL ;
    }

    return pHostEnt ;
#  endif    // HPUX
#else
    return ::gethostbyaddr (zAddr, iLen, iType) ;
#endif  // RW_MULTI_THREAD
} // gethostbyaddr

//-----------------------------------------------------------------------------
// CTMTSafe::getservbyname
//-----------------------------------------------------------------------------

struct servent* CTMTSafe::getservbyname (const char* zName, const char* zProto)
{
#ifdef  RW_MULTI_THREAD
#  ifdef    WIN32
    return ::getservbyname (zName, zProto) ;
#  endif    // WIN32
#  ifdef    DGUX
    return ::getservbyname (zName, zProto) ;
#  endif    // DGUX
#  ifdef    SUN
    struct servent* pServEnt ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = (struct dataStdCRT*) pthread_getspecific (keyStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->pHostEntThreadData == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    pDataStdCRT->pServEntThreadData = new struct servent_thread_data ;
                    if (pDataStdCRT->pServEntThreadData)
                    {
                    	memset (pDataStdCRT->pServEntThreadData, 0, sizeof (struct servent_thread_data) ) ;
                    }
		    pDataStdCRT->pServEntThreadData->servBuffer = new char[4096] ;
                }
                catch (CTxalloc&)
                {
                    pDataStdCRT->pServEntThreadData = NULL ;
                }
            }
            if (pDataStdCRT->pServEntThreadData)
            {
		int iNb = 1 ;
		int iError = 0 ;
                while (getservbyname_r (zName, zProto, &pDataStdCRT->pServEntThreadData->servEnt, pDataStdCRT->pServEntThreadData->servBuffer, iNb * 4096) == NULL)
                {   
		    if (errno == ERANGE)
		    {
		   	delete [] pDataStdCRT->pServEntThreadData->servBuffer ;
		    	iNb ++ ;
		    	pDataStdCRT->pServEntThreadData->servBuffer = new char[iNb * 4096] ;
		    } 
		    else
			return NULL ;
                }
                pServEnt = &pDataStdCRT->pServEntThreadData->servEnt ;
            }
            else
            {   // Error from pDataStdCRT->pServEntThreadData.
                pServEnt = NULL ;
            }
        }
        else
        {   // Error from pDataStdCRT.
            pServEnt = NULL ;
        }
    }
    else
    {   // Error from pthread_once ().
        pServEnt = NULL ;
    }

    return pServEnt ;
#  endif    // SUN
#  ifdef    AIX
    struct servent* pServEnt ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = (struct dataStdCRT*) pthread_getspecific (keyStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->pServEntThreadData == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    pDataStdCRT->pServEntThreadData = new struct servent_thread_data ;
                }
                catch (CTxalloc&)
                {
                    pDataStdCRT->pServEntThreadData = NULL ;
                }
                if (pDataStdCRT->pServEntThreadData)
                {
                    memset (pDataStdCRT->pServEntThreadData, 0, sizeof (struct servent_thread_data) ) ;
                }
            }
            if (pDataStdCRT->pServEntThreadData)
            {
                if (getservbyname_r (zName, zProto, &pDataStdCRT->pServEntThreadData->servEnt, &pDataStdCRT->pServEntThreadData->servEntData) == -1)
                {   // Error from getservbyname_r ().
                    pServEnt = NULL ;
                }
                else
                {
                    pServEnt = &pDataStdCRT->pServEntThreadData->servEnt ;
                }
            }
            else
            {   // Error from pDataStdCRT->pServEntThreadData.
                pServEnt = NULL ;
            }
        }
        else
        {   // Error from pDataStdCRT.
            pServEnt = NULL ;
        }
    }
    else
    {   // Error from pthread_once ().
        pServEnt = NULL ;
    }

    return pServEnt ;
#  endif    // AIX
#  ifdef    HPUX
    struct servent* pServEnt ;
    if (pthread_once (&onceStdCRT, initRoutineStdCRT) != -1)
    {
        struct dataStdCRT*  pDataStdCRT = NULL ;
        pthread_getspecific (keyStdCRT, (pthread_addr_t*) &pDataStdCRT) ;
        if (pDataStdCRT == NULL)
        {
            // Allocate some memory for this thread.
            try
            {
                pDataStdCRT = (struct dataStdCRT*) new struct dataStdCRT ;
            }
            catch (CTxalloc&)
            {
                pDataStdCRT = NULL ;
            }
            if (pDataStdCRT)
            {
                if (pthread_setspecific (keyStdCRT, pDataStdCRT) != -1)
                {
                    memset (pDataStdCRT, 0, sizeof (struct dataStdCRT) ) ;
                }
                else
                {   // Error from pthread_setspecific ().
                    delete pDataStdCRT ;
                    pDataStdCRT = NULL ;
                }
            }
        }
        if (pDataStdCRT)
        {
            if (pDataStdCRT->pServEntThreadData == NULL)
            {   // Allocate some memory for this thread.
                try
                {
                    pDataStdCRT->pServEntThreadData = new struct servent_thread_data ;
                }
                catch (CTxalloc&)
                {
                    pDataStdCRT->pServEntThreadData = NULL ;
                }
                if (pDataStdCRT->pServEntThreadData)
                {
                    memset (pDataStdCRT->pServEntThreadData, 0, sizeof (struct servent_thread_data) ) ;
                }
            }
            if (pDataStdCRT->pServEntThreadData)
            {
                if (getservbyname_r (zName, zProto, &pDataStdCRT->pServEntThreadData->servEnt, &pDataStdCRT->pServEntThreadData->servEntData) == -1)
                {   // Error from getservbyname_r ().
                    pServEnt = NULL ;
                }
                else
                {
                    pServEnt = &pDataStdCRT->pServEntThreadData->servEnt ;
                }
            }
            else
            {   // Error from pDataStdCRT->pServEntThreadData.
                pServEnt = NULL ;
            }
        }
        else
        {   // Error from pDataStdCRT.
            pServEnt = NULL ;
        }
    }
    else
    {   // Error from pthread_once ().
        pServEnt = NULL ;
    }

    return pServEnt ;
#  endif    // HPUX
#else
    return ::getservbyname (zName, zProto) ;
#endif  // RW_MULTI_THREAD
} // getservbyname

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// End of Class CTMTSafe //
///////////////////////////////////////////////////////////////////////////////
