/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
/*===========================================================================*/
/*									     */
/*			  **** C O M M O N ****   			     */
/*									     */
/*===========================================================================*/
/*									     */
/*			Module Name: PSL_GET_NODENAME_UNIX          	     */
/*			============					     */
/*									     */
/*  Description:							     */
/*  ============							     */
/*									     */
/*	Routine to retrieve the current nodename on a UNIX system.           */
/*									     */
/*									     */
/*===========================================================================*/
/*
 *  PSL Macros.
 */
#include <psl_macros.h>

/*
 *  UNIX-Specific Module.
 */
#ifdef UNIX

/*
 *  Standard ANSI header files.
 */
#include <string.h>
#include <sys/utsname.h>

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>			/* PSL Routines */

/*
 *  Structure Definitions.
 */
#include <shrmsg.h>			/* PSL Messages */


/************************************************************************/
/*									*/
/*	PSL_GET_NODENAME - Retrieve current nodename          		*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	    iret = PSL_GET_NODENAME (diag_ptr, nodename_ptr, max_size)	*/
/*									*/
/*	Where:								*/
/*									*/
/*	    iret         = Unsigned Long PSL Routine Return Status	*/
/*	    diag_ptr     = Pointer to Diagnostics Stack			*/
/*	    nodename_ptr = Pointer to nodename (returned)		*/
/*	    max_size     = Size of Output Buffer           		*/
/*									*/
/*	Return the current nodename.                              	*/
/*									*/
/************************************************************************/

unsigned long	PSL_GET_NODENAME (struct diagstk *diag_ptr,
				  char *nodename_ptr,
				  unsigned long max_size)
{
/*
 *  Local data.
 */
unsigned long	 iret;			/* Routine Return Status */
unsigned long	 start_diag;		/* Start of Diagnostics Stack */
	 long	 status;		/* Status from routine */
struct	 utsname nde_utsname;		/* Machine Information Structure */

/*
 *  Maintain the diagnostics stack.
 */
    start_diag = PSL_ADD_STATDIAG (diag_ptr, "During PSL_GET_NODENAME");

/*
 *  Get the current nodename.
 */
    status = uname (&nde_utsname);
    if (status < 0)
    {

/*
 *	Setup the error.
 */
	iret = PSL_BADPARAM;

	PSL_PUTDIAG (diag_ptr, 2, PSL_SYSERVERR,
		     "uname - PSL_GET_NODENAME)", iret);
    }
    else
    {

/*
 *	Copy the nodename.
 */
	if ((strlen (nde_utsname.nodename) + 1) > max_size)
	{
	    (void) memcpy (nodename_ptr, nde_utsname.nodename, max_size - 1);
	    *(nodename_ptr + max_size - 1) = '\0';		      
	}
	else
	{
	    (void) strcpy (nodename_ptr, nde_utsname.nodename);
	}

/*
 *	Unwind the diagnostics stack.
 */
	PSL_REMOVE_DIAGNOSTICS (diag_ptr, start_diag);

/*
 *	Return success.
 */
	iret = PSL_NORMAL;
    }

/*
 *  Return with status.
 */
    return iret;
}

#endif /* UNIX */
