/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TRecCreator.h
#ifndef _CYR_TRECCREATOR_H
#define _CYR_TRECCREATOR_H
/**
 * Class definition for rule defining a Transaction Record
 **/

#include <string>
#include <map>
#ifdef unix
#elif defined (__VMS)
#elif defined (WIN32)
#pragma warning(disable:4786)
using namespace std;
class TRecCreator;
ostream& operator<< ( ostream& os, const TRecCreator& tR);
#else //unknown
#error Unknown Platform
#endif // unknown

// local includes
#include "TRecord.h"

typedef vector<TParamRule*> prulep_vect_t;

class TRecCreator
{
protected:
    string _desc;
    TParamRule* _typRuleP;
    TOptions _options;
    prulep_vect_t _params;
public:
    // constructors & destructor
    TRecCreator();
    virtual ~TRecCreator();

    // accessors
    virtual TRecord* create() const = 0;
    string option(const string& key) const;
    bool hasAttr(const string& attr) const;
    const TParamRule* getParamRule(const string& ruleName) const;
    // mutators
    void clearDefinition();
    void addTypRule(TParamRule* prP);
    void addPrmRule(TParamRule* prP);
    void addOption(const string& text);
    void addDefVal(const string& text);
    void setAttr(const string& attr);
    // Friend operator
    friend ostream& operator<< (ostream& os, const TRecCreator& tR);
private:
    TRecCreator(const TRecCreator&);
    operator=(const TRecCreator&);
}
; // TRecCreator

// type defintion for container to hold these record creators
typedef map<AtvfKey, const TRecCreator*, less<AtvfKey> > trec_crtrs_t;

#endif // _CYR_TRECCREATOR_H
