/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _TESTINITIATOR_H_
#define _TESTINITIATOR_H_

///////////////////////////////////////////////////////////////////////////////
// @doc     TestInitiator
// @module  TestInitiator | Class for initiating the execution of a test
// Any new test Manager that is created requires the following
// 1. Project of Test to be run
// 2. Name of Test to be run
// 3. Username to log onto repository
// 4. Password to log on to repository
// 5. Location(node) of repository
// 6. The node on which the test is to be run
// This is for all Test Managers and allows the connection to a repository and the retrieval
// of test details and storage of Test data
// additionally if a Test Manager is to kick off a sub test then the various parents of the new test must be passed down
// 1. The Parent TestManager and from this the Parent TestManagerStatus
// 2. The Parent TestLevel that the First TestLevel belongs to
// 3. The Name of the Parent TestRuns that subsequent TestRuns must be stored under.
// It must be assumed that Tests and sub tests all run within the same project
//
///////////////////////////////////////////////////////////////////////////////

#include <OSTAORB/OSTAORB.h>
#include <string>
#include "TestManager.h"
#include "CECSpawnObject.h"
#ifndef CYR_NO_STD_NAMESPACE
using std::string;
#endif
class TestInitiator
{
public:
    TestInitiator(const string& ProjectName,const string& TestName,const CECNodeDefinition& node,CORBAServer* CS=NULL);
    virtual ~TestInitiator();
    virtual void SetRepositoryInformation(const string &ReposNode,const string &user,const string &pass);
    virtual long CreateTestManager(TestLevel_ptr Parent=TestLevel::_nil(),string TestRunString="");
    virtual long ExecuteTest();
    virtual TestManager_ptr GetTestManager();

private:
    string  m_ProjectName;
    string  m_TestName;
    long    m_PID;
    string  m_ReposNode;
    string  m_user;
    string  m_password;
    string m_TestRunString;
    TestManager_var m_TestManager;
    TestLevel_var   m_Parent;
    CORBAServer*    _Corbaptr;
    CECNodeDefinition   m_NodeDefinition;

};
#endif
