/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _TMTHREAD_H_
#define _TMTHREAD_H_
#include "TestManager.h"
#include "TestLevel_i.h"
#include "mdbthread.h"
#include "ScriptCache.h"
#include "TMVarFactory.h"
#include "tofprmdef.h"
class TMthreadcontrol;
class TMThread:public Cmdbthread
{
public:
    TMThread(TMthreadcontrol* threadcontrol,string filename);
    virtual ~TMThread();
    // methods related to functionality in TManagerRcrd's
    virtual int TestExecution(string name,string node);
    virtual int ThreadExecution(string threadname,string scriptname,string nodename,unsigned int number_of_threads,unsigned int interval_btw_threads,unsigned int delay_before_first_thread);
    virtual int ExitCommand(bool keepalive,const string& retvar);
    virtual bool isTestRunning(const string& testname);
    virtual bool isThreadRunning(const string& testname);
    virtual void    StartScript(const string& scriptname);
    virtual void    EndScript();
    //////////////////////////////////////////////////////
    virtual void SetThreadInfo(TestLevel_ptr TL,TestManager_ptr TMS=TestManager::_nil());
    virtual void Initialise();
    virtual void CleanUp();
    virtual void SetNoteText(const string& mesg);
    // overridden base class methods
    virtual int  declarations(trecset_t _decls);
    virtual string get_test_name(void);
    virtual int TCterminate(int result,string desc);

    typedef map<string,ThreadDef> ThreadMap_t;
    static  ThreadMap_t ThreadMap;//map of global threads read in and created
    ThreadMap_t localThreadMap;//map of all threads definition required by this Context
    static  TScriptRule* thrule;//Rule for global Threads
    static  ScriptCache* ThreadCache;//Cache of global threads


protected:
    virtual void LogStartMessage();
    virtual void LogEndMessage();
    virtual void MakeTestLevel(TestManager_ptr TM,string scriptname,string Node,TestLevel_ptr pTL=TestLevel::_nil());
    virtual void MakeTestLevel(TestLevel_ptr TL,string scriptname,string Node);

    TestManager_ptr     m_ParentTestManager;
    TestLevel_ptr       m_ParentTestLevel;
    TestLevel_var       m_TestLevel;
    TestLevelImpl*      m_TestLevelImpl;
    TMthreadcontrol*        m_tmthreadcontrol;

private:
    static CTMutex ThrAccessMtx;//for global data referenced in prepare method

}
;

#endif
