/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     TestManager
///////////////////////////////////////////////////////////////////////////////
#ifndef _TESTMANAGER_I_H_
#define _TESTMANAGER_I_H_
#include <TestManager.h>
#include <TMVarFactory_i.h>
#include <OSTAORB/OSTAORB.h>
#include "TMthreadcontrol.h"
#include <string>
#include <list>
#ifndef CYR_NO_STD_NAMESPACE
using std::string;
using std::list;
#endif

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
extern CTSemaphore blocking_semaphore;
class TestManagerImpl : public POA_TestManager ,public PortableServer::RefCountServantBase  
{
public:
    TestManagerImpl (CORBAServer* pserver,string &regname,string node,int pid,string exename,TScriptRule* rule);
    virtual ~TestManagerImpl();

    virtual CORBA::Long  Initialise(const char*  node, const char*  user, const char*  password, const char*  TestRunString);
    virtual void Stop();
    virtual void Kill();
    virtual char*  GetNode();
    virtual char*  GetNodeName();

    virtual CORBA::Long  ExecuteThread(const char*  ProjectName, const char*  TestName, TestLevel_ptr & parent);
    virtual void AddSubTestManager(TestManager_ptr & subtest);
    virtual void RemoveSubTestManager(TestManager_ptr & subtest);
    virtual CORBA::ULong  GetTestManagerList(TestManagerList & TMlist);

    virtual CORBA::Object_ptr  GetTestRunAsObject();

    virtual char*  Name();
    virtual CORBA::Long  GetPID();
    virtual CORBA::Long  GetTotalTestCasePasses();
    virtual CORBA::Long  GetTotalTestCaseFails();
    virtual CORBA::Long  GetNoThreads();
    virtual void SetTestLevel(TestLevel_ptr & TL);
    virtual TestLevel_ptr  GetTestLevel();
    virtual char*  GetUser();
    virtual CORBA::ULong  StartTime();
    void SetToCloseDown();//Set flag to say closing down is imminent
    bool WaitForCompletion();//wait for completion of threads
private:
    void CloseDown();//clear up member data
    typedef list<TestManager_var>  TestManagerList_t;

    CORBAServer*        m_corba;
    string          m_RegisterName;
    TMthreadcontrol*        m_threadcontrol;
    string          m_TestRunString;
    string          m_nodename;
    int             m_pid;
    string          m_exename;
    string          m_User;
    TMVarFactoryImpl*       m_TMVarFactory;
    TestManagerList_t       m_TestManagerList;
    unsigned long       m_StartTime;
    TestLevel_var       m_TestLevel;
    bool            m_closing_down;//set when this object believes it is closing down
}
;
#endif
