/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/****************************************************************************
 * @doc     tofprmdef.h
 * @module  tofprmdef.h | This 'C' format header file contains symbol
 *      definitions for items within a TOF file. Its purpose is to
 *      isolate in a single location the definition of TOF items
 *      (e.g. parameter codes and symbols used within expressions).
 *      These symbols should be used by code that generates or
 *      processes a TOF file.
 *
 ****************************************************************************/

#ifndef tofprmdef_h
#define tofprmdef_h

/* Variable Option Values (32 bit field). Bits 28-31 currently Unused. */
/* NOTE: Many of these bits are mutually exclusive. */

/* Datatypes */
#define VTMASK      0x0000000F
#define VTCHAR      1           /* Character Type */
#define VTINT       (1<<1)      /* Integer Type */
#define VTSPARE1    (1<<2)      /* Spare Type Bit */
#define VTSPARE2    (1<<3)      /* Spare Type Bit */
/* Data Source */
#define VSMASK      0x000000F0
#define VSLIST      (1<<4)      /* Source is Value List */
#define VSFILE      (1<<5)      /* Source is File */
#define VSRANGE     (1<<6)      /* Source is Range */
#define VSNONE      (1<<7)      /* None of the above (internal use)
/* Selection Order */
#define VOMASK      0x00000F00
#define VOINVMASK   0xFFFFF0FF
#define VONEXT      (1<<8)      /* Next Value from Source */
#define VORAND      (1<<9)      /* Random Value from Source */
#define VOREPRAND   (1<<10)     /* Repeatable Random from Source */
#define VOCUR       (1<<11)     /* Current Value From Source */
/* Variable Scope Definitions */
#define VCMASK      0x00F0F000
#define VCLOCAL     (1<<12)     /* Local Scope */
#define VCSCRIPT    (1<<13)     /* Script Scope */
#define VCGLOBAL    (1<<14)     /* Global Scope */
#define VCPRIVATE   (1<<15)     /* Private Scope */
/* External Datafile Open Modes */
#define VFMASK      (unsigned long)0x000F0000
#define VFSHARE     (unsigned long)0x00010000   /* Share Access */
#define VFEXIST     (unsigned long)0x00020000   /* Access Existing */
#define VFREWIND    (unsigned long)0x00040000   /* Access and Rewind Existing */
#define VFSPARE     (unsigned long)0x00080000   /* Spare File Mode Bit */
/* Extended Variable Scope Definitions */
#define VCTHREAD    (unsigned long)0x00100000   /* Thread Scope */
#define VCTEMP      (unsigned long)0x00200000   /* Temporary scope */
/* dg 13apr99 more scope definitions */
#define VCTESTWIDE  (1<<22)         /* Test-wide scope */
#define VCREPOSITORY    (1<<23)     /* Repository variable */
/* dg 26apr99 added definitions */
/* Extended selection Order. This is how the variables are to be defined */
#define VONOAUTONEXT (unsigned long) 0x01000000 /* No auto-next for value lists/ranges */
#define VOSPARE     (unsigned long) 0x02000000  /* Spare selection order bit */
#define VONOECHO    (unsigned long) 0x04000000  /* No echo value */
#define VOUPPERCASE (unsigned long) 0x08000000  /* Uppercase value */

/* Preferred delimiter for character string in value list */
#define STRING_VAL_DELIM            "\""
#define STRING_VAL_DELIMC           '"'
/* Alternative delimiter for character string in value list */
#define ALT_STRING_VAL_DELIM        "'"
#define ALT_STRING_VAL_DELIMC       '\''
/* Last ditch replacement for delimiter within a string value. */
/* Hex ASCII representation of double quote */
#define STRING_VAL_DELIM_ESC        "~<22>"
/* Last ditch replacement for alt delimiter within a string value. */
/* Hex ASCII representation of single quote */
#define ALT_STRING_VAL_DELIM_ESC    "~<27>"

/* Value List Delimiters */
#define SLISTC      "("
#define ELISTC      ")"
/* Range Separator String */
#define RGSEP       " - "
/* Continuation string */
#define VLCONT      " &"
/* Line terminator */
#define ENDL        "\n"
#define ENDLC       '\n'

/* Transaction Object Variable Substitution Identifier and Delimiter
 * (MUST BE SINGLE CHARACTER).
 */
#define VNIDELIM    "%"
#define VNIDELIMC   '%'

/* To indicate get current value ie. %$var% */
#define VNIDELIMO   "$"
#define VNIDELIMOC  '$'

/* Substitute Variable Name in Parameter Text Codes. */

/* Operation cancelled */
#define VTSNOSUB    0
/* Substitute string for get new value (%varname%) */
#define VTSNEW      1
/* Substitute string for use current value (%$varname%) */
#define VTSOLD      2

/* Special "index" codes for get variable value */
#define VAL_GET_RANDOM  -1

/* Set value return codes
 * Done ok, nothing more to do
 */
#define VAR_SET_DONE    0
/* Done ok but variable not private, also need to
 * set via ext link.
 */
#define VAR_SET_EXT     1
/* Failed, undefined variable or invalid index */
#define VAR_SET_FAIL    -1

/* Current TOF version ('n' in first line of TOF header record - tof.n) */
#define TOF_CURVERS     "7"

/* Log message destination codes */
#define TOF_LOGDST_AUDIT    0
#define TOF_LOGDST_HISTORY  1
#define TOF_LOGDST_REPORT   2
#define TOF_LOGDST_NOTE 3
#define TOF_LOGDST_TRACE    4
#define TOF_LOGDST_CENTRAL  5

/* Test case termination (Tcterm) result codes */
#define TOF_TCTRES_FAIL     0
#define TOF_TCTRES_PASS     1

/* Default LTXT delimiter */
/* This string is used by SCL to delimit ltxt fields. The string should */
/* be short (for efficiency) and obscure enough to be highly unlikely to */
/* appear legitimatly within an ltxt field. */
#define LTXT_DELIMITER  "#%~@"

/* Variable trailing space option specifiers */
/*** #define TRAILS_ONE ">"
#define TRAILS_MANY ">>"
#define TRAILS_NONE "" ***/

/* Operator symbols */
#define TOF_OPR_EQ      "="
#define TOF_OPR_NE          "!="
#define TOF_OPR_LE          "<="
#define TOF_OPR_GE      ">="
#define TOF_OPR_LT      "<"
#define TOF_OPR_GT      ">"
#define TOF_OPR_CONTAINS    "^"
#define TOF_OPR_NOT_CONTAINS    "<^>"
#define TOF_OPR_ADD     "+"
#define TOF_OPR_SUBTRACT    "-"
#define TOF_OPR_AND     "&"
#define TOF_OPR_OR      "|"
#define TOF_OPR_MULTIPLY    "*"
#define TOF_OPR_DIVIDE      "/"
#define TOF_OPR_MODULO      "|"
#define TOF_OPR_MATCHES     "MATCHES"
#define TOF_OPR_NOT_MATCHES "NOT_MATCHES"
#define TOF_OPR_NOT     "!"
#define TOF_OPR_SEXPR       "("     /* Start expression */
#define TOF_OPR_EEXPR       ")"     /* End expression */

#define TOF_OPR_RNGSEP      ":"     /* Range bound separator character */
#define TOF_OPR_VALSEP      ","     /* Value separator */

/* Array reference symbols */
#define TOF_SIDXDELIM       "["     /* Start array reference */
#define TOF_EIDXDELIM       "]"     /* End array reference */
#define TOF_IDXSEP      ","     /* Array index separator */

/* Function reference symbols */
#define TOF_SPRMDELIM       "("     /* Start parameter list */
#define TOF_EPRMDELIM       ")"     /* End parameter list */
#define TOF_PRMSEP      ","     /* Parameter separator */

/* Operand modifier symbols, e.g. for HTTP_STATUS */
#define TOF_SMODDELIM       "["     /* Start operand modifier */
#define TOF_EMODDELIM       "]"     /* End operand modifier */

/* TOF option strings */
#define TOF_OPTSTR_CASEBLIND    "CASE_BLIND"    /* , CASE_BLIND */

/* Wait type codes */
#define TOF_WATTYP_TEST     0
#define TOF_WATTYP_SCRIPT   1

/* Exit option mask codes */
#define TOF_EXTOPT_KEEPALIVE    1       /* KEEPALIVE */

/* Disconnect option mask codes */
#define TOF_DCNOPT_CANCEL   1       /* WITH CANCEL */

/* Mutex scope codes */
#define TOF_MTXSCP_LOCAL    0
#define TOF_MTXSCP_TESTWIDE 1

/* Semaphore scope codes */
#define TOF_SEMSCP_LOCAL    0
#define TOF_SEMSCP_TESTWIDE 1

/* File access type codes */
#define TOF_FILACC_INPUT    1
#define TOF_FILACC_OUTPUT   2
#define TOF_FILACC_APPEND   4

/* File option mask codes */
#define TOF_FILOPT_BINFIL   1
#define TOF_FILOPT_COMSEP   2
#define TOF_FILOPT_SHARED   4

/* File control (Fcntl) operation codes */
#define TOF_FCNOP_CLOSE     0
#define TOF_FCNOP_REWIND    1
#define TOF_FCNOP_MAP       2

/* Pipe scope codes */
#define TOF_PIPSCP_LOCAL    0
#define TOF_PIPSCP_TESTWIDE 1

/* Pipe type codes */
#define TOF_PIPTYP_ASYNCHRONOUS 0
#define TOF_PIPTYP_SYNCHRONOUS  1

/* Pipe access type codes */
#define TOF_PIPACC_INPUT    0
#define TOF_PIPACC_OUTPUT   1

/* Pipe definition option mask codes */
#define TOF_PIPOPT_SYNCHRONOUS  1

/* TOF ID codes (as appear in the first line of a TOF file, e.g. tof.2 */
/* These codes identify what sort of TOF file a particular TOF file is */
#define TOF_ID_SYBASE       1
#define TOF_ID_SQLSERVER    3
#define TOF_ID_ORACLE       2
#define TOF_ID_ORACLE8      4
#define TOF_ID_ORACLEAPP    5
#define TOF_ID_TUXEDO       6
#define TOF_ID_SAP      7
#define TOF_ID_TCPIP        8
#define TOF_ID_COMMAND      9
#define TOF_ID_HTP      10
#define TOF_ID_CHARCELL     11
#define TOF_ID_TEST     12
#define TOF_ID_THREAD       13
#define TOF_ID_WEB_DT      14
#define TOF_ID_NTP          15
#define TOF_ID_SMP          16
#define TOF_ID_SYM          17

/* Enable and Disable target codes */
#define TOF_ENDTGT_EXECUTION    0
#define TOF_ENDTGT_ONERROR  1
#define TOF_ENDTGT_WHEN     2
#define TOF_ENDTGT_TYPEAHEAD    3
#define TOF_ENDTGT_TRACING  4
#define TOF_ENDTGT_RECORDING    5
#define TOF_ENDTGT_SCRMODE  6

/* Load data codes for generic LOAD command */
#define TOF_LODDAT_ACTTHR   0
#define TOF_LODDAT_DATE     1
#define TOF_LODDAT_DEFDIR   2
#define TOF_LODDAT_ERROR    3
#define TOF_LODDAT_NODNAM   4
#define TOF_LODDAT_SCRNAM   5
#define TOF_LODDAT_TSTNAM   6
#define TOF_LODDAT_THRNAM   7
#define TOF_LODDAT_TIME     8
#define TOF_LODDAT_ACCOUNT  9
#define TOF_LODDAT_DEVICE   10
#define TOF_LODDAT_TIMER    11

/* Local Load data codes - for non-generic LOAD commands */
/* These MUST be the same as the values defined in the ODL */
#define TOF_LLDDAT_RSPHDR   0   /* HTTP - LOAD RESPONSE_INFO HEADER */
#define TOF_LLDDAT_RSPBDY   1   /* HTTP - LOAD RESPONSE_INFO BODY */

/* Genvar mode codes */
#define TOF_GVRMOD_SEQUENTIAL   0
#define TOF_GVRMOD_RANDOM   1
#define TOF_GVRMOD_RESET    2

/* Abort target codes */
#define TOF_ABTTGT_THREAD   0
#define TOF_ABTTGT_TEST     1

/* TOF Convert option codes:                */
/* TOF_CNVOPT_SIGN  Clear   SIGNED (default)    */
/*          Set UNSIGNED        */
/* TOF_CNVOPT_LEAD  Clear   LEADING ZEROS (default) */
/*              Set LEADING SPACES      */
/* TOF_CNVOPT_JUST  Clear   LEFT JUSTIFY (default)  */
/*          Set RIGHT JUSTIFY       */
#define TOF_CNVOPT_SIGN     0X1
#define TOF_CNVOPT_LEAD     0X2
#define TOF_CNVOPT_JUST     0X4

/* TOF When option codes:                   */
/* TOF_WHNOPT_NOCANCEL  Clear   CANCEL (default)    */
/*          Set NOCANCEL        */
#define TOF_WHNOPT_NOCANCEL 0X1

/* TOF SCL Function ID's */
#define TOF_FNCID_LOCATE    1
#define TOF_FNCID_EXTRACT   2
static char *TOF_FNCNAM[] = {
    "Illegal function",
    "~LOCATE",
    "~EXTRACT"};

/* TOF General state codes */
#define TOF_GENSTA_DISABLED 0
#define TOF_GENSTA_ENABLED  1

#define TOF_GENSTA_FALSE    0
#define TOF_GENSTA_TRUE     1

/* TOF Request option mask codes */
#define TOF_REQOPT_BINBOD   1
#define TOF_REQOPT_PRIMARY  2
#define TOF_REQOPT_CERTFIL  4

/* TOF Request types. */
/* These MUST be the same as the values defined in the ODL */
#define TOF_REQTYP_GET      0
#define TOF_REQTYP_HEAD     1
#define TOF_REQTYP_POST     2

/* TOF Thread baudrate codes */
#define TOF_THRBDR_50       1
#define TOF_THRBDR_75       2
#define TOF_THRBDR_110      3
#define TOF_THRBDR_134      4
#define TOF_THRBDR_150      5
#define TOF_THRBDR_300      6
#define TOF_THRBDR_600      7
#define TOF_THRBDR_1200     8
#define TOF_THRBDR_1800     9
#define TOF_THRBDR_2000     10
#define TOF_THRBDR_2400     11
#define TOF_THRBDR_3600     12
#define TOF_THRBDR_4800     13
#define TOF_THRBDR_7200     14
#define TOF_THRBDR_9600     15
#define TOF_THRBDR_19200    16
#define TOF_THRBDR_38400    17
#define TOF_THRBDR_57600    18
#define TOF_THRBDR_76800    19
#define TOF_THRBDR_115200   20
#define TOF_THRBDR_MAXIMUM  99

/* TOF Thread inptiming (input timing type) codes */
#define TOF_THRITM_FINE     1
#define TOF_THRITM_COARSE   2

/* TOF Thread login (type) codes */
#define TOF_THRLOG_AUTO     1
#define TOF_THRLOG_SCRIPT   2
#define TOF_THRLOG_NO       3
#define TOF_THRLOG_AUTOIM   4
#define TOF_THRLOG_AUTODELAY    5
#define TOF_THRLOG_TELNET   6
#define TOF_THRLOG_RLOGIN   7
#define TOF_THRLOG_RAW      8

/* TOF Thread recordatt (record attribute) codes */
#define TOF_THRRAT_INPUT    0X1
#define TOF_THRRAT_OUTPUT   0X2
#define TOF_THRRAT_SERVICE  0X4
#define TOF_THRRAT_RESPONSE 0X8
#define TOF_THRRAT_TRANS    0X10
#define TOF_THRRAT_COMMAND  0X20

/* TOF Thread threadid codes */
#define TOF_THRTID_VT52     1
#define TOF_THRTID_VT100    2
#define TOF_THRTID_VT101    3
#define TOF_THRTID_VT102    4
#define TOF_THRTID_VT125    5
#define TOF_THRTID_VT220    6
#define TOF_THRTID_VT240    7
#define TOF_THRTID_VT320    8
#define TOF_THRTID_VT330    9
#define TOF_THRTID_VT340    10
#define TOF_THRTID_VT420    11
#define TOF_THRTID_WYSE60   12
#define TOF_THRTID_HP       13

/* TOF Thread threadtype codes */
#define TOF_THRTTP_PSEUDO   1
#define TOF_THRTTP_PHYS     2
#define TOF_THRTTP_REMLAT   3
#define TOF_THRTTP_DECTERM  4
#define TOF_THRTTP_VWSTERM  5
#define TOF_THRTTP_REMTCPIP 6
#define TOF_THRTTP_COMMAND  7
#define TOF_THRTTP_EMULATOR 8

#endif
