/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Commander.h"
#include "RegFunctions.h"
#include "OstaDefs.h"
#include "CommanderView.h"
#include "MainFrm.h"
#include "MonitorSelect.h"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NEW_MENU_ID_START 0x1000
#define NEW_MENU_ID_END 0x1100

static CString strHelpFile = "OpenSTACommander.chm";

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_TOOLS_MONITOR, OnTestsMonitor)
	ON_COMMAND(ID_TOOLS_REPOSITORYPATH, OnToolsRepositorypath)
	ON_COMMAND(ID_TOOLS_TRACESETTINGS, OnToolsTracesettings)
	ON_COMMAND(ID_HELP_CONTENTS, OnHelpContents)
	ON_COMMAND(ID_HELP_INDEX, OnHelpIndex)
	ON_COMMAND(ID_HELP_SEARCH, OnHelpSearch)
	ON_WM_HELPINFO()
	ON_COMMAND(ID_VIEW_REPOSITORY, OnViewRepository)
	ON_UPDATE_COMMAND_UI(ID_VIEW_REPOSITORY, OnUpdateViewRepository)
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(NEW_MENU_ID_START,NEW_MENU_ID_END,OnCustomMenuCommand)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_nMenuItemsAdded = 0;
	m_pCurrentDoc=NULL;

	m_strHelpFilePath = getenv("OPENSTA_HOME");
	m_strHelpFilePath+="BaseUI\\"+strHelpFile;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
/*	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}
*/
	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	if (!m_wndWorkspace.Create(_T("Workspace"), this, 123))
	{
	    TRACE0("Failed to create workspace bar\n");
	    return -1;
	    // fail to create
	}
	m_wndWorkspace.SetBarStyle(m_wndWorkspace.GetBarStyle() |
	    CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
//	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
//	DockControlBar(&m_wndToolBar);
	m_wndWorkspace.EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndWorkspace, AFX_IDW_DOCKBAR_LEFT);

	CCreateContext *pContext = (CCreateContext*)lpCreateStruct->lpCreateParams;
	CDocTemplate *pDocTemplate = pContext->m_pNewDocTemplate;
	m_MenuIP.Attach(pDocTemplate->m_hMenuInPlace);
	m_pCurrentDoc = pContext->m_pCurrentDoc;

	LoadTestPlugins();
	LoadScriptPlugins();
	LoadProfilePlugins();

	//Generate the repository if it doesn't exist
	CreateRepository( GetRepositoryPath() );

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::LoadTestPlugins()
{
	//Load information about test plugins
	int nNumTests =	LoadPluginType(enTest);
	
	//Modify standard and IP menu to include tests
	CMenu *pMainMenu = GetMenu();
	CMenu *pFileMenu = pMainMenu->GetSubMenu(0);
	CMenu *pIPFileMenu = m_MenuIP.GetSubMenu(0);

	if(nNumTests)
	{
		CMenu *pTestMenu = pFileMenu->GetSubMenu(enNewTestMenuPos);
		CMenu *pIPTestMenu = pIPFileMenu->GetSubMenu(enNewTestMenuPos);

		POSITION pos = m_listPlugins.GetHeadPosition();
		while(pos)
		{
			PluginInfo& PI = m_listPlugins.GetNext(pos);
			if(PI.nType != enTest)
				continue;
			
			//Add plugin to new menu
			PI.uiNewCommandId = NEW_MENU_ID_START + m_nMenuItemsAdded;
			
			pTestMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
			pTestMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
			
			pIPTestMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
			pIPTestMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
			m_nMenuItemsAdded++;
		}
	}
	else
	{
		pFileMenu->RemoveMenu(enNewTestMenuPos,MF_BYPOSITION);
		pIPFileMenu->RemoveMenu(enNewTestMenuPos,MF_BYPOSITION);
	}
}

void CMainFrame::LoadScriptPlugins()
{
	//Load information about script plugins
	int nNumScripts =	LoadPluginType(enScript);
	
	//Modify standard and IP menu to include tests
	CMenu *pMainMenu = GetMenu();
	CMenu *pFileMenu = pMainMenu->GetSubMenu(0);
	CMenu *pIPFileMenu = m_MenuIP.GetSubMenu(0);

	if(nNumScripts)
	{
		CMenu *pScriptMenu = pFileMenu->GetSubMenu(enNewScriptMenuPos);
		CMenu *pIPScriptMenu = pIPFileMenu->GetSubMenu(enNewScriptMenuPos);

		POSITION pos = m_listPlugins.GetHeadPosition();
		while(pos)
		{
			PluginInfo& PI = m_listPlugins.GetNext(pos);
			if(PI.nType != enScript)
				continue;
			
			//Add plugin to new menu
			PI.uiNewCommandId = NEW_MENU_ID_START + m_nMenuItemsAdded;
			
			pScriptMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
			pScriptMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
			
			pIPScriptMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
			pIPScriptMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
			m_nMenuItemsAdded++;
		}
	}
	else
	{
		pFileMenu->RemoveMenu(enNewScriptMenuPos,MF_BYPOSITION);
		pIPFileMenu->RemoveMenu(enNewScriptMenuPos,MF_BYPOSITION);
	}
}

void CMainFrame::LoadProfilePlugins()
{
	//Load information about Profile plugins
	int nNumProfiles =	LoadPluginType(enProfile);
		
	//Modify standard and IP menu to include tests
	CMenu *pMainMenu = GetMenu();
	CMenu *pFileMenu = pMainMenu->GetSubMenu(0);
	CMenu *pIPFileMenu = m_MenuIP.GetSubMenu(0);

	if(nNumProfiles)
	{
		CMenu *pProfileMenu = pFileMenu->GetSubMenu(enNewProfileMenuPos);
		CMenu *pIPProfileMenu = pIPFileMenu->GetSubMenu(enNewProfileMenuPos);

		POSITION pos = m_listPlugins.GetHeadPosition();
		while(pos)
		{
			PluginInfo& PI = m_listPlugins.GetNext(pos);
			if(PI.nType != enProfile)
				continue;
			
			//Add plugin to new menu
			PI.uiNewCommandId = NEW_MENU_ID_START + m_nMenuItemsAdded;
			
			pProfileMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
			pProfileMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
			
			pIPProfileMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
			pIPProfileMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
			m_nMenuItemsAdded++;
		}
	}
	else
	{
		pFileMenu->RemoveMenu(enNewProfileMenuPos,MF_BYPOSITION);
		pIPFileMenu->RemoveMenu(enNewProfileMenuPos,MF_BYPOSITION);
	}
}

int CMainFrame::LoadPluginType(PluginType nType)
{
	int nCount = 0;
	CString strRegSubKey("SOFTWARE\\OpenSTA\\PlugIns\\");

	switch(nType)
	{
	case enScript:
		strRegSubKey += strScriptType;
	break;
	case enProfile:
		strRegSubKey += strProfileType;
	break;
	case enTest:
	strRegSubKey += strTestType;
	break;
	default:
		ASSERT(FALSE);
		return nCount;
	break;
	}

	HKEY hKey;
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,strRegSubKey,0,KEY_READ,&hKey)!=ERROR_SUCCESS)
		return nCount;

	DWORD dwIndex=0;
	DWORD dwNameSize=64;
	char szName[64];
	while(RegEnumKeyEx(hKey,dwIndex,szName,&dwNameSize,0,NULL,NULL,NULL)==ERROR_SUCCESS)
	{
		dwNameSize = 64;
		HKEY hKeyPlugin;
		if(RegOpenKeyEx(hKey,szName,0,KEY_READ,&hKeyPlugin) == ERROR_SUCCESS)
		{
			PluginInfo PI;
			PI.strName = szName;
			PI.nType = nType;
			PI.uiNewCommandId = 0;

			DWORD dwType;
			DWORD dwDataSize = 256;
			BYTE byData[256];
			//Get Extension for this type
			if(RegQueryValueEx(hKeyPlugin,"Extension",0,&dwType,byData,&dwDataSize)==ERROR_SUCCESS)
				PI.strExtension = (char*)byData;
			dwDataSize = 256;

			//Get Clsid
			if(RegQueryValueEx(hKeyPlugin,"CLSID",0,&dwType,byData,&dwDataSize)==ERROR_SUCCESS)
			{
				BSTR szGuid = CString((char*)byData).AllocSysString();
				CLSIDFromString(szGuid,&PI.clsid);
				SysFreeString(szGuid);
			}
			dwDataSize = 256;

			RegCloseKey(hKeyPlugin);
			m_listPlugins.AddTail(PI);
			nCount++;
		}
		dwIndex++;
	}
	return nCount;
}

void CMainFrame::OpenFile(const CString& strPath)
{
	int nDotPos = strPath.ReverseFind('.');
	if(nDotPos == -1)
		return;

	CString strExtension(strPath.Mid(nDotPos+1));

	POSITION pos = m_listPlugins.GetHeadPosition();
	while(pos)
	{
		PluginInfo &PI = m_listPlugins.GetNext(pos);
		CString strPIExts(PI.strExtension);

		char *pszExt = strtok(strPIExts.GetBuffer(0),";");
		while(pszExt)
		{
			if(strExtension.CompareNoCase(pszExt)==0)
			{
				if(PI.strName == "HTTP")		//BODGE to invoke modeller
					InvokeModeller(strPath);
				else
				{
					POSITION pos2 = m_pCurrentDoc->GetFirstViewPosition();
					if(pos2)
						((CCommanderView*)m_pCurrentDoc->GetNextView(pos2))->ShowObject(strPath,PI.clsid);
				}
				return;
			}

			pszExt = strtok(NULL,";");
		}
	}
	
	//Show unknown type
	pos = m_pCurrentDoc->GetFirstViewPosition();
	if(pos)
		((CCommanderView*)m_pCurrentDoc->GetNextView(pos))->ShowObject(strPath);
}

BOOL CMainFrame::InvokeModeller(const CString& strScript)
{
	CString strFilePath;
    if (!strScript.IsEmpty())
        strFilePath = "\"" + strScript + "\"";
	
	CString strOpenStaPath(getenv("OPENSTA_HOME"));

    CString strExePath(strOpenStaPath + "Engines\\WEB\\Modeller\\TModeller_Web.exe");
	CString strWorkDir(strOpenStaPath + "Common");

    //Start up the Http modeller
	if((int)ShellExecute(m_hWnd,NULL,strExePath,strFilePath,strWorkDir,SW_SHOWNORMAL) < 32)
	{
		AfxMessageBox("Failed to start the HTTP Modeler");
		return false;
	}

	return true;
}

void CMainFrame::OnCustomMenuCommand(UINT nID)
{
	//Create new file
	POSITION pos = m_listPlugins.GetHeadPosition();
	while(pos)
	{
		PluginInfo& PI = m_listPlugins.GetNext(pos);

		if(PI.uiNewCommandId == nID)
		{
			switch(PI.nType)
			{
			case enScript:
				m_wndWorkspace.m_ctlObjectTree.AddNewScript(PI.strExtension.SpanExcluding(";"));			return;
			case enProfile:
				m_wndWorkspace.m_ctlObjectTree.AddNewProfile(PI.strExtension.SpanExcluding(";"));
			return;
			case enTest:
				m_wndWorkspace.m_ctlObjectTree.AddNewTest();
			return;
			}
		}
	}
}


void CMainFrame::OnTestsMonitor() 
{
	CMonitorSelect dlg;
	if (dlg.DoModal() == IDOK)
	{
		OpenFile(dlg.GetTestFile());
	}
}

void CMainFrame::OnToolsRepositorypath() 
{
	char szBuffer[MAX_PATH];

	BROWSEINFO BI;
	memset(&BI,0,sizeof(BI));
	BI.hwndOwner = m_hWnd;
	BI.pszDisplayName = szBuffer;
    BI.lpszTitle = "Select the new repository path";
    BI.ulFlags = BIF_EDITBOX|BIF_VALIDATE|BIF_RETURNONLYFSDIRS|BIF_RETURNFSANCESTORS;
	
	ITEMIDLIST *pidl = SHBrowseForFolder(&BI);

	if(!SHGetPathFromIDList(pidl, szBuffer) || (szBuffer[0] == '\0'))
		return;

	CreateRepository(szBuffer);	

	m_wndWorkspace.m_ctlObjectTree.RefreshTree();
}

void CMainFrame::CreateRepository(const CString& strRespositoryPath)
{
	CString strPath(strRespositoryPath);
	strPath.TrimRight("\\");
	if(strPath.Right(10).CompareNoCase("Repository")!=0)
		strPath += "\\Repository";

	CString strDir;
	GetCurrentDirectory(MAX_PATH,strDir.GetBuffer(MAX_PATH));
	strDir.ReleaseBuffer();

	CString strCopy(strPath);
	char *pszToken = strtok(strCopy.GetBuffer(0),"\\");
	if(pszToken)
		SetCurrentDirectory(CString(pszToken) + "\\");	
	pszToken = strtok(NULL,"\\");
	while(pszToken)
	{
		CreateDirectory(pszToken,NULL);
		SetCurrentDirectory(pszToken);	
		pszToken = strtok(NULL,"\\");
	}

	SetCurrentDirectory(strDir);

	//Create the repository structure
	CFileStatus fs;
	try
	{
		CreateDirectory(strPath,NULL);
		if(!CFile::GetStatus(strPath,fs))	
			throw 1;
		strPath += "\\";

		const CString strTraceSettings("TraceSettings");
		if(!CFile::GetStatus(strPath + strTraceSettings,fs))
		{
			//Generate TraceSettings File
			HRSRC hRes = FindResource(AfxGetApp()->m_hInstance,MAKEINTRESOURCE(IDR_TRACE_SETTINGS),"TEXTFILE");
			HGLOBAL hGlobal = LoadResource(AfxGetApp()->m_hInstance,hRes);
			const char *pszTSettings = (const char*)LockResource(hGlobal);

			CFile f(strPath + strTraceSettings,CFile::modeCreate|CFile::modeWrite);
			if(pszTSettings)
				f.Write(pszTSettings,strlen(pszTSettings)+1);
		}

		//Generate Script,Test and profile folders
		CreateDirectory(strPath + strTestType,NULL);
		CreateDirectory(strPath + strProfileType,NULL);
		CreateDirectory(strPath + strScriptType,NULL);
		CreateDirectory(strPath + strDataFolder,NULL);
		CreateDirectory(strPath + strCaptureFolder,NULL);
		CreateDirectory(strPath + strObjectCodeFolder,NULL);
		
		//Generate an include folder
		CString strIncludePath(strPath + strIncludeFolder);
		CreateDirectory(strIncludePath ,NULL);

		//Check folders were created
		if(!CFile::GetStatus(strPath  + strTestType ,fs)||
			!CFile::GetStatus(strPath  + strProfileType ,fs)||
			!CFile::GetStatus(strPath  + strScriptType ,fs)||
			!CFile::GetStatus(strPath  + strDataFolder ,fs)||
			!CFile::GetStatus(strPath  + strCaptureFolder ,fs)||
			!CFile::GetStatus(strPath  + strObjectCodeFolder ,fs)||
			!CFile::GetStatus(strIncludePath ,fs))	
			throw 1;
		strIncludePath += "\\";
		
		const CString strGlobalVars("global_variables.inc");
		
		if(!CFile::GetStatus(strIncludePath + strGlobalVars,fs))
		{
			//Create Global_Variables.inc file
			HRSRC hRes = FindResource(AfxGetApp()->m_hInstance,MAKEINTRESOURCE(IDR_GLOBAL_VARIABLES),"TEXTFILE");
			HGLOBAL hGlobal = LoadResource(AfxGetApp()->m_hInstance,hRes);
			const char *pszGVars = (const char*)LockResource(hGlobal);

			CFile f(strIncludePath + strGlobalVars,CFile::modeCreate|CFile::modeWrite);
			if(pszGVars)
				f.Write(pszGVars,strlen(pszGVars)+1);
		}

		const CString strRespCodes("response_codes.inc");

		if(!CFile::GetStatus(strIncludePath + strRespCodes,fs))
		{
			//Create Response_codes.inc file
			HRSRC hRes = FindResource(AfxGetApp()->m_hInstance,MAKEINTRESOURCE(IDR_RESPONSE_CODES),"TEXTFILE");
			HGLOBAL hGlobal = LoadResource(AfxGetApp()->m_hInstance,hRes);
			const char *pszRCodes = (const char*)LockResource(hGlobal);
			CFile f(strIncludePath + strRespCodes,CFile::modeCreate|CFile::modeWrite);
			if(pszRCodes)
				f.Write(pszRCodes,strlen(pszRCodes)+1);
		}

		SetRepositoryPath(strPath );
	}
	catch(...)
	{
		CString strMsg("Failed to create repository structure.");

		LPTSTR lpMsgBuf;
        FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                       NULL,
                       GetLastError(),
                       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                       (LPTSTR) &lpMsgBuf,
                       0,
                       NULL);
		AfxMessageBox(strMsg + lpMsgBuf);
	}
}

void CMainFrame::OnToolsTracesettings() 
{
	CString baseUIPath = getenv("OPENSTA_HOME");
	baseUIPath +="\\BaseUI";

	ShellExecute(NULL, "open", "TraceSet.exe", "",baseUIPath,SW_SHOW);
}


void CMainFrame::OnHelpContents() 
{
    HtmlHelp(NULL, m_strHelpFilePath, HH_DISPLAY_TOC, 0);
}

void CMainFrame::OnHelpIndex() 
{
    HtmlHelp(NULL, m_strHelpFilePath, HH_DISPLAY_INDEX, 0);	
}

void CMainFrame::OnHelpSearch() 
{
    HH_FTS_QUERY q ;
    q.cbStruct         = sizeof(HH_FTS_QUERY) ;
    q.fUniCodeStrings  = FALSE ;
    q.pszSearchQuery   = "";
    q.iProximity       = HH_FTS_DEFAULT_PROXIMITY ;
    q.fStemmedSearch   = FALSE ;
    q.fTitleOnly       = FALSE ;
    q.fExecute         = FALSE ;
    q.pszWindow        = NULL ;
    HtmlHelp(NULL, m_strHelpFilePath, HH_DISPLAY_SEARCH, (DWORD)&q);	
}

BOOL CMainFrame::OnHelpInfo(HELPINFO* pHelpInfo) 
{
    HtmlHelp(NULL, m_strHelpFilePath, HH_DISPLAY_TOC, 0);
	
	return CFrameWnd::OnHelpInfo(pHelpInfo);
}

void CMainFrame::OnViewRepository() 
{
	if(m_wndWorkspace.IsVisible())
		m_wndWorkspace.ShowWindow(FALSE);
	else
		m_wndWorkspace.ShowWindow(TRUE);

	RecalcLayout();	
}

void CMainFrame::OnUpdateViewRepository(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_wndWorkspace.IsVisible());
}
