/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//========================================================================
//
// CECServiceStartup.cpp
// ---------------------
//
//    This is the implementation file for the CECServiceStartup class
//
//=========================================================================

#include "stdafx.h"
#include "daemoncfg.h"
#include "MainFrm.h"
#include "CECServiceStartup.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECServiceStartup dialog


CECServiceStartup::CECServiceStartup(CWnd* pParent /*=NULL*/)
        : CDialog(CECServiceStartup::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECServiceStartup)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_szNewStartup.Empty();
    m_szStartup.Empty();
    m_szService.Empty();
}


void CECServiceStartup::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECServiceStartup)
    // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECServiceStartup, CDialog)
//{{AFX_MSG_MAP(CECServiceStartup)
ON_BN_CLICKED(IDC_AUTOMATIC, OnAutomatic)
ON_BN_CLICKED(IDC_MANUAL, OnManual)
ON_BN_CLICKED(IDC_DISABLED, OnDisabled)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BOOL CECServiceStartup::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Setup the Service name in the dialog
    CStatic* pStatic = (CStatic*) GetDlgItem(IDC_SERVICE_NAME);
    ASSERT (pStatic != NULL);

    pStatic->SetWindowText(m_szName);

    // Now set up the radio butons...
    CString szManual, szAutomatic, szDisabled;

    VERIFY (szManual.LoadString(IDS_STARTUP_MANUAL));
    VERIFY (szAutomatic.LoadString(IDS_STARTUP_AUTO));
    VERIFY (szDisabled.LoadString(IDS_STARTUP_DISABLED));

    // Get the controls
    CButton* pAutomatic = (CButton*) GetDlgItem (IDC_AUTOMATIC);
    ASSERT (pAutomatic != NULL);

    CButton* pManual = (CButton*) GetDlgItem (IDC_MANUAL);
    ASSERT (pManual != NULL);

    CButton* pDisabled = (CButton*) GetDlgItem (IDC_DISABLED);
    ASSERT (pDisabled != NULL);

    // Initialization of radio boxes
    BOOL bManual = (m_szStartup.CompareNoCase(szManual) == 0) ? TRUE : FALSE;
    BOOL bAutomatic = (m_szStartup.CompareNoCase(szAutomatic) == 0) ? TRUE : FALSE;
    BOOL bDisabled = (m_szStartup.CompareNoCase(szDisabled) == 0) ? TRUE : FALSE;

    pManual->SetCheck(bManual);
    pAutomatic->SetCheck(bAutomatic);
    pDisabled->SetCheck(bDisabled);

    m_szNewStartup = m_szStartup;

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//
// OnAutomatic
// ===========
//
// Set to Automatic Mode

void CECServiceStartup::OnAutomatic()
{
    VERIFY (m_szNewStartup.LoadString(IDS_STARTUP_AUTO));
}

//
// OnManual
// ========
//
// Set to Manual Mode
void CECServiceStartup::OnManual()
{
    VERIFY (m_szNewStartup.LoadString(IDS_STARTUP_MANUAL));
}

//
// OnDisabled
// ==========
//
// Set to Disabled Mode
void CECServiceStartup::OnDisabled()
{
    VERIFY (m_szNewStartup.LoadString(IDS_STARTUP_DISABLED));
}

//
// OnOK
// ====
//
// Perform Startup Action
void CECServiceStartup::OnOK()
{
    m_szStartup = m_szNewStartup;

    PerformStartup();

    CDialog::OnOK();
}

void CECServiceStartup::PerformStartup()
{
    CMainFrame* pMainWnd = (CMainFrame*) AfxGetMainWnd();
    ASSERT(pMainWnd != NULL);

    CString szCommandLine;
    szCommandLine.Format ("%sServer\\%s" ,pMainWnd->GetCyranoHome(), m_szService);

    if (!pMainWnd->FileExists (szCommandLine)) return;

    // Now provide the requested parameter
    CString szManual, szAutomatic, szDisabled;

    VERIFY (szManual.LoadString(IDS_STARTUP_MANUAL));
    VERIFY (szAutomatic.LoadString(IDS_STARTUP_AUTO));
    VERIFY (szDisabled.LoadString(IDS_STARTUP_DISABLED));

    // TODO: Add extra validation here
    if (m_szStartup.CompareNoCase(szManual) == 0)
    {
        szCommandLine += " -m";
    }
    else if (m_szStartup.CompareNoCase(szAutomatic) == 0)
    {
        szCommandLine += " -a";
    }
    else if (m_szStartup.CompareNoCase(szDisabled) == 0)
    {
        szCommandLine += " -d";
    }

    HANDLE handle;
    DWORD  pid;

    pMainWnd->StartProcess(szCommandLine, &handle, &pid);
}
