/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ConfigurationDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "ConfigurationDlg.h"
#include "OstaDefs.h"
#include "TestFile.h"
#include "ScriptDefDlg.h"
#include "HostDefDlg.h"
#include "RegFunctions.h"
#include "TestPluginDoc.h"
#include "TaskGroupDefDlg.h"
#include "TaskGroupDefData.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CConfigurationDlg dialog
CConfigurationDlg::CConfigurationDlg(CWnd* pParent /*=NULL*/)
	: CDragDropDialog(CConfigurationDlg::IDD, pParent),
	m_nMaxTasks(17),
	m_nMaxGroups(23)
{
	CECGridCtrl::GridCtrlInit(AfxGetInstanceHandle());
	//{{AFX_DATA_INIT(CConfigurationDlg)
	m_strDescription = _T("");
	//}}AFX_DATA_INIT
	m_pTestFile = NULL;
	m_pDoc = NULL;
}

void CConfigurationDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigurationDlg)
	DDX_Control(pDX, IDC_DESCRIPTION, m_ctlTestDescription);
	DDX_Control(pDX, IDC_TESTS_LIST, m_ctlTestList);
	DDX_Text(pDX, IDC_DESCRIPTION, m_strDescription);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigurationDlg, CDialog)
	//{{AFX_MSG_MAP(CConfigurationDlg)
	ON_WM_SIZE()
	ON_NOTIFY(LVN_BEGINDRAG, IDC_TESTS_LIST, OnBegindragTestsList)
	ON_NOTIFY(NM_DBLCLK, IDC_TESTS_LIST, OnDblclkTestList)
	ON_COMMAND(ID_DELETESELECTION, OnDeleteselection)
	ON_UPDATE_COMMAND_UI(ID_DELETESELECTION, OnUpdateDeleteselection)
	ON_EN_CHANGE(IDC_DESCRIPTION, OnChangeDescription)
	//}}AFX_MSG_MAP
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConfigurationDlg message handlers

void CConfigurationDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	if(m_ctlTestDescription.GetSafeHwnd() && m_ctlTestList.GetSafeHwnd())
	{
		CRect rcDesc;
		m_ctlTestDescription.GetWindowRect(&rcDesc);
		ScreenToClient(&rcDesc);

		m_ctlTestDescription.SetWindowPos(NULL,0,0,cx-rcDesc.left-5,rcDesc.Height(),SWP_NOZORDER|SWP_NOMOVE);

		m_ctlTestList.MoveWindow(0,rcDesc.bottom + 5 ,cx,cy-rcDesc.bottom-5);
	}
}

BOOL CConfigurationDlg::OnEraseBkgnd(CDC *pDC)
{
	return CDialog::OnEraseBkgnd(pDC);
}

BOOL CConfigurationDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	SetupTestList();
    
	m_ListImages.Create(IDB_CONFIG_IMAGES,16,1,RGB(192,192,192));
	
	m_ctlTestList.SetImageList(&m_ListImages);

	LoadTestFile(m_pTestFile);

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CConfigurationDlg::SetupTestList()
{
    //Setup up the columns in the grid
	m_ctlTestList.SetColumnCount(m_nMaxTasks+enFirstTaskCol);
	m_ctlTestList.SetFixedRowCount();
	m_ctlTestList.SetItemText(0,enNameCol,"Task Group");
	m_ctlTestList.SetItemText(0,enHostCol, "Host");
	m_ctlTestList.SetItemText(0,enVUCol, "VUs");
  
	CString strHeader;
	for(int nCol=0;nCol<m_nMaxTasks;nCol++)
	{
		strHeader.Format("Task %d",nCol+1);
		m_ctlTestList.SetItemText(0,nCol+enFirstTaskCol, strHeader);
		m_ctlTestList.AutoSizeColumn(nCol+enFirstTaskCol);
	}
	m_ctlTestList.SetRowCount(m_nMaxGroups+1);

	for(nCol=0;nCol<enFirstTaskCol+m_nMaxTasks;nCol++)
	{
		for(int nRow=1;nRow<=m_nMaxGroups;nRow++)
		{
			DWORD dwState = m_ctlTestList.GetItemState(nRow,nCol);
			m_ctlTestList.SetItemState(nRow,nCol,dwState | GVIS_READONLY);
		}
	}
}

DROPEFFECT CConfigurationDlg::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	return DROPEFFECT_COPY;
}

void CConfigurationDlg::OnDragLeave()
{
}

DROPEFFECT CConfigurationDlg::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	if(!pDataObject->IsDataAvailable(m_ClipFormat))
		return DROPEFFECT_NONE;

	CRect rcTestList;
	m_ctlTestList.GetWindowRect(&rcTestList);
	ScreenToClient(&rcTestList);

	CPoint pt(point);
	pt.y -= rcTestList.top;
	int nRow,nCol;
	//Dropping on the hosts column or VUs column or off list
	if(!m_ctlTestList.HitTest(pt,nRow,nCol) || (nCol==enHostCol) || (nCol==enVUCol) || (nCol == enNameCol))
		return DROPEFFECT_NONE;

	//Can't drop on header row
	if(!nRow)
		return DROPEFFECT_NONE;
	
	int nGroup = nRow-1;
	int nTask = nCol-enFirstTaskCol;

	//Can only drop profiles or scripts(at the moment)
	HGLOBAL hData = pDataObject->GetGlobalData(m_ClipFormat);
	CString strItem = (char*)GlobalLock(hData);
	GlobalUnlock(hData);
	if(!IsScript(strItem)&&!IsProfile(strItem))
		return DROPEFFECT_NONE;

	//Disallow if the file does not exist or is a directory
	CFileStatus fs;
	if(!CFile::GetStatus(strItem,fs) || (fs.m_attribute & CFile::Attribute::directory))
		return DROPEFFECT_NONE;

	//Dropping on a new line
	if(m_pTestFile->GetNumTaskGroups()<=nGroup)	
		return DROPEFFECT_COPY;

	//Only one item in a group with a profile
	if(nTask>0 && m_pTestFile->GetNumTasks(nGroup) &&
		(m_pTestFile->GetTaskType(nGroup,0)==CTestFile::enProfile))
		return DROPEFFECT_NONE;

	//Disallow drop if dragging profile anywhere but column 1
	if(IsProfile(strItem)&&nTask)
		return DROPEFFECT_NONE;

	//Can only have scripts of the same type in a group
	CString strType = m_pTestFile->GetTaskTargetType(nGroup,0);

	int nDotPos = strItem.ReverseFind('.');
	if(strItem.Mid(nDotPos+1).CompareNoCase(strType)!=0)
		return DROPEFFECT_NONE;

	return DROPEFFECT_COPY;
}

BOOL CConfigurationDlg::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point)
{
	if(pDataObject->IsDataAvailable(m_ClipFormat))
	{
		CRect rcTestList;
		m_ctlTestList.GetWindowRect(&rcTestList);
		ScreenToClient(&rcTestList);

		CPoint pt(point);
		pt.y -= rcTestList.top;

		int nRow,nCol;
		if(!m_ctlTestList.HitTest(pt,nRow,nCol))
			return false;

		HGLOBAL hData = pDataObject->GetGlobalData(m_ClipFormat);
		
		CString strItem = (char*)GlobalLock(hData);
		GlobalUnlock(hData);

		InsertItem(nRow,nCol,strItem);

		m_pDoc->SetModifiedFlag();

		return true;
	}
	return false;
}

void CConfigurationDlg::InsertItem(int nItem,int nSubItem,const CString& strItemPath)
{
	if(IsScript(strItemPath))
		InsertScript(nItem,nSubItem,strItemPath);
	else
	if(IsProfile(strItemPath))
		InsertProfile(nItem,nSubItem,strItemPath);
}

void CConfigurationDlg::InsertScript(int nItem,int nSubItem,const CString& strItemPath)
{
	int nGroup = nItem-1;
	int nTask = nSubItem-enFirstTaskCol;
	if(nGroup >= m_pTestFile->GetNumTaskGroups())
	{
		nGroup = m_pTestFile->AddNewTaskGroup();

		m_ctlTestList.SetItemText(nGroup+1,enNameCol,m_pTestFile->GetTaskGroupName(nGroup));
		m_ctlTestList.SetItemImage(nGroup+1,enNameCol,enNameImage);
		m_ctlTestList.AutoSizeColumn(enNameCol);

		m_ctlTestList.SetItemText(nGroup+1,enHostCol,m_pTestFile->GetTaskGroupHost(nGroup));
		m_ctlTestList.SetItemImage(nGroup+1,enHostCol,enHostImage);
		m_ctlTestList.AutoSizeColumn(enHostCol);

		//Insert VUs
		m_ctlTestList.SetItemImage(nGroup+1,enVUCol,enVUImage);
		m_ctlTestList.SetItemText(nGroup+1,enVUCol,"1");
		m_ctlTestList.AutoSizeColumn(enVUCol);
	}

	int nImage = RegisterFileExtension(strItemPath);

	int nPos = strItemPath.ReverseFind('\\');
	CString strName = (nPos == -1) ? strItemPath : strItemPath.Mid(nPos+1);
		
	nPos = strName.ReverseFind('.');
	strName = strName.Left(nPos);
	
	if(m_pTestFile->GetNumTasks(nGroup)<=nTask)
		nTask = m_pTestFile->AddNewTask(nGroup,CTestFile::enScript,strName,strItemPath,GetTargetType(strItemPath));
	else
	{
		CString strMsg;
		strMsg = "Are you sure you want to overwrite \"" + m_pTestFile->GetTaskName(nGroup,nTask) + "\"?";
		if(AfxMessageBox(strMsg,MB_YESNO)==IDNO)
			return;
		m_pTestFile->SetTask(nGroup,nTask,CTestFile::enScript,strName,strItemPath,GetTargetType(strItemPath));
	}

	m_ctlTestList.SetItemText(nGroup+1,nTask+enFirstTaskCol,strName);
	m_ctlTestList.SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);
	m_ctlTestList.AutoSizeColumn(nTask+enFirstTaskCol);
	m_ctlTestList.Invalidate();

	if(!EditScriptTask(nGroup,nTask))
	{
		//Delete everything we've added
		DeleteItem(nGroup,nTask);
	}
}

void CConfigurationDlg::InsertProfile(int nItem,int nSubItem,const CString& strItemPath)
{
	//Only one task allowed in a profiles group
	int nImage = RegisterFileExtension(strItemPath);

	int nPos = strItemPath.ReverseFind('\\');
	CString strName = (nPos == -1) ? strItemPath : strItemPath.Mid(nPos+1);
		
	nPos = strName.ReverseFind('.');
	strName = strName.Left(nPos);
	
	int nGroup = nItem-1;
	int nTask = nSubItem-enFirstTaskCol;
	if(nGroup >= m_pTestFile->GetNumTaskGroups())
	{
		nGroup = m_pTestFile->AddNewTaskGroup();

		m_ctlTestList.SetItemText(nGroup+1,enNameCol,m_pTestFile->GetTaskGroupName(nGroup));
		m_ctlTestList.SetItemImage(nGroup+1,enNameCol,enNameImage);
		m_ctlTestList.AutoSizeColumn(enNameCol);

		m_ctlTestList.SetItemText(nGroup+1,enHostCol,m_pTestFile->GetTaskGroupHost(nGroup));
		m_ctlTestList.SetItemImage(nGroup+1,enHostCol,enHostImage);
		m_ctlTestList.AutoSizeColumn(enHostCol);

		nTask = m_pTestFile->AddNewTask(nGroup,CTestFile::enProfile,strName,strItemPath,GetTargetType(strItemPath));

		//Disable other columns
		m_ctlTestList.SetItemBkColour(nGroup+1,enVUCol,GetSysColor(COLOR_BTNFACE));
		for(int nCol=enFirstTaskCol+1;nCol<enFirstTaskCol+m_nMaxTasks;nCol++)
		{
			m_ctlTestList.SetItemBkColour(nGroup+1,nCol,GetSysColor(COLOR_BTNFACE));
		}
	}
	else
	{
		CString strMsg;
		strMsg = "Are you sure you want to overwrite \"" + m_pTestFile->GetTaskName(nGroup,nTask) + "\"?";
		if(AfxMessageBox(strMsg,MB_YESNO)==IDNO)
			return;
		m_pTestFile->SetTask(nGroup,nTask,CTestFile::enScript,strName,strItemPath,GetTargetType(strItemPath));
	}

	m_ctlTestList.SetItemText(nGroup+1,nTask+enFirstTaskCol,strName);
	m_ctlTestList.SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);
	m_ctlTestList.AutoSizeColumn(nTask+enFirstTaskCol);
	m_ctlTestList.Invalidate();

	if(!EditProfileTask(nGroup))
	{
		//Delete everything we've added
		DeleteItem(nGroup,nTask);
	}
}

void CConfigurationDlg::DeleteItem(int nGroup,int nTask)
{
	if(m_pTestFile->GetNumTaskGroups() >= nGroup ||
		m_pTestFile->GetNumTasks(nGroup) >= nTask)
		return;

	m_pTestFile->DeleteItem(nGroup,nTask);
	m_ctlTestList.SetRedraw(FALSE);
	m_ctlTestList.DeleteAllItems();
	SetupTestList();
	LoadTestFile(m_pTestFile);
	m_ctlTestList.SetRedraw(TRUE);
	m_pDoc->SetModifiedFlag();
}

int CConfigurationDlg::RegisterFileExtension(const CString& strFile)
{
	int nPos = strFile.ReverseFind('.');
	if(nPos == -1)
		return -1;

	CString strExtension(strFile.Mid(nPos));

	int nIndex=0;
	if(m_mapExtToImage.Lookup(strExtension,nIndex))
		return nIndex;

	SHFILEINFO shfi;
	SHGetFileInfo( strFile, 0, &shfi, sizeof( SHFILEINFO ), 
               SHGFI_ICON | SHGFI_SHELLICONSIZE | SHGFI_SMALLICON);

	if(!shfi.hIcon || !(nIndex = (WORD)m_ListImages.Add(shfi.hIcon)))
		return -1;

	DestroyIcon(shfi.hIcon);

	m_mapExtToImage.SetAt(strExtension,nIndex);	
	return nIndex;
}

void CConfigurationDlg::OnBegindragTestsList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMLISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	
	CString strData = m_ctlTestList.GetItemText(pNMListView->iItem,pNMListView->iSubItem);

	HGLOBAL hData = GlobalAlloc(GMEM_FIXED|GMEM_ZEROINIT,strData.GetLength()+1);
	char *szData = (char*)GlobalLock(hData);
	strcpy(szData,strData);
	GlobalUnlock(hData);

	COleDataSource dataSource;
	dataSource.CacheGlobalData(m_ClipFormat,hData);
	dataSource.DoDragDrop();
		
	*pResult = 0;
}

CString CConfigurationDlg::GetTargetType(const CString& strPath)
{
	int nPos = strPath.ReverseFind('.');
	return strPath.Mid(nPos+1);
}

CString CConfigurationDlg::GetScriptOutputPath()
{
	return m_strTestFolder;
}

void CConfigurationDlg::GenerateScripts()
{
	CString strOutputPath = GetScriptOutputPath();

	CString str,strTemp;
	try
	{
		CStdioFile f(strOutputPath + m_strTestName + ".TSS",CFile::modeCreate|CFile::modeWrite);

		str = GetScriptHeader(m_strTestName + ".TSS");

		//ENVIRONMENT
		str += "Environment\n";
		str += "\tDescription\t\"\"\n";
		str += "\n";

		//DEFINITIONS
		str += "Definitions\n";

		int nNumGroups = m_pTestFile->GetNumTaskGroups();
		CString strCode;
		for(int nGroup=0;nGroup<nNumGroups;nGroup++)
		{
			CString strGroupName( m_pTestFile->GetTaskGroupName(nGroup));
			switch(m_pTestFile->GetTaskType(nGroup,0))
			{
			case CTestFile::enScript:
				str += "\tTest " + strGroupName + "\n";
				strCode += "\tExecute Test " + strGroupName + "\n";
				
				GenerateTaskGroupScripts(nGroup);
			break;
			case CTestFile::enProfile:
				strTemp.Format("\tThread %s for target %s\n",strGroupName,m_pTestFile->GetTaskTargetType(nGroup,0));
				str += strTemp;

				strTemp.Format("\tExecute Thread %s using script \"%s\", on node \"%s\"\n",strGroupName,m_pTestFile->GetTaskName(nGroup,0),m_pTestFile->GetTaskGroupHost(nGroup));
				strCode += strTemp;
			break;
			}

		}
		str += "\n";
		str += "Code\n";

		str += strCode;

		str += "\n";
		str += "Exit";

		f.WriteString(str);
	}
	catch(...)
	{

	}
}

void CConfigurationDlg::GenerateTaskGroupScripts(int nGroup)
{
	GenerateTaskGroupTSS(nGroup);
	GenerateTaskGroupSCL(nGroup);
}

void CConfigurationDlg::GenerateTaskGroupTSS(int nGroup)
{
	CString strGroupName;
	strGroupName.Format("%s_%d",m_strTestName,nGroup+1);

	CString strOutputPath = GetScriptOutputPath();

	LPBYTE pData;
	DWORD dwDataSize;
	m_pTestFile->GetTaskGroupData(nGroup,&pData,dwDataSize);
	CTaskGroupDefData TGData;
	TGData.LoadFromBuffer(pData);

	CString str,strTemp;
	try
	{
		CStdioFile f(strOutputPath + strGroupName + ".TSS",CFile::modeCreate|CFile::modeWrite);
	
		str = GetScriptHeader(strGroupName + ".TSS");

		str += "Environment\n";
		str += "\n";
		str += "\tDescription\t\"\"\n";
		str += "\n";
		str += "Definitions\n";

		str += "\tThread HTTP_INJECTOR For Target " + m_pTestFile->GetTaskTargetType(nGroup,0) + "\n";
		str += "\n";
		
		str += "\tConstant HOST_NAME = \"" + m_pTestFile->GetTaskGroupHost(nGroup) + "\"\n";
		str += "\tConstant SCRIPT_NAME = \"" + strGroupName + "\"\n";

		strTemp.Format("\tConstant VU_RAMP = %d\n",TGData.GetNumVUPerInterval());
		str += strTemp;

		strTemp.Format("\tConstant VU_TODO = %d\n",TGData.GetNumVUs());
		str += strTemp;
		
		strTemp.Format("\tConstant VU_WAIT = %d\n",TGData.GetStartInterval());
		str += strTemp;
		str += "\n";

		str += "\tInteger VU_LEFT\n";
		str += "\n";
		str += "Code\n";
		str += "\tSet VU_LEFT = VU_TODO\n";
		str += "\n";
		str += "WHILE-LOOP:\n";
		str += "\tIf (VU_LEFT > VU_RAMP) Then\n";
		str += "\t\tExecute Thread HTTP_INJECTOR Using Script SCRIPT_NAME, on node HOST_NAME, number VU_RAMP\n";
		str += "\t\tSet VU_LEFT = VU_LEFT - VU_RAMP\n";
		str += "\t\tWait VU_WAIT\n";
		str += "\t\tGoto WHILE-LOOP\n";
		str += "\tEndIf\n";
		str += "\n";
		str += "\tIf (VU_LEFT > 0) Then\n";
		str += "\t\tExecute Thread HTTP_INJECTOR Using Script SCRIPT_NAME, on node HOST_NAME, number VU_LEFT\n";
		str += "\tEndIf\n";
		str += "\n";
		str += "Exit";
	
		f.WriteString(str);

		f.Close();
	}
	catch(...)
	{
	}
}

void CConfigurationDlg::GenerateTaskGroupSCL(int nGroup)
{
	CString strGroupName;
	strGroupName.Format("%s_%d",m_strTestName,nGroup+1);

	CString strOutputPath = GetScriptOutputPath();

	LPBYTE pData;
	DWORD dwDataSize;
	m_pTestFile->GetTaskGroupData(nGroup,&pData,dwDataSize);
	CTaskGroupDefData TGData;
	TGData.LoadFromBuffer(pData);

	CString str,strTemp;
	try
	{
		CStdioFile f(strOutputPath + strGroupName + ".SCL",CFile::modeCreate|CFile::modeWrite);
	
		str = GetScriptHeader(strGroupName + ".SCL");

		str += "Environment\n";
		str += "\n";
		str += "\tDescription\t\"\"\n";
		str += "\n";
		str += "Definitions\n";

		str += "\tInclude	\"GLOBAL_VARIABLES.INC\"\n";
		str += "\n";
		str += "\tConstant	USE_PAGE_TIMERS = " + (TGData.GetUsePageTimers() ? CString("1\n") :CString("0\n")); 
		str += "\tConstant  USER_AGENT = \"DEFAULT\"\n";

		strTemp.Format("\tConstant	HTTP-STAT-LV = \"%c\"\n",TGData.GetStatLevel());
		str += strTemp;
		str += "\n";

		int nMinDelay;
		int nMaxDelay;
		LPBYTE pData;
		DWORD dwDataSize;
		CScriptDefData m_scriptData;
		
		int nNumTasks = m_pTestFile->GetNumTasks(nGroup);
		for(int nTask=0;nTask <nNumTasks;nTask++)
		{
			m_pTestFile->GetTaskData(nGroup,nTask,&pData,dwDataSize);
			m_scriptData.LoadFromBuffer(pData);

			if(m_scriptData.GetUseFixedIterDelay())
				nMaxDelay = nMinDelay = m_scriptData.GetFixedIterDelay();
			else
			{
				nMinDelay = m_scriptData.GetMinIterDelay();
				nMaxDelay = m_scriptData.GetMaxIterDelay();
			}
			
			strTemp.Format("\tInteger	DELAY_%d (%d - %d)\n",nTask+1,nMinDelay,nMaxDelay);
			str += strTemp;
		}
		str += "\n";
		str += "\tInteger 	IC\n";
		str += "\tCharacter	CYR_HTTP-STAT-LV, Thread\n";

		str += "\n";
		str += "Code\n";
	
		str += "\tSet CYR_HTTP-STAT-LV = HTTP-STAT-LV\n";
		str += "\n";

		for(nTask=0;nTask <nNumTasks;nTask++)
		{
			m_pTestFile->GetTaskData(nGroup,nTask,&pData,dwDataSize);
			m_scriptData.LoadFromBuffer(pData);

			strTemp.Format("! Task #%d\n",nTask+1);
			str += strTemp;

			strTemp.Format("\tDo IC = 1, %d\n",m_scriptData.GetNumIterations());
			str += strTemp;

			strTemp.Format("\t\tCall Script \"%s\" [USER_AGENT,USE_PAGE_TIMERS]\n",m_pTestFile->GetTaskName(nGroup,nTask));
			str += strTemp;
			
			strTemp.Format("\t\tGenerate DELAY_%d\n",nTask+1);
			str += strTemp;
			
			strTemp.Format("\t\tWait DELAY_%d\n",nTask+1);
			str += strTemp;

			str += "\tEndDo\n";
			str += "\n";
		}

		str += "Exit";
	
		f.WriteString(str);

		f.Close();
	}
	catch(...)
	{
	}
}

CString CConfigurationDlg::GetScriptHeader(const CString& strName)
{
	CString str;
	str += "!/////////////////////////////////////////////////////////////////////////\n";
	str += "!//                                                                     //\n";
	str += "!// OpenSTA http://www.opensta.org/                                     //\n";
	str += "!//                                                                     //\n";
	str += "!//  This code has been generated for you as a starting template.       //\n";
	str += "!//  Please refer to the SCL Documentation if you wish to modify it     //\n";
	str += "!//  to provide further functionality to your script.                   //\n";
	str += "!//                                                                     //\n";
	str += "!//  File Name:  "+strName + "\n";
	str += "!//                                                                     //\n";
	str += "!/////////////////////////////////////////////////////////////////////////\n";
	str += "\n";
	return str;
}

void CConfigurationDlg::SetDocument(CTestPluginDoc *pDoc)
{
	m_pDoc = pDoc;
	m_pTestFile = pDoc->GetTestFile();
}

void CConfigurationDlg::LoadTestFile(CTestFile* pFile)
{
	CString strTestPath(pFile->GetTestFilePath());
	int nPos = strTestPath.ReverseFind('\\');
	if(nPos!=-1)
	{
		m_strTestFolder = strTestPath.Left(nPos+1);
		m_strTestName = strTestPath.Mid(nPos+1);
	}
	else
		m_strTestName = strTestPath;

	nPos = m_strTestName.ReverseFind('.');
	if(nPos != -1)
		m_strTestName = m_strTestName.Left(nPos);

	int nNumGroups = m_pTestFile->GetNumTaskGroups();
	for(int nGroup=0;nGroup<nNumGroups;nGroup++)
	{
		CString strName = m_pTestFile->GetTaskGroupName(nGroup);
		m_ctlTestList.SetItemText(nGroup+1,enNameCol,strName);
		m_ctlTestList.SetItemImage(nGroup+1,enNameCol,enNameImage);
		m_ctlTestList.AutoSizeColumn(enNameCol);
		
		CString strHost = m_pTestFile->GetTaskGroupHost(nGroup);
		m_ctlTestList.SetItemText(nGroup+1,enHostCol, strHost );
		m_ctlTestList.SetItemImage(nGroup+1,enHostCol,enHostImage);
		m_ctlTestList.AutoSizeColumn(enHostCol);

		int nNumTasks = m_pTestFile->GetNumTasks(nGroup);

		if(nNumTasks)
		{
			switch(m_pTestFile->GetTaskType(nGroup,0))
			{
			case CTestFile::enScript:
			{
				m_ctlTestList.SetItemImage(nGroup+1,enVUCol,enVUImage);

				LPBYTE pData;
				DWORD dwBufSize;
				m_pTestFile->GetTaskGroupData(nGroup,&pData,dwBufSize);
				CTaskGroupDefData TGData;
				TGData.LoadFromBuffer(pData);
				
				CString strNumVUs;
				strNumVUs.Format("%d",TGData.GetNumVUs());
				m_ctlTestList.SetItemText(nGroup+1,enVUCol,strNumVUs);

				m_ctlTestList.AutoSizeColumn(enVUCol);
			}
			break;
			case CTestFile::enProfile:
			{
				m_ctlTestList.SetItemBkColour(nGroup+1,enVUCol,GetSysColor(COLOR_BTNFACE));
				for(int nCol=enFirstTaskCol+1;nCol<enFirstTaskCol+m_nMaxTasks;nCol++)
				{
					m_ctlTestList.SetItemBkColour(nGroup+1,nCol,GetSysColor(COLOR_BTNFACE));
				}
			}
			break;
			}
		}

		for(int nTask=0;nTask<nNumTasks;nTask++)
		{
			CString strName = m_pTestFile->GetTaskName(nGroup,nTask);
			m_ctlTestList.SetItemText(nGroup+1,nTask+enFirstTaskCol, strName);
			
			CString strPath = m_pTestFile->GetTaskPath(nGroup,nTask);
			int nImage = RegisterFileExtension(strPath);
			m_ctlTestList.SetItemImage(nGroup+1,nTask+enFirstTaskCol,nImage);
			m_ctlTestList.AutoSizeColumn(nTask+enFirstTaskCol);
		}
	}

	m_strDescription = m_pTestFile->GetDescription();	
}

void CConfigurationDlg::OnDblclkTestList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_GRIDVIEW *pNMGV = (NM_GRIDVIEW*)pNMHDR;

	*pResult = 0;

	if(!pNMGV->iRow || (pNMGV->iRow-1 >= m_pTestFile->GetNumTaskGroups()))
		return;
	
	if(pNMGV->iColumn == enHostCol)
	{
		if(EditHost(pNMGV->iRow-1))
			m_pDoc->SetModifiedFlag();
		return;
	}
	else
	if(pNMGV->iColumn == enVUCol)
	{
		CTestFile::TaskType nType = m_pTestFile->GetTaskType(pNMGV->iRow-1,0);
		if((nType == CTestFile::enScript) && EditVUs(pNMGV->iRow-1))
			m_pDoc->SetModifiedFlag();
		return;
	}
	
	if(pNMGV->iColumn < enFirstTaskCol)
		return;
	
	if(m_pTestFile->GetNumTasks(pNMGV->iRow-1)<pNMGV->iColumn-enFirstTaskCol)
		return;

	CTestFile::TaskType nType = m_pTestFile->GetTaskType(pNMGV->iRow-1,pNMGV->iColumn-enFirstTaskCol);
	switch(nType)
	{
	case CTestFile::enScript:
		if(EditScriptTask(pNMGV->iRow-1,pNMGV->iColumn-enFirstTaskCol))
			m_pDoc->SetModifiedFlag();
	break;
	case CTestFile::enProfile:
		if(EditProfileTask(pNMGV->iRow-enFirstTaskCol))
			m_pDoc->SetModifiedFlag();
	break;
	}
}

BOOL CConfigurationDlg::EditHost(int nGroup)
{
	CHostDefDlg dlg;
	dlg.m_strHostName = m_pTestFile->GetTaskGroupHost(nGroup);
	dlg.m_pTestFile = m_pTestFile;
	if(dlg.DoModal() == IDOK)
	{
		m_pTestFile->SetTaskGroupHost(nGroup,dlg.m_strHostName);
		m_ctlTestList.SetItemText(nGroup+1,enHostCol,dlg.m_strHostName);
		m_ctlTestList.AutoSizeColumn(enHostCol);
		m_ctlTestList.Invalidate();

		return true;
	}
	return false;
}

BOOL CConfigurationDlg::EditProfileTask(int nGroup)
{
	return true;
}

BOOL CConfigurationDlg::EditScriptTask(int nGroup,int nTask)
{
	DWORD dwBufSize=0;
	LPBYTE pData = NULL;
	m_pTestFile->GetTaskData(nGroup,nTask,&pData,dwBufSize);
	
	CScriptDefDlg dlg;
	dlg.LoadFromBuffer(pData);
	if(dlg.DoModal() == IDOK)
	{
		delete []pData;

		dwBufSize = 0;
		dlg.SaveToBuffer(NULL,dwBufSize);
		pData = new BYTE[dwBufSize];
		dlg.SaveToBuffer(pData,dwBufSize);
		m_pTestFile->SetTaskData(nGroup,nTask,pData,dwBufSize);
		return true;
	}
	return false;
}


BOOL CConfigurationDlg::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
	case WM_KEYDOWN:
		switch(pMsg->wParam)
		{
		case VK_DELETE:
			DeleteSelection();
		break;
		}
	break;
	}
	
	return CDialog::PreTranslateMessage(pMsg);
}

void CConfigurationDlg::DeleteSelection()
{
	int nNumRowsDeleted=0;
	int nNumColsDeleted=0;
	CCellRange cellRange = m_ctlTestList.GetSelectedCellRange();
	
	for(int nRow=cellRange.GetMinRow();nRow<=cellRange.GetMaxRow();nRow++)
	{
		int nGroup = nRow-1-nNumRowsDeleted;
		if(nGroup>=m_pTestFile->GetNumTaskGroups())
			break;

		for(int nCol=cellRange.GetMinCol();nCol<=cellRange.GetMaxCol();nCol++)
		{
			int nTask = nCol-enFirstTaskCol-nNumColsDeleted;
			if(nCol == enHostCol || (nTask>=m_pTestFile->GetNumTasks(nGroup)))
				continue;

			if(m_ctlTestList.GetCell(nRow,nCol)->IsSelected())
			{
				if(nCol==enNameCol)
				{
					m_pTestFile->DeleteTaskGroup(nGroup);
					nNumRowsDeleted++;
					break;
				}
				else
				{
					m_pTestFile->DeleteItem(nGroup,nTask);
					nNumColsDeleted++;
				}
			}
		}
		nNumColsDeleted=0;
	}

	m_ctlTestList.SetRedraw(FALSE);
	m_ctlTestList.DeleteAllItems();
	SetupTestList();
	LoadTestFile(m_pTestFile);
	m_ctlTestList.SetRedraw(TRUE);
	m_pDoc->SetModifiedFlag();
}

BOOL CConfigurationDlg::EditVUs(int nGroup)
{
	DWORD dwBufSize=0;
	LPBYTE pData = NULL;
	m_pTestFile->GetTaskGroupData(nGroup,&pData,dwBufSize);

	CTaskGroupDefDlg dlg;
	dlg.LoadFromBuffer(pData);
	if(dlg.DoModal() == IDOK)
	{
		delete []pData;

		dwBufSize = 0;
		dlg.SaveToBuffer(NULL,dwBufSize);
		
		pData = new BYTE[dwBufSize];
		dlg.SaveToBuffer(pData,dwBufSize);
		
		m_pTestFile->SetTaskGroupData(nGroup,pData,dwBufSize);

		CString strNumVUs;
		strNumVUs.Format("%d",dlg.m_nNumVUs);
		m_ctlTestList.SetItemText(nGroup+1,enVUCol,strNumVUs);
		m_ctlTestList.RedrawCell(nGroup+1,enVUCol);

		return true;
	}

	return false;
}

void CConfigurationDlg::OnDeleteselection() 
{
	DeleteSelection();	
}

void CConfigurationDlg::OnUpdateDeleteselection(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_ctlTestList.GetSelectedCount()?TRUE:FALSE);	
}



void CConfigurationDlg::OnChangeDescription() 
{
	UpdateData(TRUE);
	m_pTestFile->SetDescription(m_strDescription);	
	m_pDoc->SetModifiedFlag();
}
