/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MonitoringDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "MonitoringDlg.h"
#include "PluginTypes.h"
#include "TestMonitorPlugin.h"
#include "CmdExec.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ID_TESTFIND_TIMER 0
static int ID_REFRESH_RATE = 1000;

/////////////////////////////////////////////////////////////////////////////
// CMonitoringDlg dialog


CMonitoringDlg::CMonitoringDlg(CWnd* pParent /*=NULL*/)
	: CDragDropDialog(CMonitoringDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMonitoringDlg)
	//}}AFX_DATA_INIT

	m_bFrameHidden = false;
	m_strTestName = "";
	m_strManagerReg = "";
	m_bTestActive = false;
	m_pSummary = NULL;
	m_strActiveTarget = "";
	m_pActiveWnd = NULL;

	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	m_pPluginManager = pApp->GetPluginManager();
}


void CMonitoringDlg::DoDataExchange(CDataExchange* pDX)
{
	CDragDropDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitoringDlg)
	DDX_Control(pDX, IDC_DISPLAY_LABEL, m_ctlDisplayLabel);
	DDX_Control(pDX, IDC_GROUP_LABEL, m_ctlGroupLabel);
	DDX_Control(pDX, IDC_VIEW_LIST, m_ctlViewList);
	DDX_Control(pDX, IDC_EXPAND_FRAME, m_ctlExpandFrame);
	DDX_Control(pDX, IDC_SHOWHIDE_BTN, m_ctlShowHideBtn);
	DDX_Control(pDX, IDC_TASK_LIST, m_ctlTaskList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitoringDlg, CDragDropDialog)
	//{{AFX_MSG_MAP(CMonitoringDlg)
	ON_WM_SHOWWINDOW()
	ON_BN_CLICKED(IDC_SHOWHIDE_BTN, OnShowhideBtn)
	ON_WM_SIZE()
	ON_LBN_SELCHANGE(IDC_TASK_LIST, OnSelchangeTaskList)
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_LBN_SELCHANGE(IDC_VIEW_LIST, OnSelchangeViewList)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitoringDlg message handlers

void CMonitoringDlg::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDragDropDialog::OnShowWindow(bShow, nStatus);
	
	if(bShow)
	{
		UpdateTaskList();
		IsActive();
		SetTimer(ID_TESTFIND_TIMER,ID_REFRESH_RATE,NULL);
	}
	else
	{
		KillTimer(ID_TESTFIND_TIMER);
	}
}

void CMonitoringDlg::SetTestFile(CTestFile* pTestFile)
{
	m_pTestFile = pTestFile;
	CString strTFPath = m_pTestFile->GetTestFilePath();

	int nLen = strTFPath.GetLength();
	int nPos = strTFPath.ReverseFind('\\');
	m_strTestPath = strTFPath.Left(nPos+1);

	m_strTestName = strTFPath.Right(nLen-nPos-1);
	nPos = m_strTestName.ReverseFind('.');
	m_strTestName = m_strTestName.Left(nPos);

}


BOOL CMonitoringDlg::OnInitDialog() 
{
	CDragDropDialog::OnInitDialog();
	
	m_bmpShowHide.LoadBitmap(m_bFrameHidden ? IDB_DOWN_ARROW : IDB_UP_ARROW);
	m_ctlShowHideBtn.SetBitmap((HBITMAP)m_bmpShowHide);
	
	UpdateTaskList();

	if(m_bTestActive)
	{
		m_ctlTaskList.SetCurSel(0);
		OnSelchangeTaskList();
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CMonitoringDlg::OnShowhideBtn() 
{
	m_bFrameHidden = !m_bFrameHidden;
	
	m_ctlTaskList.ShowWindow(m_bFrameHidden ? SW_HIDE : SW_SHOW);
	m_ctlViewList.ShowWindow(m_bFrameHidden ? SW_HIDE : SW_SHOW);
	m_ctlGroupLabel.ShowWindow(m_bFrameHidden ? SW_HIDE : SW_SHOW);
	m_ctlDisplayLabel.ShowWindow(m_bFrameHidden ? SW_HIDE : SW_SHOW);

	CRect rcClient;
	GetClientRect(&rcClient);
	OnSize(SW_NORMAL,rcClient.Width(),rcClient.Height());

	m_bmpShowHide.DeleteObject();
	m_bmpShowHide.LoadBitmap(m_bFrameHidden ? IDB_DOWN_ARROW : IDB_UP_ARROW);
	m_ctlShowHideBtn.SetBitmap((HBITMAP)m_bmpShowHide);

	Invalidate();
}

void CMonitoringDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDragDropDialog::OnSize(nType, cx, cy);
	
	if(!m_ctlShowHideBtn.GetSafeHwnd())
		return;


	if(!m_bFrameHidden)
	{
		//Get size of List box
		CRect rcTasks;
		m_ctlTaskList.GetWindowRect(&rcTasks);
		ScreenToClient(&rcTasks);

		m_ctlTaskList.SetWindowPos(&wndTop, 0,0, cx/3, rcTasks.Height(), SWP_NOMOVE);
		m_ctlViewList.SetWindowPos(&wndTop, cx/3 + 10,rcTasks.top , cx/3, rcTasks.Height(), SWP_NOZORDER);
		m_ctlDisplayLabel.SetWindowPos(&wndTop, cx/3 + 10,rcTasks.top-15 , 0, 0, SWP_NOSIZE);
		//Fit Frame around list
		m_ctlExpandFrame.MoveWindow(0,0,cx,rcTasks.bottom+6);
		// Position the show/hide button
		m_ctlShowHideBtn.MoveWindow(cx-25,rcTasks.bottom-20,23,21);
	}
	else
	{
		//Fit Frame around list
		m_ctlExpandFrame.MoveWindow(0,0,cx,25);
		// Position the show/hide button
		m_ctlShowHideBtn.MoveWindow(cx-25,2,23,21);
	}

	if(m_ctlExpandFrame.GetSafeHwnd())
	{
		CRect rcFrame;
		m_ctlExpandFrame.GetWindowRect(&rcFrame);
		ScreenToClient(&rcFrame);

		if (m_pSummary != NULL)
		{
			m_pSummary->SetWindowPos(&wndTop, 5,rcFrame.bottom+10 , cx-10, cy-(rcFrame.bottom+10), SWP_NOZORDER);
		}
		else if (m_pActiveWnd != NULL)
		{
			m_pActiveWnd->SetWindowPos(&wndTop, 5,rcFrame.bottom+10 , cx-10, cy-(rcFrame.bottom+10), SWP_NOZORDER);
		}
	}

}

void CMonitoringDlg::OnSelchangeTaskList() 
{
	int nSel = m_ctlTaskList.GetCurSel();
	CString strSel;
	m_ctlTaskList.GetText(nSel, strSel);

	if (strSel == "Summary")
	{
		m_ctlViewList.ResetContent();
		if(m_pActiveWnd != NULL)
		{
			m_pActiveWnd->SendMessage(WM_DESTROY);
			delete m_pActiveWnd;
			m_pActiveWnd = NULL;
		}
		ShowSummary();
	}
	else
	{
		if (m_pSummary != NULL)
		{
			m_pSummary->SendMessage(WM_DESTROY);
			delete m_pSummary;
			m_pSummary = NULL;
		}

		for (int group = 0; group<m_pTestFile->GetNumTaskGroups(); group++)
		{
			if(m_pTestFile->GetTaskGroupName(group) == strSel)
			{
				LoadScreens(group);
				if (m_pTestFile->GetTaskType(group,0) == CTestFile::enProfile)
				{
					m_ctlViewList.ModifyStyleEx(0,LBS_MULTIPLESEL);
					for (int s = 0; s<m_ctlViewList.GetCount(); s++)
					{
						m_ctlViewList.SetSel(s, true);
					}
					OnSelchangeViewList();
				}
				else
				{
					m_ctlViewList.ModifyStyleEx(LBS_MULTIPLESEL,0);
					m_ctlViewList.SetSel(0, true);
					OnSelchangeViewList();
				}
				break;
			}
		}

	}
}

BOOL CMonitoringDlg::FindTestRunFolder()
{

	CFileFind FF;
	if(!FF.FindFile(m_strTestPath + "*.*"))
		return false;

	BOOL bContinue=true;
	CTime tmLast(0);
	while(bContinue)
	{
		bContinue = FF.FindNextFile();

		if(FF.IsDirectory() && !FF.IsDots() && !FF.IsHidden())
		{
			CTime tmFolder;
			FF.GetCreationTime(tmFolder);
			if (tmFolder > tmLast)
			{
				m_strRunFolder = FF.GetFilePath();
			}
		}
	}

	return !m_strRunFolder.IsEmpty();
}

void CMonitoringDlg::ShowSummary()
{
	if (!m_bTestActive)
		return;
	if (m_pSummary == NULL)
	{
		m_pSummary = new CMonitorSummary();
		m_pSummary->Init(m_strManagerReg, m_pTestFile);
		m_pSummary->Create(IDD_MON_SUMMARY, this);
		m_pSummary->ShowWindow(SW_SHOW);

		CRect rcWnd;
		GetClientRect(rcWnd);
		OnSize(SW_NORMAL, rcWnd.Width(), rcWnd.Height());
	}
}

void CMonitoringDlg::OnTimer(UINT nIDEvent) 
{
	IsActive();

	CDragDropDialog::OnTimer(nIDEvent);
}

void CMonitoringDlg::OnDestroy() 
{
	if (m_pSummary)
	{
		m_pSummary->SendMessage(WM_DESTROY);
		delete m_pSummary;
	}

	if (m_pActiveWnd)
	{
		m_pActiveWnd->SendMessage(WM_DESTROY);
		delete m_pActiveWnd;
	}
	CDragDropDialog::OnDestroy();
	KillTimer(ID_TESTFIND_TIMER);
}

void CMonitoringDlg::LoadScreens(int nGroup)
{
	m_strActiveTarget = m_pTestFile->GetTaskTargetType(nGroup,0);

	CStringArray arrPlugs;
	m_pPluginManager->ListPluginsByType(TEST_MONITOR_PLUGIN, arrPlugs);

	CStringArray arrScreens;
	for(int i =0; i<arrPlugs.GetSize(); i++)
	{
		CTestMonitorPlugin plug(arrPlugs.GetAt(i), m_pPluginManager);
		if(plug.IsSupportedExtension(m_strActiveTarget))
		{
			plug.GetAvailableScreens(m_pTestFile, nGroup, arrScreens);
		}
	}

	m_ctlViewList.ResetContent();
	for (int j = 0; j<arrScreens.GetSize(); j++)
	{
		m_ctlViewList.InsertString(j, arrScreens.GetAt(j));
	}
}

void CMonitoringDlg::OnSelchangeViewList() 
{
	// Find which screen(s) we want to view
	int nCount = m_ctlViewList.GetSelCount();
	if (nCount == 0)
	{
		return;
	}

	CArray<int,int> aryListBoxSel;

	aryListBoxSel.SetSize(nCount);
	m_ctlViewList.GetSelItems(nCount, aryListBoxSel.GetData());

	CStringArray arrScreens;
	for(int i =0; i<aryListBoxSel.GetSize(); i++)
	{
		CString strScreen;
		m_ctlViewList.GetText(aryListBoxSel.GetAt(i), strScreen);
		arrScreens.Add(strScreen);
	}

	// Find the plugin responsible for the screen and activate it
	CStringArray arrPlugs;
	m_pPluginManager->ListPluginsByType(TEST_MONITOR_PLUGIN, arrPlugs);

	for(int j =0; j<arrPlugs.GetSize(); j++)
	{
		CTestMonitorPlugin plug(arrPlugs.GetAt(j), m_pPluginManager);
		if(plug.IsSupportedExtension(m_strActiveTarget))
		{
			if (m_pActiveWnd != NULL)
			{
				m_pActiveWnd->SendMessage(WM_DESTROY);
				delete m_pActiveWnd;
				m_pActiveWnd = NULL;
			}

			m_pActiveWnd = plug.ViewScreen(m_strManagerReg, m_strRunFolder, arrScreens, this);
			if (m_pActiveWnd != NULL)
			{
				CRect rcClient;
				GetClientRect(rcClient);
				OnSize(SW_NORMAL,rcClient.Width(),rcClient.Height());
				break;
			}
		}
	}	
}

void CMonitoringDlg::UpdateTaskList()
{
	m_ctlTaskList.ResetContent();

	m_ctlTaskList.AddString("Summary");
	for (int i =0; i<m_pTestFile->GetNumTaskGroups(); i++)
	{
		m_ctlTaskList.InsertString(i+1,m_pTestFile->GetTaskGroupName(i));
	}
}

BOOL CMonitoringDlg::IsActive()
{
	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	CCORBAInfo* pInfo = pApp->GetInfo();

	CString strReg;
	if(pInfo && pInfo->IsTestActive(m_strTestName, strReg))
	{
			m_bTestActive = true;
			m_ctlTaskList.EnableWindow(true);
			m_ctlViewList.EnableWindow(true);

			m_strManagerReg = strReg;
			m_strManagerReg.TrimRight(" \r\n");
			FindTestRunFolder();
			
			if (m_pSummary == NULL && m_pActiveWnd == NULL)
			{
				m_ctlTaskList.SetCurSel(0);
				OnSelchangeTaskList();
			}
	}
	else
	{
			m_bTestActive = false;
			m_ctlTaskList.SetCurSel(-1);
			m_ctlViewList.SelItemRange(false, 0, m_ctlViewList.GetCount());
			m_ctlTaskList.EnableWindow(false);
			m_ctlViewList.EnableWindow(false);
			m_strManagerReg.Empty();
			if (m_pSummary!=NULL)
			{
				delete m_pSummary;
				m_pSummary = NULL;
			}

			if (m_pActiveWnd!=NULL)
			{
				m_pActiveWnd->SendMessage(WM_DESTROY);
				delete m_pActiveWnd;
				m_pActiveWnd = NULL;
			}
	}

	return false;
}
