/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ResultsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "ResultsDlg.h"
#include "TestFile.h"
#include "PluginTypes.h"
#include "TestResultsPlugin.h"
#include "XAuditViewer.h"
#include "DeleteRunDlg.h"
#include "OstaDefs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg dialog


CResultsDlg::CResultsDlg(CWnd* pParent /*=NULL*/)
	: CDragDropDialog(CResultsDlg::IDD, pParent),
	m_strAuditLogName("Test Audit Log"),
	m_strHistoryLogName("Test History Log"),
	m_strReportLogName("Test Report Log"),
	m_strSummaryName("Test Summary")
{
	//{{AFX_DATA_INIT(CResultsDlg)
	//}}AFX_DATA_INIT
	m_pTestFile = NULL;
	m_nLastGraphAdded=0;
	m_pCurrentGraph=NULL;
}


void CResultsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDragDropDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CResultsDlg)
	DDX_Control(pDX, IDC_GRAPH_TYPES, m_ctlGraphs);
	DDX_Control(pDX, IDC_RUNS, m_ctlRuns);
	DDX_Control(pDX, IDC_GRAPHS_CAPTION, m_ctlGraphsCaption);
	DDX_Control(pDX, IDC_RUNS_CAPTION, m_ctlRunsCaption);
	DDX_Control(pDX, IDC_FILTERS, m_ctlFilters);
	DDX_Control(pDX, IDC_LIST_FRAME, m_ctlListFrame);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CResultsDlg, CDragDropDialog)
	//{{AFX_MSG_MAP(CResultsDlg)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_FILTERS, OnFilters)
	ON_WM_DESTROY()
	ON_WM_SHOWWINDOW()
	ON_COMMAND(ID_DELETE_RUNS, OnDeleteRuns)
	ON_CBN_SELCHANGE(IDC_GRAPH_TYPES, OnSelchangeGraphTypes)
	ON_CBN_SELCHANGE(IDC_RUNS, OnSelchangeRuns)
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI(IDC_FILTERS,OnUpdateFilters)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg message handlers

BOOL CResultsDlg::OnInitDialog() 
{
	CDragDropDialog::OnInitDialog();

	m_ctlRuns.Refresh();
		
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CResultsDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDragDropDialog::OnSize(nType, cx, cy);
	
	if(!m_ctlRuns.GetSafeHwnd() || !m_ctlGraphs.GetSafeHwnd())
		return;

	int nWidth = cx;
	const int nSeparator = 5;

	//Get size of runs box
	CRect rcRuns;
	m_ctlRuns.GetWindowRect(&rcRuns);
	ScreenToClient(&rcRuns);

	//Fit Frame around 
	m_ctlListFrame.MoveWindow(0,0,cx,rcRuns.bottom+4);

	//Position filter button to right of window
	CRect rcFilter;
	m_ctlFilters.GetWindowRect(&rcFilter);
	ScreenToClient(&rcFilter);
	m_ctlFilters.SetWindowPos(NULL,cx-(rcFilter.Width()+nSeparator),4*nSeparator,0,0,SWP_NOZORDER|SWP_NOSIZE);
				
	int nAvailableWidth = cx - (rcFilter.Width()+2*nSeparator);
		
	//Position captions
	m_ctlRunsCaption.SetWindowPos(NULL,nSeparator,nSeparator,0,0,SWP_NOZORDER|SWP_NOSIZE);
	m_ctlGraphsCaption.SetWindowPos(NULL,(nAvailableWidth/2)+nSeparator,nSeparator,0,0,SWP_NOZORDER|SWP_NOSIZE);

	//Get size of caption
	CRect rcGraphCaption;
	m_ctlGraphsCaption.GetWindowRect(&rcGraphCaption);
	ScreenToClient(&rcGraphCaption);

	//Position List boxes
	m_ctlRuns.SetWindowPos(NULL,nSeparator,rcGraphCaption.bottom+nSeparator,(nAvailableWidth/2)-nSeparator,rcRuns.Height(),SWP_NOZORDER);
	m_ctlGraphs.SetWindowPos(NULL,(nAvailableWidth/2)+nSeparator,rcGraphCaption.bottom+nSeparator,(nAvailableWidth/2)-nSeparator,rcRuns.Height(),SWP_NOZORDER);

	CRect rcFrame;
	m_ctlListFrame.GetWindowRect(&rcFrame);
	ScreenToClient(&rcFrame);

	//Position chart below frame
	if(m_pCurrentGraph)
		m_pCurrentGraph->SetWindowPos(NULL,0,rcFrame.bottom+(nSeparator/2),cx,cy-rcFrame.bottom-(nSeparator/2),SWP_NOZORDER);
}

void CResultsDlg::OnFilters() 
{
	if(m_pCurrentGraph && !m_CurrentGraphType.strPlugin.IsEmpty())
	{
		CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
		CPluginManager *pPluginManager = pApp->GetPluginManager();
		
		CTestResultsPlugin plug(m_CurrentGraphType.strPlugin, pPluginManager);
		plug.ShowFilter(m_pCurrentGraph);
	}
}

void CResultsDlg::OnSelchangeGraphTypes() 
{
	if(m_pCurrentGraph)
	{
		delete m_pCurrentGraph;
		m_pCurrentGraph = NULL;
		m_CurrentGraphType.strGraphTitle.Empty();
	}

	int nCurSel = m_ctlGraphs.GetCurSel();
	if(nCurSel == LB_ERR)
		return;

	int nIndex = m_ctlGraphs.GetItemData(nCurSel);
	
	for(int nGraph=0;nGraph<m_arGraphs.GetSize();nGraph++)
	{
		GraphType &GT = m_arGraphs[nGraph];
		if(GT.nGraphIndex != nIndex)
			continue;

		CRect rcClient,rcList;
		GetClientRect(&rcClient);
		m_ctlRuns.GetWindowRect(&rcList);
		ScreenToClient(&rcList);
		rcClient.top = rcList.bottom + 10;

		m_CurrentGraphType = GT;
		if(GT.strPlugin.IsEmpty())
		{
			if(GT.strGraphTitle == m_strAuditLogName ||
			   GT.strGraphTitle == m_strHistoryLogName||
			   GT.strGraphTitle == m_strReportLogName)
			{
				CString strRun;
				if(!m_ctlRuns.GetCurrentRun(strRun))
					return;
				
				CXAuditViewer *pAudit = new CXAuditViewer;
				pAudit->Create(NULL,WS_CHILD|WS_VISIBLE,rcClient,this,0);
				pAudit->SetRunFolder(strRun);

				int nType = 0;
				if(GT.strGraphTitle == m_strHistoryLogName)
					nType = 1;
				else
				if(GT.strGraphTitle == m_strReportLogName)
					nType = 2;
				
				pAudit->SetLogType(nType);
				m_pCurrentGraph	= pAudit;
			}
			else
			if(GT.strGraphTitle == m_strSummaryName)
			{
				CString strRun;
				if(!m_ctlRuns.GetCurrentRun(strRun))
					return;
				
				CFile f;
				f.Open(strRun + strSummaryFile,CFile::modeRead);
				int nLength = f.GetLength();
				char *szBuffer = new char[nLength+1];
				f.Read(szBuffer,nLength);
				szBuffer[nLength] = NULL;
				f.Close();
				
				CEdit *pEdit = new CEdit;
				pEdit->Create(ES_READONLY|ES_MULTILINE|WS_VSCROLL|WS_HSCROLL|WS_CHILD|WS_VISIBLE|WS_BORDER,rcClient,this,0);
				pEdit->SetWindowText(szBuffer);
				pEdit->SetFont(GetFont());
				m_pCurrentGraph = pEdit;

				delete []szBuffer;
			}

			UpdateDialogControls(this,TRUE);
			return;
		}
		else
		{
			CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
			CPluginManager *pPluginManager = pApp->GetPluginManager();
			
			CTestResultsPlugin plug(GT.strPlugin, pPluginManager);

			m_pCurrentGraph = plug.DisplayGraph(GT.strGraphTitle,this,rcClient);
			if(m_pCurrentGraph)
				m_pCurrentGraph->ModifyStyle(0,WS_BORDER);

			Invalidate();
			UpdateDialogControls(this,TRUE);
			return;
		}
	}
}

CXChartCtrl* CResultsDlg::AddGraph()
{
	CXChartCtrl* pChart = new CXChartCtrl;
	pChart->Create(NULL,WS_CHILD|WS_VISIBLE,CRect(0,50,500,500),this,0);
	m_listCharts.AddTail(pChart);
	return pChart;
}

void CResultsDlg::SetTestFile(CTestFile* pFile)
{
	m_pTestFile = pFile;

	CString strTestFile(m_pTestFile->GetTestFilePath());
	int nPos = strTestFile.ReverseFind('\\');
	
	if(nPos!=-1)
	{
		m_strTestFolder = strTestFile.Left(nPos+1);
		m_ctlRuns.SetTestFolder(m_strTestFolder);
	}
}


void CResultsDlg::OnDestroy() 
{
	CDragDropDialog::OnDestroy();
	
	delete m_pCurrentGraph;
	m_pCurrentGraph = NULL;
}

BOOL CResultsDlg::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
	case WM_KEYDOWN:
		switch(pMsg->wParam)
		{
		case VK_F5:
			m_ctlRuns.Refresh();
		break;
		}
	break;
	}
	
	return CDragDropDialog::PreTranslateMessage(pMsg);
}

void CResultsDlg::OnUpdateFilters(CCmdUI *pCmdUI)
{
	if(m_CurrentGraphType.strPlugin.IsEmpty())
	{
		if(m_CurrentGraphType.strGraphTitle == m_strAuditLogName ||
		   m_CurrentGraphType.strGraphTitle == m_strHistoryLogName ||
		   m_CurrentGraphType.strGraphTitle == m_strReportLogName ||
		   m_CurrentGraphType.strGraphTitle == m_strSummaryName)
			pCmdUI->Enable(FALSE);
		return;
	}
	else
	{
		CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
		CPluginManager *pPluginManager = pApp->GetPluginManager();
		
		CTestResultsPlugin plug(m_CurrentGraphType.strPlugin, pPluginManager);

		pCmdUI->Enable( m_pCurrentGraph && plug.DoesGraphHaveFilters(m_pCurrentGraph));
		return;
	}
}

void CResultsDlg::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDragDropDialog::OnShowWindow(bShow, nStatus);
	if(bShow)
	{
		m_ctlRuns.Refresh();
		m_ctlRuns.ShowDropDown();
	}
}

void CResultsDlg::OnDeleteRuns() 
{
	CDeleteRunDlg dlg(m_strTestFolder);
	if(dlg.DoModal()==IDOK)
		m_ctlRuns.Refresh();
}

void CResultsDlg::OnSelchangeRuns() 
{
	if(m_pCurrentGraph)
	{
		delete m_pCurrentGraph;
		m_pCurrentGraph = NULL;
		m_CurrentGraphType.strGraphTitle.Empty();
	}
	
	CString strRun;
	if(!m_ctlRuns.GetCurrentRun(strRun))
		return;
	strRun.TrimRight('\\');

	//Load the selected run
	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	CPluginManager *pPluginManager = pApp->GetPluginManager();

	m_arGraphs.RemoveAll();
	m_nLastGraphAdded = 0;

	GraphType GT;
	GT.strPlugin.Empty();
	CFileStatus fs;

	if(CFile::GetStatus(strRun + "\\" + strSummaryFile,fs))
	{
		GT.strGraphTitle = m_strSummaryName;
		GT.nGraphIndex = m_nLastGraphAdded++;
		m_arGraphs.Add(GT);
	}
	
	if(CFile::GetStatus(strRun + "\\TestLog.txt",fs))
	{
		GT.strGraphTitle = m_strAuditLogName;
		GT.nGraphIndex = m_nLastGraphAdded++;
		m_arGraphs.Add(GT);
	}

	if(CFile::GetStatus(strRun + "\\TestRep.txt",fs))
	{
		GT.strGraphTitle = m_strReportLogName;
		GT.nGraphIndex = m_nLastGraphAdded++;
		m_arGraphs.Add(GT);
	}

	if(CFile::GetStatus(strRun + "\\TestHis.txt",fs))
	{
		GT.strGraphTitle = m_strHistoryLogName;
		GT.nGraphIndex = m_nLastGraphAdded++;
		m_arGraphs.Add(GT);
	}
	
	CStringArray arResultsPlugIns;
	pPluginManager->ListPluginsByType(TEST_RESULTS_PLUGIN, arResultsPlugIns);
	
	for (int nPlugIn = 0; nPlugIn<arResultsPlugIns.GetSize();nPlugIn++)
	{
		CTestResultsPlugin plug(arResultsPlugIns[nPlugIn], pPluginManager);
		plug.LoadRun(strRun);

		//Get the graphs which are now available
		CStringArray arstrTypes;
		plug.GetAvailableGraphs(arstrTypes);

		GT.strPlugin = arResultsPlugIns[nPlugIn];
		for(int nNdx=0;nNdx<arstrTypes.GetSize();nNdx++)
		{
			GT.strGraphTitle = arstrTypes[nNdx];
			GT.nGraphIndex = m_nLastGraphAdded++;
			m_arGraphs.Add(GT);
		}
	}

	m_ctlGraphs.SetRedraw(FALSE);
	m_ctlGraphs.ResetContent();

	for(int nNdx=0;nNdx<m_arGraphs.GetSize();nNdx++)
	{
		int nItem = m_ctlGraphs.AddString(m_arGraphs[nNdx].strGraphTitle);
		m_ctlGraphs.SetItemData(nItem,m_arGraphs[nNdx].nGraphIndex);
	}
	
	m_ctlGraphs.SetRedraw();
	m_ctlGraphs.ShowDropDown();
}

