/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TaskGroupDefDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "TaskGroupDefDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTaskGroupDefDlg dialog


CTaskGroupDefDlg::CTaskGroupDefDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTaskGroupDefDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTaskGroupDefDlg)
	m_nStatLevel = -1;
	m_nStartInterval = 0;
	m_nNumVUs = 0;
	m_nVUPerInterval = 0;
	m_bGenPageTimers = FALSE;
	//}}AFX_DATA_INIT
}


void CTaskGroupDefDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTaskGroupDefDlg)
	DDX_Control(pDX, IDC_VUS_SPIN, m_ctlVUsSpin);
	DDX_Control(pDX, IDC_VU_PER_INTERVAL_SPIN, m_ctlVUPerIntSpin);
	DDX_Control(pDX, IDC_STAT_LEVEL, m_ctlStatLevel);
	DDX_Control(pDX, IDC_START_INTERVAL_SPIN, m_ctlStartIntervalSpin);
	DDX_CBIndex(pDX, IDC_STAT_LEVEL, m_nStatLevel);
	DDX_Text(pDX, IDC_START_INTERVAL, m_nStartInterval);
	DDX_Text(pDX, IDC_VIRTUAL_USERS, m_nNumVUs);
	DDX_Text(pDX, IDC_VU_PER_INTERVAL, m_nVUPerInterval);
	DDX_Check(pDX, IDC_GEN_PAGE_TIMERS, m_bGenPageTimers);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTaskGroupDefDlg, CDialog)
	//{{AFX_MSG_MAP(CTaskGroupDefDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTaskGroupDefDlg message handlers

BOOL CTaskGroupDefDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
    const int nNumAccel=4;
    UDACCEL Accel[nNumAccel];
    Accel[0].nSec=1;
    Accel[0].nInc=1;
    Accel[1].nSec=2;
    Accel[1].nInc=10;
    Accel[2].nSec=3;
    Accel[2].nInc=50;
    Accel[3].nSec=4;
    Accel[3].nInc=200;

	m_ctlVUsSpin.SetRange(1,UD_MAXVAL);
    m_ctlVUsSpin.SetAccel(nNumAccel,Accel);
	m_ctlVUsSpin.SetBuddy(GetDlgItem(IDC_VIRTUAL_USERS));

	m_ctlVUPerIntSpin.SetRange(1,UD_MAXVAL);
    m_ctlVUPerIntSpin.SetAccel(nNumAccel,Accel);
    m_ctlVUPerIntSpin.SetBuddy(GetDlgItem(IDC_VU_PER_INTERVAL));

	m_ctlStartIntervalSpin.SetRange(0,UD_MAXVAL);
    m_ctlStartIntervalSpin.SetAccel(nNumAccel,Accel);
    m_ctlStartIntervalSpin.SetBuddy(GetDlgItem(IDC_START_INTERVAL));

	m_ctlStatLevel.AddString("None");
	m_ctlStatLevel.AddString("Low");
	m_ctlStatLevel.AddString("High");
	m_ctlStatLevel.SetCurSel(m_nStatLevel);

	UpdateData(FALSE);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CTaskGroupDefDlg::OnOK() 
{
	UpdateData();

	m_TaskGroupData.SetStartInterval(m_nStartInterval);
	m_TaskGroupData.SetNumVUs(m_nNumVUs);
	m_TaskGroupData.SetNumVUPerInterval(m_nVUPerInterval);
	m_TaskGroupData.SetUsePageTimers(m_bGenPageTimers);

	switch(m_nStatLevel)
	{
	case 0:
		m_TaskGroupData.SetStatLevel('N');
	break;
	case 1:
		m_TaskGroupData.SetStatLevel('L');
	break;
	case 2:
		m_TaskGroupData.SetStatLevel('H');
	break;
	}

	
	CDialog::OnOK();
}

void CTaskGroupDefDlg::LoadFromBuffer(LPBYTE pData)
{
	m_TaskGroupData.LoadFromBuffer(pData);

	m_nStartInterval = m_TaskGroupData.GetStartInterval();
	m_nNumVUs = m_TaskGroupData.GetNumVUs();
	m_nVUPerInterval = m_TaskGroupData.GetNumVUPerInterval();
	m_bGenPageTimers = m_TaskGroupData.GetUsePageTimers();

	switch(m_TaskGroupData.GetStatLevel())
	{
	case 'N':
		m_nStatLevel = 0;
	break;
	case 'L':
		m_nStatLevel = 1;
	break;
	case 'H':
		m_nStatLevel = 2;
	break;
	}
}

BOOL CTaskGroupDefDlg::SaveToBuffer(LPBYTE pData,DWORD& dwBufferSize)
{
	return m_TaskGroupData.SaveToBuffer(pData,dwBufferSize);
}
