/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestFile.h: interface for the CTestFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TESTFILE_H__A79E82F3_D599_11D4_94C7_00C04F89E392__INCLUDED_)
#define AFX_TESTFILE_H__A79E82F3_D599_11D4_94C7_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CTestFile : public CObject
{
public:
	CTestFile();
	virtual ~CTestFile();

	typedef enum{enProfile,enScript}TaskType;

	void Serialize(CArchive& ar);

	CString GetTestFilePath();
	void SetTestFilePath(const CString& strPath);

	CString GetDescription();
	void SetDescription(const CString& strDescription);

	void Clear();
	
	int AddNewTaskGroup();
	BOOL SetTaskGroupHost(int nGroup,const CString& strHost);
	int GetNumTaskGroups();
	BOOL GetTaskGroupData(int nGroup,LPBYTE* pData,DWORD &dwBufSize);
	BOOL SetTaskGroupData(int nGroup,LPBYTE pData,DWORD dwDataSize);
	BOOL SetTaskGroupName(int nGroup, const CString& strName);
	CString GetTaskGroupName(int nGroup);
	
	BOOL SetTaskData(int nGroup,int nTask,LPBYTE pData,DWORD dwDataSize);
	int GetNumTasks(int nGroup);
	CString GetTaskGroupHost(int nGroup);

	BOOL SetTask(int nGroup,int nTask,TaskType nType,const CString& strName,
					const CString& strPath,const CString& strTarget);
	int AddNewTask(int nGroup,TaskType nType,const CString& strName,
					const CString& strPath,const CString& strTarget);

	CString GetTaskPath(int nGroup,int nTask);
	TaskType GetTaskType(int nGroup,int nTask);
	CString GetTaskName(int nGroup,int nTask);
	BOOL GetTaskData(int nGroup,int nTask,LPBYTE* pData,DWORD& dwBufSize);
	CString GetTaskTargetType(int nGroup,int nTask);

	void DeleteItem(int nGroup,int nTask);
	void DeleteTaskGroup(int nGroup);

	void GetSummary(CString& strSummary);
private:
	CString m_strFilePath;
	CString m_strTestName;
	CString m_strDescription;

	typedef struct{
		TaskType nType;
		CString strName;
		CString strPath;
		CString strTarget;
		DWORD dwDataSize;
		LPBYTE pData;
	}Task;

	typedef struct{
		CString strHost;
		CString strName;
		DWORD dwDataSize;
		LPBYTE pData;
		CArray <Task*,Task*> m_arTasks;
	}TaskGroup;

	CArray <TaskGroup*,TaskGroup*> m_arGroups;

	void ClearArrays();
	CString GetRelativePath(const CString& strPath);
};

#endif // !defined(AFX_TESTFILE_H__A79E82F3_D599_11D4_94C7_00C04F89E392__INCLUDED_)
