/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestResultsPlugin.cpp: implementation of the CTestResultsPlugin class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TestPlugin.h"
#include "PluginTypes.h"
#include "TestResultsPlugin.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

typedef void ( * LPLOADRUNFUNC)  (const CString& strRunFolder);
typedef void ( * LPGETGRAPHSFUNC)  (CStringArray& arstrGraphs);
typedef CWnd* ( * LPDISPLAYGRAPHFUNC)  (const CString& strGraph,CWnd *pParent,const CRect& rcBounds);
typedef BOOL ( * LPHASFILTERSFUNC)  (CWnd *pGraph);
typedef void ( * LPSHOWFILTERFUNC)  (CWnd *pGraph);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTestResultsPlugin::CTestResultsPlugin(CString szTitle,CPluginManager* pPluginManager)
{
	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	m_pPlugin = pPluginManager->GetPlugin(TEST_RESULTS_PLUGIN, szTitle);
}

CTestResultsPlugin::~CTestResultsPlugin()
{

}

void CTestResultsPlugin::LoadRun(const CString& strRunFolder)
{
	LPLOADRUNFUNC func = (LPLOADRUNFUNC)m_pPlugin->GetFunction("LoadRun");
	if (!func)
		return;

	func(strRunFolder);
}

void CTestResultsPlugin::GetAvailableGraphs(CStringArray& arstrGraphs)
{
	LPGETGRAPHSFUNC func = (LPGETGRAPHSFUNC)m_pPlugin->GetFunction("GetAvailableGraphs");
	if (!func)
		return;

	func(arstrGraphs);
}

CWnd* CTestResultsPlugin::DisplayGraph(const CString& strGraph,CWnd* pParent,const CRect& rcBounds)
{
	LPDISPLAYGRAPHFUNC func = (LPDISPLAYGRAPHFUNC)m_pPlugin->GetFunction("DisplayGraph");  
	if(!func)
		return NULL;
	
	return func(strGraph,pParent,rcBounds);	
}

BOOL CTestResultsPlugin::DoesGraphHaveFilters(CWnd *pGraph)
{
	LPHASFILTERSFUNC func = (LPHASFILTERSFUNC)m_pPlugin->GetFunction("DoesGraphHaveFilter");  
	if(!func)
		return NULL;
	
	return func(pGraph);	
}

void CTestResultsPlugin::ShowFilter(CWnd* pGraph)
{
	LPSHOWFILTERFUNC func = (LPSHOWFILTERFUNC)m_pPlugin->GetFunction("ShowFilter");  
	if(!func)
		return;
	
	func(pGraph);	
}
