/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "ChangeDialog.h"
#include "LevelList.h"
#include "resource.h"
#include <CyrException.h>
#include <Logging\CyrLog.h>
#include "CECSimpleParse.h"

#define NUM_COLS 3

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC( CLevelList, CListCtrl )

/////////////////////////////////////////////////////////////////////////////
// CLevelList

CLevelList::CLevelList()
{}


CLevelList::~CLevelList()
{}



BEGIN_MESSAGE_MAP(CLevelList, CListCtrl)
ON_WM_CONTEXTMENU()
//{{AFX_MSG_MAP(CLevelList)
ON_NOTIFY_REFLECT(NM_CLICK, OnClick)
ON_WM_LBUTTONDBLCLK()
ON_COMMAND(ID_POPUP_ADD, OnPopupAdd)
ON_COMMAND(ID_POPUP_DELETE, OnPopupDelete)
ON_COMMAND(ID_POPUP_EDIT, OnPopupEdit)
ON_COMMAND(ID_POPUP_IMPORT, OnPopupImport)
ON_COMMAND(ID_POPUP_EXPORT, OnPopupExport)
ON_COMMAND(ID_POPUP_REFRESH, OnPopupRefresh)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////
// CLevelList message handlers

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is overriden to provide custom drawing of the items
//          within the list.  The custom handler draws a selection rectangle
//          around the entire item and not just the left-hand column.  See the
//          Visual C++ documentation for more details.
void CLevelList::DrawItem( LPDRAWITEMSTRUCT lpDrawItemStruct )
{
    // Get rectangle for item and DC to draw on
    CRect rect;
    GetItemRect( lpDrawItemStruct->itemID, &rect, LVIR_BOUNDS );
    CDC* pDC = CDC::FromHandle( lpDrawItemStruct->hDC );

    // We draw item differently if selected so set up colours
    if ( lpDrawItemStruct->itemState & ODS_SELECTED )
    {
        pDC->SetTextColor( GetSysColor( COLOR_HIGHLIGHTTEXT ));
        pDC->SetBkColor( GetSysColor( COLOR_HIGHLIGHT ));
    }
    else
    {
        pDC->SetTextColor( GetSysColor( COLOR_WINDOWTEXT ));
        pDC->SetBkColor( GetSysColor( COLOR_WINDOW ));
    }

    // Draw text into all columns
    CString strText;
    for ( int i = 0; i < NUM_COLS; i++ )
    {
        // Get text to draw
        strText = GetItemText( lpDrawItemStruct->itemID, i );

        // Calculate dimensions of rectangle
        LVCOLUMN lvCol;
        lvCol.mask = LVCF_FMT | LVCF_WIDTH;
        GetColumn( i, &lvCol );
        rect.right = rect.left + lvCol.cx;
        if ( i == 0 )
        {
            // Take into account drawn circle
            rect.left += rect.Height();
        }

        // Specify text position based on alignment property of column
        int x = rect.left + 4;
        int y = rect.top + 1;
        if ( lvCol.fmt & LVCFMT_RIGHT )
        {
            CSize cSize = pDC->GetOutputTextExtent( strText );
            x = rect.right - cSize.cx - 4;
        }

        // Draw text into specified position
        pDC->ExtTextOut( x, y, ETO_CLIPPED | ETO_OPAQUE, &rect, strText, NULL );

        // Update rectangle
        rect.left = rect.right;
    }

    // Draw square background for icon
    GetItemRect( lpDrawItemStruct->itemID, &rect, LVIR_BOUNDS );
    pDC->FillSolidRect( rect.left, rect.top,
                        rect.left + rect.Height(),
                        rect.top + rect.Height(),
                        pDC->GetBkColor() );

    // Determine if current log entry is active or not and depending on
    // status create a brush of the correct colour.
    CBrush newBrush;
    strText = GetItemText( lpDrawItemStruct->itemID, 0 );
    bool bActive = GetStatus( strText );
    if ( bActive == true )
        newBrush.CreateSolidBrush( RGB( 0, 128, 0 ));
    else
        newBrush.CreateSolidBrush( RGB( 250, 0, 0 ));

    // Select brush into DC and remember old brush
    CBrush* oldBrush = pDC->SelectObject( &newBrush );

    // Draw circle showing whether log is active or not
    pDC->Ellipse( rect.left + 2, rect.top + 2,
                  rect.left + rect.Height() - 2,
                  rect.top + rect.Height() - 2 );

    // Reselect old brush back into DC
    pDC->SelectObject( oldBrush );

    // If we changed the colours for a selected item undo them
    if ( lpDrawItemStruct->itemState & ODS_SELECTED )
    {
        // Set the text background and foreground colors
        pDC->SetTextColor( GetSysColor( COLOR_WINDOWTEXT ));
        pDC->SetBkColor( GetSysColor( COLOR_WINDOW ));
    }

    // If the item has focus, draw a focus rectangle around the entire row
    if( lpDrawItemStruct->itemState & ODS_FOCUS )
        pDC->DrawFocusRect( &rect );
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is overridden so that a click anywhere in the area of an
//          item will select it.  Without this override the user would have to
//          select an area surrounding only the left-hand entry. See the Visual
//          C++ documentation for more details.
void CLevelList::OnClick(NMHDR* pNMHDR, LRESULT* pResult)
{
    // Get the current mouse location and convert it to client coordinates.
    DWORD pos = GetMessagePos();
    CPoint pt( LOWORD(pos), HIWORD(pos) );
    ScreenToClient( &pt );

    // Get indexes of the first and last visible items in listview control.
    int start = GetTopIndex();
    int stop = start + GetCountPerPage();
    if ( stop > GetItemCount())
        stop = GetItemCount();

    // Loop until selected item is found or end of list is reached
    for ( int i = start; i < stop; i++ )
    {
        // Get the bounding rectangle of an item. If the mouse
        // location is within the bounding rectangle of the item,
        // we know we've found the item that was being clicked on
        CRect rect;
        GetItemRect( i, &rect, LVIR_BOUNDS );
        if ( rect.PtInRect( pt ))
        {
            UINT flag = LVIS_SELECTED | LVIS_FOCUSED;
            SetItemState( i, flag, flag );
            break;
        }
    }

    *pResult = 0;
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method produces a context menu in the list control.  This code
//          was added by the Visual C++ wizard for insertion of common code.
//          The only changes were the removal of the message routing code (to
//          prevent menu events being sent to the parent dialog) and the addition
//          of some code to enable/disable particular entries if no item in
//          the list control is selcetd.
void CLevelList::OnContextMenu(CWnd*, CPoint point)
{
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;
    VERIFY(menu.LoadMenu(CG_IDR_POPUP_LEVEL_LIST));

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    // Determine if we have clicked on an entry.
    int pos = GetNextItem( -1, LVNI_SELECTED );
    if ( pos == -1 )
    {
        pPopup->EnableMenuItem( ID_POPUP_EDIT, MF_DISABLED | MF_GRAYED );
        pPopup->EnableMenuItem( ID_POPUP_DELETE, MF_DISABLED | MF_GRAYED );
    }

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method adds a single trace setting entry to the list control.
//          If the entry already exists then it is simply updated.
// @parm    The name of the category to add/update.
// @parm    The level to set the category to.
// @parm    Determines if the status of the log is currently active or not.
void CLevelList::AddLevel( const char* category, int level, bool active )
{
    // List control has been modified
    SetDirtyFlag( true );

    // Build list control entry
    LVITEM lvItem;
    lvItem.mask = LVIF_TEXT;
    lvItem.iItem = GetItemCount();
    lvItem.iSubItem = 0;
    lvItem.pszText = (char*)category;

    // Determine if entry is already in list control
    LVFINDINFO lvFind;
    lvFind.flags = LVFI_STRING;
    lvFind.psz = lvItem.pszText;
    lvItem.iItem = FindItem( &lvFind );

    // If we have not found main item add it
    if ( lvItem.iItem == -1 )
        lvItem.iItem = InsertItem( &lvItem );

    // Add level details
    CString strLevel;

    // Translate the numeric value into a named level
    ConvertLevel(level, strLevel);

    lvItem.iSubItem = 1;
    lvItem.pszText = strLevel.GetBuffer(0);
    SetItem( &lvItem );
    strLevel.ReleaseBuffer();

    // Add status entry
    CString strStatus;
    if ( active == true )
        strStatus.LoadString( IDS_STATUS_ACTIVE );
    else
        strStatus.LoadString( IDS_STATUS_INACTIVE );
    lvItem.iSubItem = 2;
    lvItem.pszText = strStatus.GetBuffer(0);
    SetItem( &lvItem );
    strStatus.ReleaseBuffer();
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the current status of the list control entries.
// @rdesc   True if entries have been modified, else false.
bool CLevelList::GetDirtyFlag()
{
    return m_bDirty;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method sets the current status of the list control entries.
// @rdesc   The previous status.
// @parm    The new status for the entries.
bool CLevelList::SetDirtyFlag(bool bDirty)
{
    bool bOld = m_bDirty;
    m_bDirty = bDirty;
    return bOld;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   If the user double clicks in the list control then the edit dialog
//          will be shown.  If an entry is double-clicked on then the dialog
//          will be populated with the values of the entry.
void CLevelList::OnLButtonDblClk(UINT nFlags, CPoint point)
{
    // Perform same action as selecting add from menu
    CreateLevel( false );

    CListCtrl::OnLButtonDblClk(nFlags, point);
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method inserts a column into the list control at the specified
//          position.  The format of this oclumn is taken from the string entry
//          in the resource file.
// @parm    The position that the column will be inserted into.
// @parm    The resource ID of the string describing the column.
void CLevelList::CreateColumn(int pos, UINT uID)
{
    // Get column header information from resources
    CString strInfo;
    strInfo.LoadString( uID );

    // Extract column information from string
    CString strText = strtok(strInfo.GetBuffer(0), "," );              // Locate text for column header
    int nFormat = atol( strtok(strInfo.GetBuffer(0), "," ) );      // Locate format of text
    int nSize = atol( strtok(strInfo.GetBuffer(0), "," ) ) + 10;  // Locate default size of column

    // Add column to list control
    InsertColumn( pos, strText, nFormat,  nSize );
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method attempts to create a completely new trace category.
void CLevelList::OnPopupAdd()
{
    // Create a new entry with default properties
    CreateLevel( true );
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method attempts to delete a trace category.
void CLevelList::OnPopupDelete()
{
    DeleteLevel();
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method attempts to edit an existing trace category.
void CLevelList::OnPopupEdit()
{
    // Create new entry and pick up details from selected item
    CreateLevel( false );
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method displays the edit dialog box for a category and its
//          level.  If the passed parameter is false then the details are picked
//          from the currently selected entry.
// @rdesc   Returns true if the category is added or changed.
// @parm    If true then a completely new entry will be generated.
bool CLevelList::CreateLevel(bool bNew)
{
    bool bActive = false;

    // Determine if we have clicked on an entry.
    int pos = GetNextItem( -1, LVNI_SELECTED );
    if (( pos != -1 ) && ( bNew == false ))
    {
        CChangeDialog dlg;

        // If we have clicked on an entry then populate change
        // dialog with the correct entries.
        dlg.m_strCategory = GetItemText( pos, 0 );
        dlg.m_szLevel = GetItemText( pos, 1 );
        bActive = GetStatus( dlg.m_strCategory );

        // Show change dialog
        if ( dlg.DoModal() == IDOK )
        {

            if (dlg.m_bAllLevels)
            {
                // Add/adjust ALL levels in main list control
                for ( int i = 0; i < GetItemCount(); i++ )
                {
                    CString strCat = GetItemText( i, 0 );
                    AddLevel( strCat, ConvertLevel(dlg.m_szLevel), bActive );
                }
            }
            else
            {
                // Add/adjust level in main list control
                AddLevel( dlg.m_strCategory, ConvertLevel(dlg.m_szLevel), bActive );
            }
            return true;
        }
    }
    else
    {
        CAddDialog dlg;

        // Show change dialog
        if ( dlg.DoModal() == IDOK )
        {
            // Add/adjust level in main list control
            AddLevel( dlg.m_strCategory, ConvertLevel(dlg.m_szLevel), bActive );
            return true;
        }
    }
    return false;
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method deletes the currently selected trace category from the
//          list control.  The change is not permanent until the settings are
//          saved.
// @rdesc   Returns true if the item was removed.
bool CLevelList::DeleteLevel()
{
    // Determine if we have clicked on an entry.
    int pos = GetNextItem( -1, LVNI_SELECTED );
    if ( pos != -1 )
    {
        // Delete selected item
        DeleteItem( pos );

        // Have deleted an entry so list is now dirty
        SetDirtyFlag( true );
        return true;
    }

    return false;
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method imports a collection of trace settings from a specified
//          file.
// @parm    The fully qualified path name of the file.
void CLevelList::ImportFile( const char* filename )
{
    // Display wait cursor
    CWaitCursor cursor;

    try
    {
        // Open file and read contents
        CStdioFile inFile( filename,
                           CFile::modeRead | CFile::shareDenyNone | CFile::typeText );

        CString strEntry;
        inFile.ReadString( strEntry );

        // Parse details from file

        // First we tokenize the file entry

        CString strCategory;
        int level;

        // Then we advance along the tokenized list until
        // we are out of categories
        strCategory = strtok(strEntry.GetBuffer(0), "|" );
        while(!strCategory.IsEmpty())
        {
            // If we have a category then we advance one token to collect
            // the level set for this category
            level = atol( strtok(NULL, "|" ) );

            // And add the entry to the list control
            AddLevel( (LPCTSTR)strCategory, level, false );


            // Get the next Entry
            strCategory = strtok(NULL, "|" );

        }

    }
    catch( CFileException )
    {}

}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method exports the current list of trace categories and
//          settings to the specified file.
// @parm    The fully qualified path name of the file.
void CLevelList::ExportFile( const char* filename )
{
    // Display wait cursor
    CWaitCursor cursor;

    try
    {
        // Open text file to store trace settings
        CStdioFile outFile( filename,
                            CFile::modeWrite | CFile::modeCreate | CFile::shareExclusive );

        CString strOut = "";

        // Write contents of list control to it
        for( int i = 0; i < GetItemCount(); i++ )
        {
            for ( int j = 0; j < NUM_COLS - 1; j++ )
            {
                // Convert the levels field
                if (j==1)
                {
                    CString szLevel;
                    szLevel.Format("%d",ConvertLevel(GetItemText(i,j)));
                    strOut += szLevel;
                }
                else
                {
                    strOut += GetItemText( i, j );
                }
                strOut += SEPARATOR;
            }
        }

        outFile.WriteString( strOut );
    }
    catch( CFileException )
    {}

}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method produces an open file dialog and allows the user to
//          import the file into the current category list.
void CLevelList::OnPopupImport()
{
    // Load file list filter from resources
    CString filter;
    filter.LoadString( IDS_FILE_FILTER );

    // Create and show open file dialog
    CFileDialog dlg( TRUE, NULL, LOG_SETTINGS_FILE,
                     OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                     filter );
    if ( dlg.DoModal() == IDOK )
        ImportFile( dlg.GetPathName() );
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method produces a save file dialog and allows the user to
//          export the current category list to a file.
void CLevelList::OnPopupExport()
{
    // Load file list filter from resources
    CString filter;
    filter.LoadString( IDS_FILE_FILTER );

    // Create and show open file dialog
    CFileDialog dlg( FALSE, NULL, LOG_SETTINGS_FILE,
                     OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                     filter );
    if ( dlg.DoModal() == IDOK )
        ExportFile( dlg.GetPathName() );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method locates a particular category in the list control and
//          returns its index.
// @rdesc   The index of the requested entry.
// @parm    The category to searh for.
int CLevelList::FindCategory( const char* category )
{
    // Loop through all items looking for category
    for( int i = 0; i < GetItemCount(); i++ )
    {
        CString str = GetItemText( i, 0 );
        if ( str == category )
            return i;
    }

    return -1;
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the current status of a particular category.
// @rdesc   True if the category is currently active, or false if it isn't.
// @parm    The category to searh for.
bool CLevelList::GetStatus( const char* category )
{
    int index = FindCategory( category );
    if ( index != -1 )
    {
        CString text;
        text.LoadString( IDS_STATUS_ACTIVE );
        CString status = GetItemText( index, 2 );
        return ( status == text );
    }

    return false;
}

////////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the value of the requested trace setting.
// @rdesc   The value of the current trace level.
// @parm    The category to searh for.
int CLevelList::GetLevel( const char* category )
{
    int index = FindCategory( category );
    if ( index != -1 )
    {
        return ConvertLevel(GetItemText( index, 1 ));
    }

    return 0;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method loops through all currently active logs in the system.
//          Any entries in the list that match an entry in the log has its
//          value updated.
// @rdesc   Returns true if there were no errors during import.
bool CLevelList::ImportCorba()
{
    // Display wait cursor
    CWaitCursor cursor;
    bool bError = false;

    // Get the current dirty status.  This will be restored once the following
    // updates have been carried out.
    bool bDirty = m_bDirty;

    // Reset all levels to inactive until later
    for ( int i = 0; i < GetItemCount(); i++ )
    {
        CString strCat = GetItemText( i, 0 );
        int level = GetLevel( strCat );
        AddLevel( strCat, level, false );
    }

    try
    {
        // Get a list of all the logs available on the network
        TRACE( "Retrieving Factory list.\n" );
		CORBAServer::NameList_t ItemList=m_corba->GetListOfNames(CyrLog::CLASS_NAME);

        // Tokenize list and get first entry in list
        if ( ItemList.size() == 0 )
            bError = true;

        // Loop through all possible logs looking for the current set of entries
		for (std::vector<std::string>::size_type iter=0;iter<ItemList.size();iter++)
        {
			CString logName = ItemList[iter].Id.c_str();

            // Get pointer to log object
            TRACE( "Connecting to Log: %s\n", logName );
            CORBA::Object_ptr obj = m_corba->FindObjectWithName( CyrLog::CLASS_NAME, logName.GetBuffer(0) );
            CyrLog_var log = CyrLog::_narrow(obj );
            if ( !CORBA::is_nil( log ))
            {
                TraceCat_var cats = log->GetTraceCategories();
                for ( int i = 0; i < cats->length(); i++ )
                {
                    // Determine if category exists within list control.  If not then
                    // we ignore it as the user is not interested in this logging type.
//                    TRACE( "%s\n", cats[i] );
                    if ( FindCategory( cats[i] ) != -1 )
                    {
                        // Update entry
                        int level = log->GetLevel( cats[i] );
                        AddLevel( cats[i], level, true );
                    }
                }
            }
        }
    }
    catch( CyrCORBAException ex )
    {
#ifdef _DEBUG
        CString strError;
        strError.LoadString( IDS_CYRANO_EXCEPTION );
        TRACE( strError, (LPCTSTR)ex.what().c_str() );
#endif
        bError = true;
    }
    catch( CORBA::Exception& ex )
    {
#ifdef _DEBUG
        CString strError;
        strError.LoadString( IDS_CORBA_EXCEPTION );
        TRACE( strError, (LPCTSTR)CorbaExceptionString( ex ) );
#endif
        bError = true;
    }

    // Reset dirty flag and redraw list control
    m_bDirty = bDirty;
    Invalidate();
    return !bError;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method exports the current log settings to all active logs on
//          the network.  It connects to the NameService, retrieves all
//          registered log objects and sets each entry in the list to the
//          specified value.
// @rdesc   Returns true if there were no erros during export
bool CLevelList::ExportCorba()
{
    // Display wait cursor
    CWaitCursor cursor;

    try
    {
        // Get a list of all the logs available on the network
        TRACE( "Retrieving Factory list.\n" );
		CORBAServer::NameList_t ItemList=m_corba->GetListOfNames(CyrLog::CLASS_NAME);

        // Tokenize list and get first entry in list
        if ( ItemList.size() == 0 )
            return false;

        // Loop through all possible logs looking for the current set of entries
		for (std::vector<std::string>::size_type iter=0;iter<ItemList.size();iter++)
        {
			CString logName = ItemList[iter].Id.c_str();
            // Get pointer to log object
            TRACE( "Connecting to Log: %s\n", logName );
            CORBA::Object_ptr obj = m_corba->FindObjectWithName( CyrLog::CLASS_NAME, logName );
            CyrLog_var log = CyrLog::_narrow( obj );
            if ( !CORBA::is_nil( log ))
            {
                for ( int i = 0; i < GetItemCount(); i++ )
                {
                    // Get category and level
                    CString strCat = GetItemText( i, 0 );
                    int level = GetLevel( strCat );

                    log->SetLevel( strCat, level );
                    AddLevel( strCat, level, true );
                }
            }
        }
    }
    catch( CyrCORBAException ex )
    {
#ifdef _DEBUG
        CString strError;
        strError.LoadString( IDS_CYRANO_EXCEPTION );
        TRACE( strError, (LPCTSTR)ex.what().c_str() );
#endif
    }
    catch( CORBA::Exception& ex )
    {
#ifdef _DEBUG
        CString strError;
        strError.LoadString( IDS_CORBA_EXCEPTION );
        TRACE( strError, (LPCTSTR)CorbaExceptionString( ex ) );
#endif
    }

    Invalidate();
    return true;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method sets the CORBA helper class for the list control.  This
//          class is used to locate logs on the network.
// @parm    Pointer to the helper class.
void CLevelList::SetCorbaHelper( CORBAServer* pCorba )
{
    m_corba = pCorba;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is called when the 'Refresh' entry is selected on the
//          popup context menu.
void CLevelList::OnPopupRefresh()
{
    if ( ImportCorba() == false )
    {
        // Some kind of exception or error occurred so show error message.
        CString strTitle, strError;
        strError.LoadString( IDS_FACTORY_NOT_FOUND );
        strTitle= "XDiagnose";
        MessageBox( strError, strTitle, MB_ICONEXCLAMATION | MB_OK );
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is used to convert a numeric level into the text version.
void CLevelList::ConvertLevel(int level, CString& szLevel)
{

    szLevel.Empty();

    if (level>100)
    {
        VERIFY(szLevel.LoadString(IDS_LEVEL_HIGH));
    }
    else if (level>20)
    {
        VERIFY(szLevel.LoadString(IDS_LEVEL_MEDIUM));
    }
    else if (level>0)
    {
        VERIFY(szLevel.LoadString(IDS_LEVEL_LOW));
    }
    else
    {
        VERIFY(szLevel.LoadString(IDS_LEVEL_OFF));
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method is used to convert a text level into a numeric version.
int CLevelList::ConvertLevel(const CString& szLevel)
{
    CString szLow, szMedium, szHigh;

    VERIFY(szLow.LoadString(IDS_LEVEL_LOW));
    VERIFY(szMedium.LoadString(IDS_LEVEL_MEDIUM));
    VERIFY(szHigh.LoadString(IDS_LEVEL_HIGH));

    if (szHigh.CompareNoCase(szLevel) == 0)
    {
        return 1000;
    }
    else if (szMedium.CompareNoCase(szLevel) == 0)
    {
        return 100;
    }
    else if (szLow.CompareNoCase(szLevel) == 0)
    {
        return 20;
    }
    else
    {
        return 0;
    }
}
