/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#if !defined(AFX_LEVELLIST_H__1E515816_7643_11D3_9BD9_00600856D346__INCLUDED_)
#define AFX_LEVELLIST_H__1E515816_7643_11D3_9BD9_00600856D346__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <OSTAORB\OSTAORB.h>

#define LOG_SETTINGS_FILE _T( "TraceSettings" )

///////////////////////////////////////////////////////////////////////////////
// @class   CLevelList | This class is derived from a standard CListCtrl
//          component.  This has been done for several reasons - the first is
//          to provide custom drawing and selction facilities for the control.
//          This has been done to allow selection of an entire row instead of
//          just the first entry.  Secondly the ability to import a log
//          settings file has been added along with the ability to search a
//          network for active logs.
// @base    public | CListCtrl
class CLevelList : public CListCtrl
{
    DECLARE_DYNAMIC( CLevelList )

    // Construction
public:
    CLevelList();               // @cmember Default constructor
    virtual ~CLevelList();      // @cmember Destructor

    // @cmember Overloaded item drawing method
    virtual void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);

    // Attributes
public:

    // Operations
public:

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CLevelList)
    //}}AFX_VIRTUAL

    // Implementation
public:
    void ImportFile( const char* filename );    // @cmember Import trace settings file
    void ExportFile( const char* filename );    // @cmember Export trace settings file
    bool ImportCorba();                         // @cmember Import settings from network
    bool ExportCorba();                         // @cmember Export settings to network
    void CreateColumn( int pos, UINT uID );     // @cmember Create a column in list control
    bool GetDirtyFlag();                        // @cmember Determines if entries have changed
    bool SetDirtyFlag( bool bDirty );           // @cmember Sets the dirty flag
    void AddLevel( const char* category, int level, bool active );  // @cmember Adds an entry to the list control
    int GetLevel( const char* category );      // @cmember Gets level of specified entry
    bool GetStatus( const char* category );     // @cmember Gets status of specified entry
    int  FindCategory( const char* category );  // @cmember Determines index of specifed entry
    void SetCorbaHelper( CORBAServer* pCorba ); // @cmember Sets CORBA helper class
    void ConvertLevel(int level, CString& szLevel); // @cmember Converts Numeric Level
    int ConvertLevel(const CString& szLevel);   // @cmember Converts Text Level

    // Generated message map functions
protected:
    afx_msg void OnContextMenu(CWnd*, CPoint point);
    //{{AFX_MSG(CLevelList)
    afx_msg void OnClick(NMHDR* pNMHDR, LRESULT* pResult);
    afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
    afx_msg void OnPopupAdd();
    afx_msg void OnPopupDelete();
    afx_msg void OnPopupEdit();
    afx_msg void OnPopupImport();
    afx_msg void OnPopupExport();
    afx_msg void OnPopupRefresh();
    //}}AFX_MSG

    DECLARE_MESSAGE_MAP()
private:
    bool CreateLevel( bool bNew );  // @cmember Creates or edits a trace level
    bool DeleteLevel();             // @cmember Removes selected entry from control
    bool m_bDirty;                  // @cmember If false then no entry has changed
    CORBAServer* m_corba;           // @cmember Pointer to CORBA helper class
}
;

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_LEVELLIST_H__1E515816_7643_11D3_9BD9_00600856D346__INCLUDED_)
