/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __REG_FUNCS__
#define __REG_FUNCS__
#include "OstaDefs.h"

inline CString GetRepositoryPath()
{
	HKEY hKey;
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\OpenSTA\\Repository",0,KEY_READ,&hKey)!=ERROR_SUCCESS)
		return "";

	DWORD dwType;
	DWORD dwLen = MAX_PATH;
	char szPath[MAX_PATH];
	if(RegQueryValueEx(hKey,"Path",0,&dwType,(LPBYTE)szPath,&dwLen)==ERROR_SUCCESS)
	{
		RegCloseKey(hKey);
		return szPath;
	}
	
	RegCloseKey(hKey);
	return "";
}

inline BOOL SetRepositoryPath(CString strPath)
{
	HKEY hKey;
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\OpenSTA\\Repository",0,KEY_WRITE,&hKey)!=ERROR_SUCCESS)
		return FALSE;

	BOOL bResult = (RegSetValueEx(hKey,"Path",0,REG_SZ,
						(LPBYTE)strPath.GetBuffer(0),strPath.GetLength()+1)
						==ERROR_SUCCESS);
	
	return bResult;
}

inline BOOL SearchPluginsForExtension(const CString& strExtension,const CString& strType)
{
	HKEY hKeyScripts;
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\OpenSTA\\Plugins\\"+strType,0,KEY_READ,&hKeyScripts)!=ERROR_SUCCESS)
		return false;
	
	DWORD dwIndex=0;
	char szName[256];
	DWORD dwNameSize = 256;
	while(RegEnumKeyEx(hKeyScripts,dwIndex,szName,&dwNameSize,NULL,NULL,NULL,NULL) == ERROR_SUCCESS)
	{
		dwNameSize = 256;
		
		HKEY hKeyPlugin;
		if(RegOpenKeyEx(hKeyScripts,szName,NULL,KEY_READ,&hKeyPlugin)==ERROR_SUCCESS)
		{
			char szExtension[256];
			DWORD dwExtensionSize = 256;
			DWORD dwType;
			if(RegQueryValueEx(hKeyPlugin,"Extension",NULL,&dwType,(LPBYTE)szExtension,&dwExtensionSize)==ERROR_SUCCESS)
			{
				char *pszToken = strtok(szExtension,";");
				while(pszToken)
				{
					if(strExtension.CompareNoCase(pszToken)==0)
					{
						RegCloseKey(hKeyPlugin);
						RegCloseKey(hKeyScripts);
						return true;
					}
					pszToken = strtok(NULL,";");
				}
			}
			RegCloseKey(hKeyPlugin);
		}
		dwIndex++;
	}
	RegCloseKey(hKeyScripts);
	
	return false;
}

inline BOOL IsScript(const CString& strPath)
{
	//Is it in the script folder?
	CString strRepositoryPath = GetRepositoryPath();
	CString strTemp = strPath.Mid(strRepositoryPath.GetLength());
	if(strTemp.SpanExcluding("\\").CompareNoCase(strScriptType)!=0)	
		return false;

	//Compare with known extension in registry
	int nDotPos = strPath.ReverseFind('.');
	CString strExtension(strPath.Mid(nDotPos+1));
	
	return SearchPluginsForExtension(strExtension,strScriptType);
}


inline BOOL IsProfile(const CString& strPath)
{
	//Check file is in repository
	CString strRepositoryPath = GetRepositoryPath();
	CString strTemp = strPath.Mid(strRepositoryPath.GetLength());
	if(strTemp.SpanExcluding("\\").CompareNoCase(strProfileType)!=0)
		return false;

	//Compare with known extension in registry
	int nDotPos = strPath.ReverseFind('.');
	CString strExtension(strPath.Mid(nDotPos+1));

	return SearchPluginsForExtension(strExtension,strProfileType);
}

inline BOOL IsTest(const CString& strPath)
{
	//Check if the file has a .TST extension and
	//that the path contains \\<TESTNAME>\\<TESTNAME>.TST
	//and that we are within the tests folder of the repository
	CString strTestFile(strPath);
	int nDotPos = strTestFile.ReverseFind('.');
	if(nDotPos == -1)
		return false;
	
	if(strTestFile.Mid(nDotPos) != strTestExtension)
		return false;

	strTestFile = strTestFile.Left(nDotPos);
	int nSlashPos = strTestFile.ReverseFind('\\');
	if(nSlashPos == -1)
		return false;

	CString strTestName(strTestFile.Mid(nSlashPos+1));
	
	strTestFile = strTestFile.Left(nSlashPos);
	nSlashPos = strTestFile.ReverseFind('\\');
	if(nSlashPos == -1)
		return false;

	CString strTestFold(strTestFile.Mid(nSlashPos+1));

	if(strTestFold.CompareNoCase(strTestName)!=0)
		return false;
	
	CString strRepositoryPath = GetRepositoryPath();
	strTestFile.Delete(0,strRepositoryPath.GetLength());
	if(strTestFile.SpanExcluding("\\") == strTestType)
		return true;
	
	return false;
}

inline CString GetCompilerOutputDir()
{
	return GetRepositoryPath() + strObjectCodeFolder + "\\";
}

inline void GetNodeName(DWORD dwIP,CString& strNode)
{
	LPBYTE pbyIP = (LPBYTE)&dwIP;
	strNode.Format("%d.%d.%d.%d",pbyIP[0],pbyIP[1],pbyIP[2],pbyIP[3]);
}


#endif //__REG_FUNCS__