/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Hooker.h: interface for the CHooker class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HOOKER_H__6D37227D_F1FD_4E8E_9B6D_7209A5E24E61__INCLUDED_)
#define AFX_HOOKER_H__6D37227D_F1FD_4E8E_9B6D_7209A5E24E61__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <list>

class CHooker
{
public:
    static BOOL Init();
    static BOOL Uninit();

public:
	CHooker(PCSTR zFunctionName, PCSTR zExportingModuleName, PROC pfnHookFunction);
	~CHooker();
    PROC GetOriginalFunction() const {return m_pfnOriginalFunction;}

private:
    //// Typedefs.
    typedef std::list<CHooker*> HookList;
    typedef std::list<HMODULE>  ModuleList;

    //// Static data.
    static HookList   s_HookList;           // List of hooks.
    static ModuleList s_ModuleList;         // List of currently loaded modules.
    static HMODULE    s_hHookerModule;      // Handle of the hooker module.
    static CRITICAL_SECTION s_cs;           // Protecting concurrent accesses to static data.

    //// Data.
    PCSTR m_zExportingModuleName;   // Name of module exporting the original function.
    PCSTR m_zFunctionName;          // Function name.
    PROC  m_pfnOriginalFunction;    // Original function's address.
    PROC  m_pfnHookFunction;        // Hook function's address.

    //// Forbidden.
    CHooker();
    CHooker(const CHooker&);
    void operator=(const CHooker&);

    //// Helpers needing to access the private members.
    static BOOL GetNewModules(ModuleList& newModules);
    static BOOL ResolveHooks(HMODULE hModule);
    static BOOL ResolveHooks(const ModuleList& newModules);
    static BOOL HookInModule(HMODULE hImportingModule, PCSTR zFunctionName,
        PCSTR zExportingModuleName, PROC pfnFunctionToHook, PROC pfnHookFunction,
        BOOL bReplace, BOOL bVerbose = TRUE);
    static BOOL HookInAllModules(PCSTR zFuncName, PCSTR zExportingModuleName,
        PROC pfnFunctionToHook, PROC pfnHookFunction, BOOL bReplace);
    static BOOL InstallHooksInNewModule(HMODULE hNewModule);
    static BOOL InstallHooksInNewModules(const ModuleList& newModules);
    static BOOL ProcessNewModules();
    static HMODULE WINAPI Hook_LoadLibraryA(PCSTR pFileName);
    static HMODULE WINAPI Hook_LoadLibraryW(PCWSTR pFileName);
    static HMODULE WINAPI Hook_LoadLibraryExA(PCSTR pFileName, HANDLE hFile, DWORD dwFlags);
    static HMODULE WINAPI Hook_LoadLibraryExW(PCWSTR pFileName, HANDLE hFile, DWORD dwFlags);
    static FARPROC WINAPI Hook_GetProcAddress(HMODULE hModule, PCSTR pProcName);

    //// Built-in hooks.
    static CHooker* s_Hook_LoadLibraryA;
    static CHooker* s_Hook_LoadLibraryW;
    static CHooker* s_Hook_LoadLibraryExA;
    static CHooker* s_Hook_LoadLibraryExW;
    static CHooker* s_Hook_GetProcAddress;
};

#endif // !defined(AFX_HOOKER_H__6D37227D_F1FD_4E8E_9B6D_7209A5E24E61__INCLUDED_)
