/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"

#include "TEMainStructure.hpp"

CMainExecuterStructure *g_pTExecuter = NULL;

extern "C"
{

    __declspec(dllexport) int TELibMain(HWND hwind, char *TofFile, char *OdlDir)
    {
        if(g_pTExecuter == NULL)
            g_pTExecuter = new CMainExecuterStructure();
        if(g_pTExecuter->Init(hwind, TofFile, OdlDir) == FALSE)
            return 1;

        // Inform modeller the replay is beginning
        logr->AddTrace(STARTUP, TRACELEVEL, "!Begin replay" );

        BOOL bSucceeded = TRUE;
        string msg;

	    // Read in atvf(tof) file to create the Script Rules, and commence execution
        try
        {
            bSucceeded = g_pTExecuter->Run();
        }
        catch(const string& es)
        {
            bSucceeded = FALSE;
            msg = es;
        }
        catch(const mdbIOException& e)
        {
            msg = e.What() + ": " + e.ioname();
            bSucceeded = FALSE;
        }
        catch(const mdbParseException& e)
        {
            bSucceeded = FALSE;
            msg = e.what();
        }
        catch(const CyrRepositoryException& e)
        {
            bSucceeded = FALSE;
            msg = e.what();
        }
        catch(const exception& e)
        {
            bSucceeded = FALSE;
            msg = e.what();
        }
        catch( ... )
        {
            bSucceeded = FALSE;
            msg = "Unknwon exception caught";
        }

        int iRes = 1;
        if(bSucceeded == FALSE)
        {
            logr->AddTrace(STARTUP, ERRORLEVEL, msg.c_str());
            g_pTExecuter->Complete();
        }
        else
            iRes = 0;

        return iRes;
    }



    __declspec( dllexport ) void TELibStop()
    {
        if(g_pTExecuter)
        {
			g_pTExecuter->Stop();
            delete g_pTExecuter;
            g_pTExecuter = NULL;
        }
    }
} // extern "C"
