/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _OPENSTA_ASYNCHTTPREQUESTMANAGER_H_
#define _OPENSTA_ASYNCHTTPREQUESTMANAGER_H_

////////////////////////////////////////////////////////////////////////////////
// Asynchronous request manager definition file.
//
// Return values: unless otherwise stated, a DWORD return value is a
// GetLastError() error code.
////////////////////////////////////////////////////////////////////////////////

#include "AsyncRequestManager.hpp"
#include "AsyncHTTPRequest.hpp"
#include "Trace.hpp"

// This class implements a container of HTTP requests. At most PARALLEL_REQUESTS
// requests can execute in parallel.
// THREAD SAFE, except Init().
class CAsyncHTTPRequestManager : public CAsyncRequestManager
{
public:
    //// --------------------------------------------------------------------------------
    //// Construction / Destruction.
    //// --------------------------------------------------------------------------------
    CAsyncHTTPRequestManager()
        : CAsyncRequestManager()
    {}
    virtual ~CAsyncHTTPRequestManager()
    {
        // Delete the requests.
        for(RequestsType::const_iterator ppReq = m_requests.begin(); ppReq != m_requests.end(); ++ppReq)
            delete *ppReq;
    }

    //// --------------------------------------------------------------------------------
    //// User methods.
    //// --------------------------------------------------------------------------------

    // Initialization. Creates the requests.
    virtual DWORD Init(CThreadPoolController& controller, DWORD dwManagerID, DWORD dwParallelism, CTrace* pTrace = NULL)
    {
        DWORD dwError = 0;

        dwError = CAsyncRequestManager::Init(controller, dwManagerID, pTrace);
        if(!dwError)
        {
            for(DWORD dw = 0; dw < dwParallelism; dw++)
            {
                CAsyncHTTPRequest* pNewReq = new CAsyncHTTPRequest(m_pController->GetIocp(), *this, dw, pTrace);
                pNewReq->Init();

                AddRequest(pNewReq);
            }
        }

        return dwError;
    }

private:
    void operator=(const CAsyncHTTPRequestManager&);
};

#endif  // _OPENSTA_ASYNCHTTPREQUESTMANAGER_H_
