/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _CONFIG_FILE_H_
#define _CONFIG_FILE_H_

#include <string>

class CConfigFile
{
public:
    //// Construction.
    CConfigFile() {}
    CConfigFile(const std::string& strFile);

    // Set the INI file.
    void SetFile(const std::string& strFile);

    // Set the default section.
    void SetDefaultSection(const std::string& strSection) {m_strSection = strSection;}


    //// Write

    // Set a string. If zSection is NULL, use the default section.
    BOOL Set(const std::string& strKey, const std::string& strValue, LPCSTR zSection = NULL) const;

    // Set a long. If zSection is NULL, use the default section.
    BOOL Set(const std::string& strKey, long lValue, LPCSTR zSection = NULL) const;


    //// Read

    // Get a string. If zSection is NULL, use the default section.
    const CConfigFile& Get(const std::string& strKey, std::string& strResult, LPCSTR zDefault = NULL, LPCSTR zSection = NULL) const;

    // Get a long. If zSection is NULL, use the default section.
    const CConfigFile& Get(const std::string& strKey, long& lResult, long lDefault = 0, LPCSTR zSection = NULL) const;

private:
    std::string m_strFile;      // File path.
    std::string m_strSection;   // Default section.

    //// Forbidden.
    CConfigFile(const CConfigFile&);
    operator=(const CConfigFile&);
};

#endif  // _CONFIG_FILE_H_
