/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#ifndef __MAIN_STRUCTURE_HPP__
#define __MAIN_STRUCTURE_HPP__

#include <string>
#include "mdbgetopt.h"
#include "ConfigFile.hpp"

#ifndef CYRANO_TP_STANDALONE
#   include "OSTAORB/OSTAORB.h"
#else
#   include <CECSACORBA.h>
#endif

#include "AsyncVirtualUserController.h"

#ifdef __DLLSA__
struct stUsersParams
{
    stUsersParams() : pThreadPoolController(NULL) {}
    string zOdl;
    string zTof;
    CAsyncVirtualUserController *pThreadPoolController;
private:
    //// Forbidden:
    stUsersParams(const stUsersParams&);
    operator=(const stUsersParams&);
};
#endif

class CMainExecuterStructure
{
public:
    CMainExecuterStructure();
    ~CMainExecuterStructure();

#ifdef __DLLSA__
    BOOL    Init(HWND hWnd, char *TofFile, char *OdlDir);
#else
    BOOL    Init(int argc, char* argv[]);
#endif

    BOOL    Run();
    BOOL    Complete();

    void    Stop();

    DWORD   GetProcessId()              const   { return m_dwProcessId; }
    const std::string &GetCurrentDir()  const   { return m_zCurrentDir; }
#ifdef __DLLSA__
    const std::string &GetTofFileDir()          const   { return m_zTofFileDir; }
    const std::string &GetTofFileNameAndPath()  const   { return m_zTofFileNameAndPath; }
#else
    const Cmdbgetopt &GetCmdLineOptions()       const { return m_cmdLine; }
#endif
    CAsyncVirtualUserController *GetThreadPoolController()  { return m_pThreadPoolController; }

protected:
    BOOL    CommonInit();
    void    UnInit();
    void    CreateTrace(int nTraceVal, LPCSTR zTraceFileName);

#ifdef USE_PERF_DATA
    enum ePrfCtrsAction { PrfCtrsUse, PrfCtrsInstall, PrfCtrsUninstall };
    BOOL    InitPerfCounters(ePrfCtrsAction installType) const;
    void    ReleasePerfCounters() const;
#endif

    void    TraceMsg(const std::string &zMsg) const;

    void    EvaluateTPParams(DWORD &dwThreadPoolSize, DWORD &dwNbConcurrentThreads, DWORD &dwInitialVUCount, DWORD &dwVUCountGrowBy, DWORD &dwBrowserParallelism);
    BOOL    RunThreadPoolController();
    BOOL    RegisterCurrentDebugExecuter();

#ifndef __DLLSA__
    BOOL    ParseCommandLine(int argc, char* argv[]);

    BOOL    InitialiseCORBA(int argc, char* argv[]);
    BOOL    ReleaseCORBA();
#endif

#ifdef CYRANO_TP_STANDALONE
    BOOL    StartUsersCreationThread();
    static  void CreateUsers(void *pVoidParams);
#endif

#ifdef __DLLSA__
    BOOL    StartThreadPoolHandlerThread();
#endif

    static  void ThreadPoolHandler(void *pVoidParams);

protected:
    TScriptRule    *m_pRule;

#ifndef __DLLSA__
    std::string m_zExeName;
    std::string m_zUnderscoredHostIpAddress;
    std::string m_zStrProcessName;
    std::string m_zStatBaseFileName;        // Name of the files that will store the statistics. Extension must be added.

    Cmdbgetopt  m_cmdLine;  // Command line options.
#else
    std::string m_zTofFileNameAndPath;
    std::string m_zTofFileDir;
    HWND        m_hWnd;
    HANDLE      m_hThreadPoolHandlerThread;
#endif

    CConfigFile m_configFile;

    DWORD       m_dwProcessId;

    std::string     m_zCurrentDir;

#ifdef __DLLSA__
    stUsersParams   m_usersStartParams;
#endif

    CORBAServer                    *m_pServer;
    CAsyncVirtualUserController    *m_pThreadPoolController;

    CTrace *m_pTrace;
    BOOL    m_bInitialised;
};

#endif
