/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __ADMINISTRATION_LISTEN_CLIENT__
#define __ADMINISTRATION_LISTEN_CLIENT__


//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4710)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __SOCKET__
#  include <socket.hpp>
#endif  // __SOCKET__

#ifndef __LIBRARY__
#  include <library.hpp>
#endif  // __LIBRARY__

#ifndef __GATEWAY__
#  include "gateway.hpp"
#endif  // __GATEWAY__

#ifndef __GATEWAY_COMMAND__
#  include "gwcmd.hpp"
#endif  // __GATEWAY_COMMAND__


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// Class CGAdministrationListen //
///////////////////////////////////////////////////////////////////////////////
class CGAdministrationListen : public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Types.

public:                         // Constructor(s) and destructor.
    CGAdministrationListen (CGTask& task) ;
    ~CGAdministrationListen () ;

public:                         // Operators.

public:                         // Controls methodes.
    static void                 abort (void*) ;
    void                        init (void) ;
    void                        listenForConnections (void) ;
    void                        newClient (void) ;

public:                         // Get methodes.
    CTSocket&                   getSocket ()
    {
        return listen_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Operators.
    CGAdministrationListen&     operator = (const CGAdministrationListen&) ;

private:                        // Members.
    CGTask&                     task_ ;
    CTSocket                    listen_ ;

    //-----------------------------------------------------------------------------
}

; // CGAdministrationListen

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////// End of Class CGAdministrationListen //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// Class CGAdministrationClient //
///////////////////////////////////////////////////////////////////////////////
class CGAdministrationClient : public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                                         // Types.

public:                                         // Constructor(s) and destructor.
    CGAdministrationClient (CGTask& task) ;
    ~CGAdministrationClient () ;

public:                                         // Operators.

public:                                         // Controls methodes.
    static void                                 abort (void*) ;
    void                                        init (void) ;
    void                                        processConnection (void) ;
    void                                        newListenTask (const CGGatewayInfo& gatewayInfo) ;
    void                                        stopTaskById (const int iTaskId, const bool bClientsAlso = false) ;
    void                                        shutdown (bool bWithNoWait = false) ;
    static bool                         findTaskFromPointer (const CGTask* pTask, const CGTask* pSearchTask) ;
    static bool                         findTaskFromData (const CGTask* pTask, const CGGatewayInfo* pGatewayInfo) ;
    static bool                         findTaskFromId (const CGTask* pTask, const int* pTaskId) ;
    static bool                         findTaskFromType (const CGTask* pTask, const CGTask::enumTaskType* peTaskType) ;

public:                                         // Get methodes.
    CTSocket*                                   getSocket ()
    {
        return pClient_ ;
    }
    const CTLibrary*                            getLibrary (void)
    {
        return pLibrary_ ;
    } ;

    //-----------------------------------------------------------------------------
private:                                        // Operators.
    CGAdministrationClient&                     operator = (const CGAdministrationClient&) ;

private:                                        // Members.
    CGTask&                                     task_ ;
    CTSocket*                                   pClient_ ;
    CTLibrary*                                  pLibrary_ ;
    CGWParseCommand::fnNewGatewayParseCommand   newGatewayParseCommand_ ;
    CGWParseCommand*                            pGWParseCommand_ ;

    //-----------------------------------------------------------------------------
}

; // CGAdministrationClient

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////// End of Class CGAdministrationClient //
///////////////////////////////////////////////////////////////////////////////

#endif  // __ADMINISTRATION_LISTEN_CLIENT__
