/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "RegExp.h"
#include "gwhttp.hpp"
#include "socket.hpp"
#include  "targetEvaluation.hpp"


CTargetEvaluation::CTargetEvaluation(string servers, string exceptions, bool bLocal)
        :exceptions_(""),
        bLocal_(bLocal),
        domainName_(""),
        httpProxy_(""),
        iHttpProxyPort_(0),
        httpsProxy_(""),
        iHttpsProxyPort_ (0),
        ftpProxy_(""),
        iFtpProxyPort_(0),
        gopherProxy_(""),
        iGopherProxyPort_(0),
        socksProxy_(""),
        iSocksProxyPort_(0)
{
    // retreive the domain name
    char localHostName[2000] ;
    gethostname(localHostName, sizeof(localHostName) );
    struct hostent * h = gethostbyname(localHostName) ;
    if (h != NULL)
    {
        char * domainName = NULL ;
        if ((domainName = strstr(h->h_name, ".")) != NULL)
        {
            domainName++ ;
            domainName_ = domainName ;
        }
    }

    int iEndToken = servers.find_first_of(";", 0);
    string token ;
    int iBeginToken = 0;
    int iIndexColon ;
    string serverDefinition = "" ;
    string strPort = "" ;
    int iToken = 0;
    while (!(token = servers.substr(iBeginToken, iEndToken)).empty())
    {
        if (token.length() >= 5 && token.substr(0,5) == "HTTP:")
        {
            serverDefinition = token.substr(5, token.length() - 5) ;
            iIndexColon = CGWHttp::find_in_string(serverDefinition,":",false,0);
            httpProxy_ = serverDefinition.substr(0, iIndexColon) ;
            strPort = serverDefinition.substr(iIndexColon+1, serverDefinition.length() - iIndexColon - 1) ;
            iHttpProxyPort_ = atoi(strPort.c_str()) ;
        }
        else if (token.length() >= 4 && token.substr(0,4) == "SSL:")
        {
            serverDefinition = token.substr(4, token.length() - 4) ;
            iIndexColon = CGWHttp::find_in_string(serverDefinition,":",false,0);
            httpsProxy_ = serverDefinition.substr(0, iIndexColon) ;
            strPort = serverDefinition.substr(iIndexColon+1, serverDefinition.length() - iIndexColon - 1);
            iHttpsProxyPort_ = atoi(strPort.c_str()) ;
        }
        else if (token.length() >= 4 && token.substr(0,4) == "FTP:")
        {
            serverDefinition = token.substr(4, token.length() - 4) ;
            iIndexColon = CGWHttp::find_in_string(serverDefinition,":",false,0);
            ftpProxy_ = serverDefinition.substr(0, iIndexColon) ;
            strPort = serverDefinition.substr(iIndexColon+1, serverDefinition.length() - iIndexColon - 1);
            iFtpProxyPort_ = atoi(strPort.c_str()) ;
        }
        else if (token.length() >= 7 && token.substr(0,7) == "GOPHER:")
        {
            serverDefinition = token.substr(7, token.length() - 7) ;
            iIndexColon = CGWHttp::find_in_string(serverDefinition,":",false,0);
            gopherProxy_ = serverDefinition.substr(0, iIndexColon) ;
            strPort = serverDefinition.substr(iIndexColon+1, serverDefinition.length() - iIndexColon - 1);
            iGopherProxyPort_ = atoi(strPort.c_str()) ;
        }
        else if (token.length() >= 6 && token.substr(0,6) == "SOCKS:")
        {
            serverDefinition = token.substr(6, token.length() - 6) ;
            iIndexColon = CGWHttp::find_in_string(serverDefinition,":",false,0);
            socksProxy_ = serverDefinition.substr(0, iIndexColon) ;
            strPort = serverDefinition.substr(iIndexColon+1, serverDefinition.length() - iIndexColon - 1);
            iSocksProxyPort_ = atoi(strPort.c_str()) ;
        }
        if (iEndToken == string::npos)
            break ;
        iBeginToken = iEndToken + 1 ;
        iEndToken = servers.find_first_of(";", iBeginToken);
    }

    int iIndexDot = 0 ;
    int iIndexStar = 0 ;
    int iIndexInterro = 0 ;
    string strTemp = exceptions ;
    // replace '.' by "\."
    while ((iIndexDot = CGWHttp::find_in_string(strTemp,".",false,0)) != string::npos)
    {
        exceptions_ += strTemp.substr(0, iIndexDot) ;
        exceptions_ += "\\." ;
        strTemp = strTemp.substr(iIndexDot+1, strTemp.length() - iIndexDot - 1) ;
    }
    exceptions_ += strTemp ;

    // replace '*' by ".*"
    strTemp = exceptions_ ;
    exceptions_ = "";
    while ((iIndexStar = CGWHttp::find_in_string(strTemp,"*",false,0)) != string::npos)
    {
        exceptions_ += strTemp.substr(0, iIndexStar) ;
        exceptions_ += ".*" ;
        strTemp = strTemp.substr(iIndexStar+1, strTemp.length() - iIndexStar - 1) ;
    }
    exceptions_ += strTemp ;

    // replace '?' by "."
    while ((iIndexInterro = CGWHttp::find_in_string(exceptions_,"?",false,0)) != string::npos)
    {
        exceptions_[(size_t)iIndexInterro] = '.';
    }
}

bool CTargetEvaluation::isInLocalDomain(string FQDNhostName)
{
    int iLength = FQDNhostName.length() ;
    int iDomainLength = domainName_.length() ;

    if (iLength > iDomainLength)
    {
        string hostDomainName = FQDNhostName.substr(iLength - iDomainLength, iDomainLength) ;
        if (CGWHttp::find_in_string(hostDomainName,domainName_,false,0) == 0)
            return TRUE ;
    }
    return FALSE ;
}

void CTargetEvaluation::getProxyInfos(string & type, string & host, int & iPort)
{
    if (CGWHttp::find_in_string(type,"HTTP",false,0) == 0)
    {
        host = httpProxy_ ;
        iPort = iHttpProxyPort_ ;
    }
    else if (CGWHttp::find_in_string(type,"HTTPS",false,0) == 0)
    {
        host = httpsProxy_ ;
        iPort = iHttpsProxyPort_ ;
    }
    else if (CGWHttp::find_in_string(type,"FTP", false,0) == 0)
    {
        host = ftpProxy_ ;
        iPort = iFtpProxyPort_ ;
    }
    else if (CGWHttp::find_in_string(type,"GOPHER",false,0) == 0)
    {
        host = gopherProxy_ ;
        iPort = iGopherProxyPort_ ;
    }
    else if (CGWHttp::find_in_string(type,"SOCKS", false,0) == 0)
    {
        host = socksProxy_ ;
        iPort = iSocksProxyPort_ ;
    }
}

string CTargetEvaluation::getHostByName(string strConnect)
{
    int iIndex = 0 ;
    int iNbDot = 0 ;
    string strTemp = strConnect ;
    while ((iIndex = CGWHttp::find_in_string(strTemp,".",false,0)) != string::npos)
    {
        iNbDot ++ ;
        strTemp[iIndex] = ' ' ;
    }
    // it seems to be an IP address.
    bool bIp = FALSE ;
    char * zBuf = new char[strTemp.size() + 1] ;
    strcpy(zBuf, strTemp.c_str()) ;
    if (iNbDot == 3)
    {
        bIp = TRUE ;
        char * pToken = strtok(zBuf, " ");
        do
        {
            string token = pToken ;
            for (size_t i=0; i<token.length();i++)
            {
                if (!isdigit(token[i]))
                {
                    token = "999";
                    break ;
                }
            }
            if (atoi(token.c_str()) > 255)
            {
                bIp = FALSE ;
                break ;
            }
        }
        while ((pToken =strtok(NULL, " ")) != NULL) ;
    }

    if (zBuf)
        delete [] zBuf ;
    if (bIp)
    {
        // Add IP addresse in a map to avoid to execute gethostbyaddr
        // several times for a same IP address (can take a long time).
        CTLockGuard<CTMutex> guard(mapMutex_) ;
        if (ipAddressMap_.count(strConnect) > 0)
            return ipAddressMap_[strConnect] ;

        unsigned long ulInetAddr = inet_addr(strConnect.c_str());
        struct hostent *h = gethostbyaddr((char *)&ulInetAddr, sizeof(unsigned long), AF_INET) ;
        if (h != NULL && h->h_name != NULL)
        {
            ipAddressMap_[strConnect] = h->h_name ;
            return string(h->h_name) ;
        }
        else
        {
            ipAddressMap_[strConnect] = "" ;
            return string("") ;
        }
    }
    else
    {
        struct hostent *h = gethostbyname(strConnect.c_str()) ;
        if (h != NULL && h->h_name != NULL)
            return string(h->h_name) ;
        else
            return string("") ;
    }

}

void CTargetEvaluation::evaluate(string strConnect, string type, string & host, int & iPort, bool & bProxy)
{
    int iConnectPort = -1 ;
    int iIndexColon = CGWHttp::find_in_string(strConnect,":",false, 0);
    if (iIndexColon != string::npos)
    {
        string strPort = strConnect.substr(iIndexColon+1, strConnect.length() - iIndexColon - 1) ;
        iConnectPort = atoi(strPort.c_str());
        strConnect = strConnect.substr(0, iIndexColon) ;
    }
    else
        iConnectPort = 80 ;

    host = "" ;
    iPort = 0 ;
    getProxyInfos(type, host, iPort) ;

    // If no proxy, it is a direct connection
    if (host == "" && iPort == 0)
    {
        host = strConnect ;
        if (iConnectPort != -1)
            iPort = iConnectPort ;
        return ;
    }

    // Test to check if it is a local address.
    if (bLocal_)
    {
        bool bFound = TRUE;

        string machineName = getHostByName(strConnect) ;
        if (isInLocalDomain(machineName))
        {
            host = strConnect ;
            if (iConnectPort != -1)
                iPort = iConnectPort ;
            return ;
        }
    }

    char * zBuf = new char[exceptions_.size() + 1] ;
    strcpy(zBuf, exceptions_.c_str()) ;
    char * token = strtok(zBuf, "; \t\r\n");
    if (token != NULL)
    {
        do
        {
            string tokenReal = "^";
            tokenReal += token ;
            CRegExp regExp ;
            CRegExp * regExp2 ;
            regExp2 = regExp.RegComp(tokenReal.c_str()) ;
            if (regExp.RegFind(strConnect.c_str(), TRUE) != -1)
            {
                host = strConnect ;
                if (iConnectPort != -1)
                    iPort = iConnectPort ;
                return ;
            }
        }
        while ((token =strtok(NULL, "; \t\r\n")) != NULL) ;
    }

    bProxy = TRUE ;
}
