/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ExtendedPropertyPage.cpp : implementation file
//

#include "stdafx.h"
#include "ExtendedPropertySheet.h"
#include "ExtendedPropertyPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CExtendedPropertyPage, CPropertyPage)

//CONSTRUCTOR
//---------------------------------------------------------------------------
// nIDTemplate = Dialog ID
//nIDCaption = Caption of the property page : default value = 0
CExtendedPropertyPage::CExtendedPropertyPage(UINT nIDTemplate, UINT nIDCaption)
        :   CPropertyPage( nIDTemplate , nIDCaption )
{
    m_bActivated = FALSE;
    m_bEnabled = TRUE;
    m_pPropSheet = NULL;
}

//---------------------------------------------------------------------------

// lpszTemplateName = Dialog
//nIDCaption = Caption of the property page : default value = 0
CExtendedPropertyPage::CExtendedPropertyPage(LPCTSTR lpszTemplateName, UINT nIDCaption)
        :   CPropertyPage( lpszTemplateName , nIDCaption )
{
    m_pPropSheet = NULL;
    m_bEnabled = TRUE;
}

//---------------------------------------------------------------------------

CExtendedPropertyPage::CExtendedPropertyPage()
{
    m_pPropSheet = NULL;
    m_bEnabled = TRUE;
}

//---------------------------------------------------------------------------

CExtendedPropertyPage::~CExtendedPropertyPage()
{}

//---------------------------------------------------------------------------
//GetNextPage

LRESULT CExtendedPropertyPage::GetNextPage(BOOL forward)
{
    ASSERT( m_pPropSheet != NULL );

    //Get the current page
    int act  = m_pPropSheet->GetActiveIndex();
    int count= m_pPropSheet->GetPageCount();

    //compute the next page
    act += (forward ? 1: -1);

    BOOL found = FALSE;
    CExtendedPropertyPage *pPage;

    CString tmp;

    //Look for the next page enabled
    while (!found && act < count && act >=0)
    {
        pPage = (CExtendedPropertyPage*) m_pPropSheet->GetPage(act);
        if (pPage->IsEnabled())
        {
            found = TRUE;
        }
        else
        {
            act += (forward ? 1: -1);
        }

    }

    //return the next page
    if (found)
    {
        return (LRESULT) pPage->m_lpszTemplateName;
    }
    //stay at the same position
    return -1; // Nowhere to go :)
}

//---------------------------------------------------------------------------
// return if the page is enabled or not

BOOL CExtendedPropertyPage::IsEnabled(void)
{
    return m_bEnabled;
}

//---------------------------------------------------------------------------
//Set the page to enable or not
BOOL CExtendedPropertyPage::Enable(BOOL bEnable)
{
    BOOL old = m_bEnabled;
    m_bEnabled = bEnable;
    return old;
}

//---------------------------------------------------------------------------
//Go to the next page
LRESULT CExtendedPropertyPage::OnWizardNext()
{
    if( SaveValues() )
    {
        m_pPropSheet->OnWizardNext(this);
        return GetNextPage();
    }
    else
        return -1 ;
}

//---------------------------------------------------------------------------
//Go to the back page
LRESULT CExtendedPropertyPage::OnWizardBack()
{
    m_pPropSheet->OnWizardBack(this);
    return GetNextPage(FALSE);
}

//---------------------------------------------------------------------------
//Finish the wizard
BOOL CExtendedPropertyPage::OnWizardFinish()
{
    if ( ! m_pPropSheet->OnOK() )
        return FALSE;

    if( SaveValues() )
        return CPropertyPage::OnWizardFinish();
    else
        return FALSE ;
}

//---------------------------------------------------------------------------
// Cancel the propertySheet mode
void CExtendedPropertyPage::OnCancel()
{
    m_pPropSheet->OnCancel();
    CPropertyPage::OnCancel();
}

//---------------------------------------------------------------------------
//Finish the PropertySheet Mode
void CExtendedPropertyPage::OnOK()
{
    if ( ! m_pPropSheet->OnOK() )
        return;

    if( SaveValues() )
        CPropertyPage::OnOK();
}

//---------------------------------------------------------------------------
//Send message to parent to warn it that something happens
BOOL CExtendedPropertyPage::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
    return CPropertyPage::OnNotify(wParam,lParam,pResult);
}


//---------------------------------------------------------------------------
// event which happens when the page got the focus
BOOL CExtendedPropertyPage::OnSetActive()
{
    ASSERT( m_pPropSheet != NULL );

    m_pPropSheet->OnSetActive(this);
    m_bActivated = TRUE;

    return CPropertyPage::OnSetActive();
}

//---------------------------------------------------------------------------
// event which happens when the page lose the focus
BOOL CExtendedPropertyPage::OnKillActive()
{
    ASSERT( m_pPropSheet != NULL );

    m_pPropSheet->OnKillActive(this);

    return CPropertyPage::OnKillActive();
}

//---------------------------------------------------------------------------
// save the values from the structure to the control
void CExtendedPropertyPage::Values2Ctrl()
{}

//---------------------------------------------------------------------------
// save  the control values in the structure

void CExtendedPropertyPage::Ctrl2Values()
{}

//---------------------------------------------------------------------------
// check if the data are OK.

BOOL CExtendedPropertyPage::Verify()
{
    return TRUE ;
}

//---------------------------------------------------------------------------
// check if the data have been modified.
BOOL CExtendedPropertyPage::Modify()
{
    return TRUE ;
}

//---------------------------------------------------------------------------
// Display error message.
void CExtendedPropertyPage::DisplayErrorMessage()
{}

//---------------------------------------------------------------------------
// check if the data have been modified.

BOOL CExtendedPropertyPage::OnApply()
{
    if( Verify() )
        return CPropertyPage::OnApply() ;
    else
    {
        DisplayErrorMessage() ;
        return FALSE ;
    }
}

//---------------------------------------------------------------------------
// check if the data have been modified.
BOOL CExtendedPropertyPage::SaveValues()
{
    if( !Verify() )
    {
        DisplayErrorMessage() ;
        return FALSE ;
    }

    if( Modify() )
        Ctrl2Values() ;

    return TRUE ;
}
