/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// OSEditCtrl.cpp: implementation of the COSEditCtrl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "OSEditCtrl.h"
#include "ConstantesLib.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(COSEditCtrl, CCodeMaxCtrl)
//{{AFX_MSG_MAP(COSEditCtrl)
ON_WM_CREATE()
ON_WM_CONTEXTMENU()
//}}AFX_MSG_MAP
ON_NOTIFY_REFLECT(CMN_SELCHANGE,OnSelChange)
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

COSEditCtrl::COSEditCtrl(LPCSTR pczRegistryPath)
{
    ASSERT( pczRegistryPath != NULL ) ;
    CString strRegSection = pczRegistryPath ;
    strRegSection += _T("\\Editor") ;
    m_Serialization.SetLocation( strRegSection );

}

//---------------------------------------------------------------------------

COSEditCtrl::~COSEditCtrl()
{
}

//---------------------------------------------------------------------------


int COSEditCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CCodeMaxCtrl::OnCreate(lpCreateStruct) == -1)
        return -1;

    Init();


    return 0;
}

//---------------------------------------------------------------------------

void COSEditCtrl::Init()
{
    InitKeywords();
    InitComments();
    ReadPreferences() ;
    EnableColorSyntax();

    CMRegisterLanguage( GetLanguageName(), &m_OSLanguage.m_Language );
    SetLanguage(GetLanguageName());

    if(m_ImageList.GetSafeHandle())
        SetImageList(m_ImageList.GetSafeHandle());

    ShowScrollBar(true,true);
}

//---------------------------------------------------------------------------

CString COSEditCtrl::GetLanguageName()
{
    return _T("SCL");
}

//---------------------------------------------------------------------------

void COSEditCtrl::InitKeywords()
{}

//---------------------------------------------------------------------------


void COSEditCtrl::InitComments()
{}

//---------------------------------------------------------------------------


void COSEditCtrl::AddKeywords( char *_pszBuffer)
{
    m_OSLanguage.AddKeywords(_pszBuffer);
}

//-----------------------------------------------------------------------

void COSEditCtrl::ReadPreferences()
{
    ReadGenericPreferences() ;
}

//-----------------------------------------------------------------------

void COSEditCtrl::SavePreferences()
{
    SaveGenericPreferences() ;
}

//-----------------------------------------------------------------------

void COSEditCtrl::ReadGenericPreferences()
{
    m_Serialization.Read() ;

}

//-----------------------------------------------------------------------

void COSEditCtrl::SaveGenericPreferences()
{

    m_Serialization.Save();

}

//-----------------------------------------------------------------------

void COSEditCtrl::ApplyPreferences()
{
}

//-----------------------------------------------------------------------


void COSEditCtrl::AddPages( CPreferencesPSheet *pPSheet)
{}

//-----------------------------------------------------------------------


void COSEditCtrl::SetWindowText( LPCTSTR lpszString )
{
    SetText( lpszString );
}

//-----------------------------------------------------------------------

CString COSEditCtrl::GetString(long _lBeginLine, long _lEndLine)
{
    CString     strText;
    CM_RANGE    sel;
    //---
    ASSERT( _lEndLine >= _lBeginLine );

    _lBeginLine --;
    _lEndLine --;

    int nNumLines = GetLineCount();
    if(_lEndLine > nNumLines)
        _lEndLine = nNumLines-1;

    sel.posStart.nLine  = _lBeginLine;
    sel.posStart.nCol   = 0;
    sel.posEnd.nLine    = _lEndLine;
    sel.posEnd.nCol     = GetLineLength( _lEndLine );
    sel.bColumnSel      = FALSE;

    GetText( strText, &sel );
    return strText;
}

//-----------------------------------------------------------------------

void COSEditCtrl::DeleteLines(int nStartLine,int nEndLine)
{
    CM_RANGE    sel;
    //---
    ASSERT( nEndLine >= nStartLine );

    nStartLine --;

    sel.posStart.nLine  = nStartLine;
    sel.posStart.nCol   = 0;
    sel.posEnd.nLine    = nEndLine;
    sel.posEnd.nCol     = 0;
    sel.bColumnSel      = FALSE;

    SetSel( &sel, FALSE );
    DeleteSel();
}

//-----------------------------------------------------------------------

long COSEditCtrl::GetCurrentLine()
{
    CM_RANGE sel ;  //$$$ in case of selection from right to left
    //---           //$$$ sel.posEnd.nLine = the beginning of the selection
    GetSel( &sel,FALSE );

    return ( sel.posEnd.nLine + 1 );
}

//-----------------------------------------------------------------------

long COSEditCtrl::GetCurrentCol()
{
    CM_RANGE sel ;  //$$$ in case of selection from right to left
    //---           //$$$ sel.posEnd.nLine = the beginning of the selection
    GetSel( &sel ,FALSE);

    return (sel.posEnd.nCol + 1);
}

//-----------------------------------------------------------------------

void COSEditCtrl::InsertString( long lRow , CString strText )
{
    CM_POSITION pos;
    //---
    lRow --;
    pos.nLine   = lRow;
    pos.nCol    = 0;

    InsertText( strText, &pos);
}

//-----------------------------------------------------------------------

CString COSEditCtrl::GetWindowText()
{
    CM_RANGE sel;
    CString strValue;
    int nEndLine;
    //---
    nEndLine    = GetLineCount() - 1;

    sel.posStart.nLine  = 0;
    sel.posStart.nCol   = 0;
    sel.posEnd.nLine    = nEndLine;
    sel.posEnd.nCol     = GetLineLength( nEndLine );
    sel.bColumnSel      = FALSE;

    GetText( strValue, &sel );

    return strValue;
}

//-----------------------------------------------------------------------

void COSEditCtrl::GoToLine( int nLine)
{
    CM_RANGE    sel;
    //---
    ASSERT( nLine > 0 );

    nLine --;

    sel.posStart.nLine  = nLine;
    sel.posStart.nCol   = 0;
    sel.posEnd.nLine    = nLine;
    sel.posEnd.nCol     = 0;
    sel.bColumnSel      = FALSE;

    SetSel( &sel, TRUE );
}

//---------------------------------------------------------------------------

void COSEditCtrl::ClearSelection()
{
    CM_RANGE    sel;
    //---

    GetSel( &sel );

    sel.posStart.nLine  = sel.posEnd.nLine;
    sel.posStart.nCol   = sel.posEnd.nCol;
    sel.bColumnSel      = FALSE;

    SetSel( &sel );
}

//---------------------------------------------------------------------------
void COSEditCtrl::OnSelChange(NMHDR * pNotifyStruct, LRESULT* pResult)
{
    CWnd *pMainWnd = AfxGetMainWnd();
    if(pMainWnd)
        pMainWnd->SendMessage(WM_CARET_POS_CHANGED,0,MAKELPARAM(GetCurrentLine(),GetCurrentCol()));

    SetFocus();
    *pResult = 0;
}

