// GraphTypes.cpp: implementation of the CTimerGraph class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GraphTypes.h"
#include "XChartCtrl.h"
#include "ExcelExport.h"
#include "FilterURLDlg.h"
#include "TimerSelDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


extern HINSTANCE m_hInstance;
//////////////////////////////////////////////////////////////////////
// CGraphTypeBase Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGraphTypeBase::CGraphTypeBase(CXChartCtrl *pChart)
{
	m_bHasFilter = false;
    m_nNextGroup = 1;
	m_pChart = pChart;
}

CGraphTypeBase::~CGraphTypeBase()
{}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Initialize the display component

void CGraphTypeBase::SetGraphType(int nType)
{
    for(int nGroup=1;nGroup<=m_nNextGroup;nGroup++)
        m_pChart->SetSeriesStyle(nGroup,nType);
}

int CGraphTypeBase::GetGraphType()
{
    return m_pChart->GetSeriesStyle(1);
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Add a legend to the graph
void CGraphTypeBase::DisplayLegend()
{
    //  pLegend->GetStyle()->SetColor(RGB(192,192,192));
}

BOOL CGraphTypeBase::RemoveTestRun(int nDataGroup)
{
    CString strTestRun;
	strTestRun.Format("%d",nDataGroup);

	int nNumGroups=m_nNextGroup;
    for(int nGroup=1;nGroup<=nNumGroups;nGroup++)
	{
		CString str = m_pChart->GetSeriesAnnotation(nGroup);
		if(str == strTestRun)
			m_nNextGroup--;
	}

	m_pChart->RemoveSeries(strTestRun);

    return true;
}

BOOL CGraphTypeBase::ExportToExcel()
{
    CExcelExport ExcelExport;
    return ExcelExport.SetGraph(m_pChart);
}

	void CGraphTypeBase::GetMenuItems(MenuItemArray& arMenuItems)
{
    arMenuItems.RemoveAll();
}

void CGraphTypeBase::OnCommand(UINT uiCommandID,CWnd *pParent)
{}


/*void CGraphTypeBase::AddDataFromGraph(CXChartCtrl *pChart)
{
	VARIANT &var = pChart->m_ctlChart.GetData();
	m_pChart->SetData(var);

	COleSafeArray arData(var);
    long lNumGroups;
    arData.GetUBound(2,&lNumGroups);

    int nFirstGroup = m_nNextGroup;
    m_nNextGroup += lNumGroups;

    for(int nSeries=1;nSeries<=lNumGroups+1;nSeries++)
    {
        int nStyle = pChart->m_ctlChart.GetSeriesStyle(nSeries);
        m_pChart->SetSeriesStyle(nSeries+nFirstGroup-1,nStyle);
		
		CString strText = pChart->m_ctlChart.GetLegendText(nSeries);
		m_pChart->SetLegendText(nSeries+nFirstGroup-1,strText);

        CString str = pChart->m_ctlChart.GetSeriesAnnotation(nSeries);
        m_pChart->SetSeriesAnnotation(nSeries+nFirstGroup-1,str);

        BOOL b = pChart->m_ctlChart.GetSeriesOnSecondaryAxis(nSeries);
       m_pChart->SetSeriesOnSecondaryAxis(nSeries+nFirstGroup-1,b);
    }

}
*/
void CGraphTypeBase::SetValue(int nIndex,int nGroup,double dValue,const CString& strAnnotation)
{
}
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CTimerGraph::CTimerGraph(CXChartCtrl *pChart):
        CGraphTypeBase(pChart)
{
}

CTimerGraph::~CTimerGraph()
{
}


BOOL CTimerGraph::AddTestRun(TimerList& listTimers,DWORD dwTestStartTime)
{
    return true;
}

//////////////////////////////////////////////////////////////////////
// CRTATGraph Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStringArray CRTATGraph::m_arstrTimers;
int CRTATGraph::m_nInstanceCount =0;

CRTATGraph::CRTATGraph(CXChartCtrl *pChart):
        CTimerGraph(pChart)
{
	m_nInstanceCount++;
    m_pChart->SetXAxisLabel("Active Users");
    m_pChart->SetYAxisLabel("Timer Values");
    m_pChart->ShowLegend(TRUE);
}

CRTATGraph::~CRTATGraph()
{
	m_nInstanceCount--;
	if(!m_nInstanceCount)
		m_arstrTimers.RemoveAll();
}


BOOL CRTATGraph::AddTestRun(TimerList& listTimers,DWORD dwTestStartTime)
{
	if(!listTimers.GetCount())
		return false;

	if(!m_arstrTimers.GetSize())
	{
		AfxSetResourceHandle(m_hInstance);
		CTimerSelDlg dlg(listTimers);
		int nResult = dlg.DoModal();
		AfxSetResourceHandle(AfxGetInstanceHandle());
		if(nResult != IDOK)
			return false;

		m_arstrTimers.Copy(dlg.m_arstrTimers);
	}
	
	CString strTestRun;
	strTestRun.Format("%d",m_nNextGroup);
    struct RTAT_DATA_t
    {
        int nNumTimers;
        double dTotalTime;
    }
    *pRtatData;

    // Calculations for RTAT graph
    CMap <DWORD,DWORD,void*,void *> mapThreadsToData;
    CDWordArray arNumThreads;

	for(int nTimer=0;nTimer<m_arstrTimers.GetSize();nTimer++)
	{
		POSITION pos = listTimers.GetHeadPosition();
		while(pos)
		{
			Timer *pTimer = listTimers.GetNext(pos);
			if(!pTimer->bInFilter)
				continue;

			if(pTimer->strTimerName != m_arstrTimers[nTimer])
				continue;

			DWORD dwNumThreads = pTimer->dwActiveUsers;

			if(mapThreadsToData.Lookup(dwNumThreads,(void*&)pRtatData))
			{
				pRtatData->nNumTimers++;
				pRtatData->dTotalTime += pTimer->dValue;
			}
			else
			{
				pRtatData = new RTAT_DATA_t;
				pRtatData->nNumTimers = 1;
				pRtatData->dTotalTime = pTimer->dValue;
				mapThreadsToData.SetAt(dwNumThreads,pRtatData);

				for(int nNdx=0;nNdx<arNumThreads.GetSize();nNdx++)
					if(arNumThreads[nNdx]>dwNumThreads)
						break;

				//  if(!nNdx)nNdx++;

				arNumThreads.InsertAt(nNdx/*-1*/,dwNumThreads);
			}
		}

		COleSafeArray saData;

		DWORD dwMaxNumThreads = 1;
		VARIANT *pVar = NULL;
		if(arNumThreads.GetSize())
			dwMaxNumThreads = arNumThreads[arNumThreads.GetSize()-1];

		DWORD dwNumElements[]= {dwMaxNumThreads, 2};
		saData.Create(VT_VARIANT,2,dwNumElements);

		saData.AccessData((void**)&pVar);
		pVar[0].vt = VT_EMPTY;

		CString strLabel;
		int nLastTerm = 1;
		for(int nNdx=0;nNdx<arNumThreads.GetSize();nNdx++)
		{
			DWORD dwNumThreads = arNumThreads[nNdx];
			mapThreadsToData.Lookup(dwNumThreads,(void*&)pRtatData);
			double dTime = pRtatData->dTotalTime / pRtatData->nNumTimers;

			// Add null data for all no of terminals between the last value and this one
			for (int nNdx2=nLastTerm;nNdx2<(int)dwNumThreads; nNdx2++)
			{
				VariantInit(&pVar[nNdx2-1+dwMaxNumThreads]);

				strLabel.Format("%d", nNdx2);
				pVar[nNdx2-1].vt = VT_BSTR;
				pVar[nNdx2-1].bstrVal = strLabel.AllocSysString();
			}

			nLastTerm = dwNumThreads+1;

			// Add the data
			pVar[dwNumThreads-1+dwMaxNumThreads].vt = VT_R8;
			pVar[dwNumThreads-1+dwMaxNumThreads].dblVal = dTime;

			strLabel.Format("%d", (int)dwNumThreads);
			pVar[dwNumThreads-1].vt = VT_BSTR;
			pVar[dwNumThreads-1].bstrVal = strLabel.AllocSysString();
			delete pRtatData;
		}

		saData.UnaccessData();
		m_pChart->SetData(saData);
		m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);
		m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
		m_pChart->SetLegendText(m_nNextGroup,m_arstrTimers[nTimer]);
		m_nNextGroup++;
	    
		mapThreadsToData.RemoveAll();
		arNumThreads.RemoveAll();
    }

    return true;
}

CRTTimeGraph::CRTTimeGraph(CXChartCtrl *pChart):
        CTimerGraph(pChart)
{
    m_pChart->SetXAxisLabel("Elapsed Time");
    m_pChart->SetYAxisLabel("Timer Values");
}

CRTTimeGraph::~CRTTimeGraph()
{}


BOOL CRTTimeGraph::AddTestRun(TimerList& listTimers,DWORD dwTestStartTime)
{
	if(!listTimers.GetCount())
		return false;

	CString strTestRun;
	strTestRun.Format("%d",m_nNextGroup);
    int nLastRow = listTimers.GetCount();
    if(!nLastRow)
        return true;

    double dStartTime = listTimers.GetHead()->dwTimeStamp;
    double dEndTime = listTimers.GetTail()->dwTimeStamp;


    //For elapsed time graphs the data may need to be offset
    //so zero is the same for all graphs
    int nTimeOffset = (int)(dStartTime - dwTestStartTime);

    double dTimeStamp=0;
    double dTotalRespTime=0;
    double dRespTime=0;
    int nCurrentRow=0;
    int nElapsedTime =0;
    double dValue=0;
    int nCountAtTime=-1;
    CString strLabel;
    Timer *pTimer = NULL;

    COleSafeArray saData;

    int nNumSeconds = (int)(dEndTime-dStartTime+1);
    DWORD dwNumElements[]= {nNumSeconds, 2};
    saData.Create(VT_VARIANT,2,dwNumElements);

    VARIANT *pVar;
    saData.AccessData((void**)&pVar);

    POSITION pos = listTimers.GetHeadPosition();
    for(double dTime=dStartTime;dTime<=dEndTime;dTime++)
    {
        while(dTimeStamp<=dTime)
        {
            dTotalRespTime += dRespTime;
            nCountAtTime++;

            if(pos)
            {
                pTimer = listTimers.GetNext(pos);
                while(pos && !pTimer->bInFilter)
                    pTimer = listTimers.GetNext(pos);
            }

            if(pos && pTimer)
            {
                dTimeStamp = pTimer->dwTimeStamp;
                dRespTime = pTimer->dValue;
            }
            else
                break;
        }

        if(nCountAtTime)
        {
            dValue = dTotalRespTime / nCountAtTime;
            dTotalRespTime = 0;
            nCountAtTime=0;

            pVar[nElapsedTime+nNumSeconds].vt = VT_R8;
            pVar[nElapsedTime+nNumSeconds].dblVal = dValue;
        }
        else
        {
            pVar[nElapsedTime+nNumSeconds].vt = VT_EMPTY;
        }

        nElapsedTime++;

        strLabel.Format("%d", nElapsedTime+nTimeOffset);
        pVar[nElapsedTime-1].vt = VT_BSTR;
        pVar[nElapsedTime-1].bstrVal = strLabel.AllocSysString();
    }

    saData.UnaccessData();

    m_pChart->SetData(saData);
    m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);
    m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
	m_nNextGroup++;
    return true;
}

//////////////////////////////////////////////////////////////////////
// CHttpGraph Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHttpGraph::CHttpGraph(CXChartCtrl *pChart):
        CGraphTypeBase(pChart)
{
    m_bHasFilter = false;
	m_pmapUrls = NULL;
}

CHttpGraph::~CHttpGraph()
{}


BOOL CHttpGraph::AddTestRun(HttpArray& arHttpData,URLMap* pUrls,DWORD dwTestStartTime)
{
	m_dwTestStartTime = dwTestStartTime;
	m_parHttpData = &arHttpData;
	m_pmapUrls = pUrls;
    return true;
}

void CHttpGraph::GetMenuItems(MenuItemArray& arMenuItems)
{
    arMenuItems.RemoveAll();

    MenuItems MI;

    if(m_bHasFilter)
    {
        MI.strItem = "Filter URLs";
        MI.uiCommandID = enMenuItemFilter;
        arMenuItems.Add(MI);
    }
}

void CHttpGraph::OnCommand(UINT uiCommandID,CWnd *pParent)
{
    switch(uiCommandID)
    {
    case enMenuItemFilter:
	{
		ShowFilter();
	}
	break;
    }
}

void CHttpGraph::ShowFilter()
{
    //Put up filter dialog
	AfxSetResourceHandle(m_hInstance);
    CFilterURLDlg dlg(m_pmapUrls,m_ardwURLFilters[0]);
	int nResult = dlg.DoModal();
	AfxSetResourceHandle(AfxGetInstanceHandle());
    if(nResult==IDOK)
    {
        //reload data
		RemoveTestRun(1);
		AddTestRun(*m_parHttpData,m_pmapUrls,m_dwTestStartTime);
    }
}

BOOL CHttpGraph::IsURLInFilter(int nURLNdx,int nGroup)
{
    if(!m_ardwURLFilters[nGroup].GetSize())
        return true;

    for(int nURL=0;nURL<m_ardwURLFilters[nGroup].GetSize();nURL++)
    {
        if(nURLNdx == (int)m_ardwURLFilters[nGroup][nURL])
            return true;
    }

    return false;
}
//////////////////////////////////////////////////////////////////////
// CThroughputGraph Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CThroughputGraph::CThroughputGraph(CXChartCtrl *pChart):
        CHttpGraph(pChart)
{
    m_bHasFilter = true;
    m_pChart->SetXAxisLabel("Elapsed Time");
    m_pChart->SetYAxisLabel("HTTP Monitored Bytes / Second");
}

CThroughputGraph::~CThroughputGraph()
{
}


BOOL CThroughputGraph::AddTestRun(HttpArray& arHttpData,URLMap* pUrls,DWORD dwTestStartTime)
{
	if(!arHttpData.GetSize() || !pUrls->GetCount())
		return false;

	CString strTestRun;
	strTestRun.Format("%d",m_nNextGroup);

    int nNumStats = arHttpData.GetSize();
    if(!nNumStats)
        return false;

    DWORD dwStartTime = arHttpData[0]->dwTimeStamp;
    DWORD dwEndTime = arHttpData[nNumStats-1]->dwTimeStamp;

    int nTimeOffset = dwStartTime - dwTestStartTime;

    COleSafeArray saData;

    DWORD dwNumPoints = dwEndTime-dwStartTime+1;
    DWORD dwNumElements[]= {dwNumPoints , 2};
    saData.Create(VT_VARIANT,2,dwNumElements);

    VARIANT *pVar;
    saData.AccessData((void**)&pVar);

    CString strLabel;
    int nElapsedTime=0;
    DWORD dwTotalBytes=0;
    DWORD dwCurrentTime=0;
    int nCurrentStat=0;

    DWORD dwTime = dwStartTime;

    while(1)
    {
        for(;nCurrentStat<nNumStats;nCurrentStat++)
            if(IsURLInFilter( arHttpData[nCurrentStat]->nURLNdx ,m_nNextGroup-1))
                break;

        if(nCurrentStat<nNumStats)
            dwCurrentTime = arHttpData[nCurrentStat]->dwTimeStamp;
        else
            dwCurrentTime = 0;

        if(dwTime == dwCurrentTime)
        {
            dwTotalBytes += arHttpData[nCurrentStat]->dwReplySize;
        }
        else
        {
            nElapsedTime = dwTime-dwStartTime;

            pVar[nElapsedTime + dwNumPoints].vt = VT_R8;
            pVar[nElapsedTime + dwNumPoints].dblVal = dwTotalBytes;

            strLabel.Format("%d", nElapsedTime+nTimeOffset);
            pVar[nElapsedTime].vt = VT_BSTR;
            pVar[nElapsedTime].bstrVal = strLabel.AllocSysString();

            if(nCurrentStat>=nNumStats)
                break;

            dwTotalBytes = arHttpData[nCurrentStat]->dwReplySize;

            //Fill in any empty values
            while(++dwTime<dwCurrentTime)
            {
                nElapsedTime = dwTime-dwStartTime;

                pVar[nElapsedTime + dwNumPoints].vt = VT_EMPTY;
                pVar[nElapsedTime + dwNumPoints].dblVal = 0;

                strLabel.Format("%d", nElapsedTime+nTimeOffset);
                pVar[nElapsedTime].vt = VT_BSTR;
                pVar[nElapsedTime].bstrVal = strLabel.AllocSysString();
            }
        }
        nCurrentStat++;
    }

    saData.UnaccessData();

    m_pChart->SetData(saData);
    m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);

   // Set the data group name with the test run name.
    m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
	m_nNextGroup++;

    return CHttpGraph::AddTestRun(arHttpData,pUrls,dwTestStartTime);
}


//////////////////////////////////////////////////////////////////////
// CHitsPerSecondGraph Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHitsPerSecondGraph::CHitsPerSecondGraph(CXChartCtrl *pChart):
        CHttpGraph(pChart)
{
    m_pChart->SetXAxisLabel("Elapsed Time");
    m_pChart->SetYAxisLabel("HTTP Responses / Second");
}

CHitsPerSecondGraph::~CHitsPerSecondGraph()
{
}


BOOL CHitsPerSecondGraph::AddTestRun(HttpArray& arHttpData,URLMap* pUrls,DWORD dwTestStartTime)
{
	if(!arHttpData.GetSize() || !pUrls->GetCount())
		return false;

	CString strTestRun;
	strTestRun.Format("%d",m_nNextGroup);

    int nNumStats = arHttpData.GetSize();
    if(!nNumStats)
        return true;

    DWORD dwStartTime = arHttpData[0]->dwTimeStamp;
    DWORD dwEndTime= arHttpData[nNumStats-1]->dwTimeStamp;

    COleSafeArray saData;

    DWORD dwNumPoints = dwEndTime-dwStartTime+1;
    DWORD dwNumElements[]= {dwNumPoints ,2};
    saData.Create(VT_VARIANT,2,dwNumElements);

    VARIANT *pVar;
    saData.AccessData((void**)&pVar);

    int nTimeOffset = dwStartTime - dwTestStartTime;

    CString strLabel;
    int nElapsedTime=0;
    DWORD dwCount = 0;
    DWORD dwCurrentTime=0;
    int nCurrentStat=0;

    DWORD dwTime = dwStartTime;

    while(1)
    {
        for(;nCurrentStat<nNumStats;nCurrentStat++)
            if(IsURLInFilter( arHttpData[nCurrentStat]->nURLNdx ,m_nNextGroup-1))
                break;

        if(nCurrentStat<nNumStats)
            dwCurrentTime = arHttpData[nCurrentStat]->dwTimeStamp;
        else
            dwCurrentTime = 0;

        if(dwTime == dwCurrentTime)
        {
            dwCount++;
        }
        else
        {
            nElapsedTime = dwTime-dwStartTime;

            pVar[nElapsedTime + dwNumPoints].vt = VT_R8;
            pVar[nElapsedTime + dwNumPoints].dblVal = dwCount;

            strLabel.Format("%d", nElapsedTime+nTimeOffset);
            pVar[nElapsedTime].vt = VT_BSTR;
            pVar[nElapsedTime].bstrVal = strLabel.AllocSysString();

            if(nCurrentStat>=nNumStats)
                break;

            dwCount=1;

            //Fill in any empty values
            while(++dwTime<dwCurrentTime)
            {
                nElapsedTime = dwTime-dwStartTime;

                pVar[nElapsedTime + dwNumPoints].vt = VT_EMPTY;

                strLabel.Format("%d", nElapsedTime+nTimeOffset);
                pVar[nElapsedTime].vt = VT_BSTR;
                pVar[nElapsedTime].bstrVal = strLabel.AllocSysString();
            }
        }
        nCurrentStat++;
    }

    saData.UnaccessData();

    m_pChart->SetData(saData);
    m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);
    // Set the data group name with the test run name.
    m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
	m_nNextGroup++;

    return CHttpGraph::AddTestRun(arHttpData,pUrls,dwTestStartTime);
}


//////////////////////////////////////////////////////////////////////
// CAvPageRTPerUserGraph Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAvPageRTPerUserGraph::CAvPageRTPerUserGraph(CXChartCtrl *pChart):
        CHttpGraph(pChart)
{
    m_bHasFilter = true;
    m_pChart->SetXAxisLabel("Number Of Responses");
    m_pChart->SetYAxisLabel("HTTP Response Time");
}

CAvPageRTPerUserGraph::~CAvPageRTPerUserGraph()
{
}


BOOL CAvPageRTPerUserGraph::AddTestRun(HttpArray& arHttpData,URLMap* pUrls,DWORD dwTestStartTime)
{
	if(!arHttpData.GetSize() || !pUrls->GetCount())
		return false;

	CString strTestRun;
	strTestRun.Format("%d",m_nNextGroup);

    typedef struct
    {
        int nCount;     //Number of occurences of x VUs
        double dTotalTime;  //Total Response time for x VUs
    }

    RespTime;

    CMap <int,int&,LPVOID,LPVOID> mapVUsToRespTime;

    int nNumVUs=0;
    int nMaxVUs = 0;
    DWORD dwLastTimeStamp = 0;
    DWORD dwTimeStamp = 0;
    DWORD dwRespTime=0;
    double dAvRespTime=0;
    int nNumIncludedVUs=0;

    bool bURLFound=true;

    RespTime *pRespTime;

    int nLastRow = arHttpData.GetSize();

    for(int nRow=0;nRow<=nLastRow;nRow++)
    {
        if(nRow != nLastRow)
        {
            //Apply URL Filter
            if(m_ardwURLFilters[m_nNextGroup-1].GetSize())
            {
                int nURLNdx = arHttpData[nRow]->nURLNdx;

                bURLFound=false;
                for(int nURL=0;nURL<m_ardwURLFilters[m_nNextGroup-1].GetSize()&&!bURLFound;nURL++)
                    bURLFound = (nURLNdx == (int)m_ardwURLFilters[m_nNextGroup-1][nURL]);
            }

            dwLastTimeStamp = dwTimeStamp;

            dwTimeStamp = arHttpData[nRow]->dwTimeStamp;
            if(bURLFound)
                dwRespTime = arHttpData[nRow]->dwResponseTime;
            else
                dwRespTime=0;

            if(!dwLastTimeStamp || (dwTimeStamp == dwLastTimeStamp))
            {
                nNumVUs++;
                dAvRespTime += dwRespTime;
                if(bURLFound)
                    nNumIncludedVUs++;
                continue;
            }
        }

        ASSERT(nNumIncludedVUs || (!nNumIncludedVUs && !dAvRespTime));
        //Get average for this occurence of nNumVUs
        if(nNumIncludedVUs)
        {
            dAvRespTime /= nNumIncludedVUs;

            if(mapVUsToRespTime.Lookup(nNumVUs,(LPVOID&)pRespTime))
            {
                pRespTime->nCount++;
                pRespTime->dTotalTime += dAvRespTime;
            }
            else
            {
                pRespTime = new RespTime;
                pRespTime->nCount=1;
                pRespTime->dTotalTime = dAvRespTime;
                mapVUsToRespTime.SetAt(nNumVUs,pRespTime);
            }
        }

        if(nMaxVUs < nNumVUs) nMaxVUs = nNumVUs;
        nNumVUs=1;
        nNumIncludedVUs= (bURLFound ? 1 : 0);

        dAvRespTime = dwRespTime;
    }

    COleSafeArray saData;

    DWORD dwNumPoints = nMaxVUs;
    DWORD dwNumElements[]= {dwNumPoints , 2};
    saData.Create(VT_VARIANT,2,dwNumElements);

    VARIANT *pVar;
    saData.AccessData((void**)&pVar);

    //Put values into the chart
    CString strLabel;
    for(nNumVUs = 1;nNumVUs<=nMaxVUs;nNumVUs++)
    {
        if(mapVUsToRespTime.Lookup(nNumVUs,(LPVOID&)pRespTime))
        {
            dAvRespTime = (pRespTime->dTotalTime / pRespTime->nCount);
            delete pRespTime;
        }
        else
        {
            dAvRespTime = 0;
            pRespTime=NULL;
        }

        pVar[nNumVUs-1 + dwNumPoints].vt = VT_R8;
        pVar[nNumVUs-1 + dwNumPoints].dblVal = dAvRespTime;

        strLabel.Format("%d", nNumVUs);
        pVar[nNumVUs-1].vt = VT_BSTR;
        pVar[nNumVUs-1].bstrVal = strLabel.AllocSysString();
    }

    saData.UnaccessData();

    m_pChart->SetData(saData);
    m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);

    // Set the data group name with the test run name.
    m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
	m_nNextGroup++;
    return CHttpGraph::AddTestRun(arHttpData,pUrls,dwTestStartTime);
}

//////////////////////////////////////////////////////////////////////
// CErrorsPerUserGraph Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CErrorsPerUserGraph::CErrorsPerUserGraph(CXChartCtrl *pChart):
        CHttpGraph(pChart)
{
    m_bHasFilter = true;
    m_pChart->SetXAxisLabel("Active Users");
    m_pChart->SetYAxisLabel("HTTP Errors");
}

CErrorsPerUserGraph::~CErrorsPerUserGraph()
{
}


BOOL CErrorsPerUserGraph::AddTestRun(HttpArray& arHttpData,URLMap* pUrls,DWORD dwTestStartTime)
{
	if(!arHttpData.GetSize() || !pUrls->GetCount())
		return false;

	CString strTestRun;
	strTestRun.Format("%d",m_nNextGroup);
    CMap <int,int&,int,int&> mapVUsToErrors;

    int nNumErrors=0;
    int nNumVUs=0;
    int nMaxVUs = 0;
    WORD wResponseCode=0;
    DWORD dwLastTimeStamp = 0;
    DWORD dwTimeStamp = 0;
    bool bURLFound=true;
    CString strURL;

    int nLastRow = arHttpData.GetSize();

    for(int nRow=0;nRow<=nLastRow;nRow++)
    {
        if(nRow!=nLastRow)
        {
            //Apply URL Filter
            if(m_ardwURLFilters[m_nNextGroup-1].GetSize())
            {
                int nURLNdx = arHttpData[nRow]->nURLNdx;

                bURLFound=false;
                for(int nURL=0;nURL<m_ardwURLFilters[m_nNextGroup-1].GetSize()&&!bURLFound;nURL++)
                    bURLFound = (nURLNdx == (int)m_ardwURLFilters[m_nNextGroup-1][nURL]);
            }

            dwLastTimeStamp = dwTimeStamp;

            dwTimeStamp = arHttpData[nRow]->dwTimeStamp;
            if(bURLFound)
                wResponseCode = arHttpData[nRow]->wResponseCode;
            else
                wResponseCode=0;

            if(!dwLastTimeStamp || (dwTimeStamp == dwLastTimeStamp))
            {
                nNumVUs++;

                //Only an error if above 400
                if(wResponseCode>=400)
                    nNumErrors++;

                continue;
            }
        }

        int nNumPrevErrors=0;
        if(mapVUsToErrors.Lookup(nNumVUs,nNumPrevErrors))
            nNumErrors+=nNumPrevErrors;

        mapVUsToErrors.SetAt(nNumVUs,nNumErrors);

        if(nMaxVUs < nNumVUs) nMaxVUs = nNumVUs;
        nNumVUs=1;

        //Only an error if above 400
        if(wResponseCode>=400)
            nNumErrors=1;
        else
            nNumErrors=0;
    }

    COleSafeArray saData;

    DWORD dwNumPoints = nMaxVUs;
    DWORD dwNumElements[]= {dwNumPoints  , 2};
    saData.Create(VT_VARIANT,2,dwNumElements);

    VARIANT *pVar;
    saData.AccessData((void**)&pVar);

    //Put values into the chart
    CString strLabel;
    for(nNumVUs = 1;nNumVUs<=nMaxVUs;nNumVUs++)
    {
        if(!mapVUsToErrors.Lookup(nNumVUs,nNumErrors))
            nNumErrors=0;

        pVar[nNumVUs-1 + dwNumPoints].vt = VT_R8;
        pVar[nNumVUs-1 + dwNumPoints].dblVal = nNumErrors;

        strLabel.Format("%d", nNumVUs);
        pVar[nNumVUs-1].vt = VT_BSTR;
        pVar[nNumVUs-1].bstrVal = strLabel.AllocSysString();
    }

    saData.UnaccessData();

    m_pChart->SetData(saData);
    m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);

    // Set the data group name with the test run name.
    m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
	m_nNextGroup++;

    return CHttpGraph::AddTestRun(arHttpData,pUrls,dwTestStartTime);
}

//////////////////////////////////////////////////////////////////////
// CErrorsTimeGraph Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CErrorsTimeGraph::CErrorsTimeGraph(CXChartCtrl *pChart):
        CHttpGraph(pChart)
{
    m_bHasFilter = true;
    m_pChart->SetXAxisLabel("Elapsed Time");
    m_pChart->SetYAxisLabel("HTTP Errors");
}

CErrorsTimeGraph::~CErrorsTimeGraph()
{
}


BOOL CErrorsTimeGraph::AddTestRun(HttpArray& arHttpData,URLMap* pUrls,DWORD dwTestStartTime)
{
	if(!arHttpData.GetSize() || !pUrls->GetCount())
		return false;

	CString strTestRun;
	strTestRun.Format("%d",m_nNextGroup);

    int nLastRow = arHttpData.GetSize()-1;

    DWORD dwStartTime = arHttpData[0]->dwTimeStamp;
    DWORD dwEndTime = arHttpData[nLastRow]->dwTimeStamp;

    int nTimeOffset = dwStartTime - dwTestStartTime;

    DWORD dwTimeStamp=0;
    CString strLabel;
    int nElapsedTime=0;
    WORD wResponseCode=0;
    int nTotalErrors=0;
    CString strURL;
    bool bFound=false;

    COleSafeArray saData;

    DWORD dwNumPoints = dwEndTime-dwStartTime+1;
    DWORD dwNumElements[]= {dwNumPoints  , 2};
    saData.Create(VT_VARIANT,2,dwNumElements);

    VARIANT *pVar;
    saData.AccessData((void**)&pVar);

    int nCurrentRow=0;
    for(int nTime=dwStartTime;nTime<=(int)dwEndTime;nTime++)
    {
        while((dwTimeStamp<=(DWORD)nTime) && (nCurrentRow<=nLastRow+1))
        {
            //Only an error if above 400
            if(wResponseCode>=400)
            {
                nTotalErrors++;
                wResponseCode =0;
            }

            if(nCurrentRow<=nLastRow)
            {
                dwTimeStamp = arHttpData[nCurrentRow]->dwTimeStamp;
                wResponseCode = arHttpData[nCurrentRow]->wResponseCode;

                //Apply URL Filter
                if(m_ardwURLFilters[m_nNextGroup-1].GetSize())
                {
                    int nURLNdx = arHttpData[nCurrentRow]->nURLNdx;

                    bFound=false;
                    for(int nURL=0;nURL<m_ardwURLFilters[m_nNextGroup-1].GetSize()&&!bFound;nURL++)
                    {
                        bFound = (nURLNdx == (int)m_ardwURLFilters[m_nNextGroup-1][nURL]);
                    }

                    if(!bFound)
                        wResponseCode = 0;
                }
            }
            nCurrentRow++;
        }

        nElapsedTime = nTime-dwStartTime;

        pVar[nElapsedTime + dwNumPoints].vt = VT_R8;
        pVar[nElapsedTime + dwNumPoints].dblVal = nTotalErrors;

        strLabel.Format("%d", nElapsedTime+nTimeOffset+1);
        pVar[nElapsedTime].vt = VT_BSTR;
        pVar[nElapsedTime].bstrVal = strLabel.AllocSysString();
    }

    saData.UnaccessData();

    m_pChart->SetData(saData);
    m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);

    // Set the data group name with the test run name.
    m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
	m_nNextGroup++;
    return CHttpGraph::AddTestRun(arHttpData,pUrls,dwTestStartTime);
}

//////////////////////////////////////////////////////////////////////
// CPagesPerSecondGraph Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPagesPerSecondGraph::CPagesPerSecondGraph(CXChartCtrl *pChart):
        CHttpGraph(pChart)
{
    m_bHasFilter = true;
    m_pChart->SetXAxisLabel("Elapsed Time");
    m_pChart->SetYAxisLabel("HTTP Response Time");
}

CPagesPerSecondGraph::~CPagesPerSecondGraph()
{
}


BOOL CPagesPerSecondGraph::AddTestRun(HttpArray& arHttpData,URLMap* pUrls,DWORD dwTestStartTime)
{
	if(!arHttpData.GetSize() || !pUrls->GetCount())
		return false;

   	CString strTestRun;
	strTestRun.Format("%d",m_nNextGroup);

    int nNumStats = arHttpData.GetSize();
    if(!nNumStats)
        return true;

    for(int nStat=0;nStat<nNumStats;nStat++)
        if(IsURLInFilter( arHttpData[nStat]->nURLNdx ,m_nNextGroup-1))
            break;
	
    DWORD dwStartTime = arHttpData[nStat]->dwTimeStamp;
    DWORD dwEndTime = arHttpData[nNumStats-1]->dwTimeStamp;

    int nTimeOffset = dwStartTime - dwTestStartTime;

    COleSafeArray saData;

    DWORD dwNumPoints = dwEndTime-dwStartTime+1;
    DWORD dwNumElements[]= {dwNumPoints , 2};
    saData.Create(VT_VARIANT,2,dwNumElements);

    VARIANT *pVar;
    saData.AccessData((void**)&pVar);

    CString strLabel;
    int nElapsedTime=0;
    DWORD dwTotalRespTime=0;
    DWORD dwCount=0;
    DWORD dwCurrentTime=0;
    int nCurrentStat=0;

    DWORD dwTime = dwStartTime;

    while(1)
    {
        for(;nCurrentStat<nNumStats;nCurrentStat++)
            if(IsURLInFilter( arHttpData[nCurrentStat]->nURLNdx ,m_nNextGroup-1))
                break;

        if(nCurrentStat<nNumStats)
            dwCurrentTime = arHttpData[nCurrentStat]->dwTimeStamp;
        else
            dwCurrentTime = 0;

        if(dwTime == dwCurrentTime)
        {
            dwTotalRespTime += arHttpData[nCurrentStat]->dwResponseTime;
            dwCount++;
        }
        else
        {
            nElapsedTime = dwTime-dwStartTime;

            double dAvPageRespTime = (double)dwTotalRespTime / dwCount;
            pVar[nElapsedTime + dwNumPoints].vt = VT_R8;
            pVar[nElapsedTime + dwNumPoints].dblVal = dAvPageRespTime;

            strLabel.Format("%d", nElapsedTime+nTimeOffset);
            pVar[nElapsedTime].vt = VT_BSTR;
            pVar[nElapsedTime].bstrVal = strLabel.AllocSysString();

            if(nCurrentStat>=nNumStats)
                break;

            dwTotalRespTime = arHttpData[nCurrentStat]->dwResponseTime;
            dwCount=1;

            //Fill in any empty values
            while(++dwTime<dwCurrentTime)
            {
                nElapsedTime = dwTime-dwStartTime;

                pVar[nElapsedTime + dwNumPoints].vt = VT_EMPTY;

                strLabel.Format("%d", nElapsedTime+nTimeOffset);
                pVar[nElapsedTime].vt = VT_BSTR;
                pVar[nElapsedTime].bstrVal = strLabel.AllocSysString();
            }
        }
        nCurrentStat++;
    }

    saData.UnaccessData();

    m_pChart->SetData(saData);
    m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);

    // Set the data group name with the test run name.
    m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
	m_nNextGroup++;
    return CHttpGraph::AddTestRun(arHttpData,pUrls,dwTestStartTime);
}

CVUsersTimeGraph::CVUsersTimeGraph(CXChartCtrl *pChart)
	 : CGraphTypeBase(pChart)
{

    m_pChart->SetXAxisLabel("Elapsed Time");
    m_pChart->SetYAxisLabel("HTTP Active Users");
}

CVUsersTimeGraph::~CVUsersTimeGraph()
{

}

BOOL CVUsersTimeGraph::AddTestRun(VUList& VUs,DWORD dwTestStartTime)
{
	if(!VUs.GetCount())
		return false;

	CString strTestRun;
	strTestRun.Format("%d",m_nNextGroup);
	
	double dStartTime = VUs.GetHead()->dwTimeStamp;
    double dEndTime = VUs.GetTail()->dwTimeStamp;
	
    //For elapsed time graphs the data may need to be offset
    //so zero is the same for all graphs
    int nTimeOffset = (int)(dStartTime - dwTestStartTime)+1;

    int nElapsedTime =0;
    CString strLabel;

    COleSafeArray saData;

    int nNumSeconds = (int)(dEndTime-dStartTime+1);
    DWORD dwNumElements[]= {nNumSeconds, 2};
    saData.Create(VT_VARIANT,2,dwNumElements);

    VARIANT *pVar;
    saData.AccessData((void**)&pVar);

    POSITION pos = VUs.GetHeadPosition();
    for(double dTime=dStartTime;dTime<=dEndTime;dTime++)
    {
		VUData *pVUData = VUs.GetNext(pos);

		while(pVUData->dwTimeStamp>dTime)
        {
			//Fill with previous count
            pVar[nElapsedTime+nNumSeconds].vt = VT_EMPTY;
			
			strLabel.Format("%d", nElapsedTime+nTimeOffset);
			pVar[nElapsedTime].vt = VT_BSTR;
			pVar[nElapsedTime].bstrVal = strLabel.AllocSysString();

			dTime++;
			nElapsedTime++;
        }
		
		pVar[nElapsedTime+nNumSeconds].vt = VT_R8;
		pVar[nElapsedTime+nNumSeconds].dblVal = pVUData->dwCount;

        strLabel.Format("%d", nElapsedTime+nTimeOffset);
        pVar[nElapsedTime].vt = VT_BSTR;
        pVar[nElapsedTime].bstrVal = strLabel.AllocSysString();

		nElapsedTime++;
    }

    saData.UnaccessData();

    m_pChart->SetData(saData);
    m_pChart->SetSeriesStyle(m_nNextGroup,VtChSeriesType2dLine);
    m_pChart->SetSeriesAnnotation(m_nNextGroup,strTestRun);
	m_nNextGroup++;

    return true;
}
