/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CaptureOpenDlg.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "CaptureOpenDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "htmlhelp.h"
#include "Help\CaptureOpen.inl"

/////////////////////////////////////////////////////////////////////////////
// CCaptureOpenDlg dialog
extern CModellerApp theApp;

CCaptureOpenDlg::CCaptureOpenDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CCaptureOpenDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CCaptureOpenDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
}



void CCaptureOpenDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CCaptureOpenDlg)
    DDX_Control(pDX, IDC_LIST_CAPTURE, m_ctlList);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CCaptureOpenDlg, CDialog)
//{{AFX_MSG_MAP(CCaptureOpenDlg)
ON_NOTIFY(NM_DBLCLK, IDC_LIST_CAPTURE, OnDblclkListCapture)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCaptureOpenDlg message handlers

void CCaptureOpenDlg::OnOK()
{
    POSITION pos = m_ctlList.GetFirstSelectedItemPosition( ) ;
    if( pos )
    {
        int nItem = m_ctlList.GetNextSelectedItem( pos ) ;
		int nNdx = m_ctlList.GetItemData(nItem);
		m_strCapture = m_arstrCaptures[nNdx];
    }
    else
    {
        AfxMessageBox("Please select a capture from the list");
        return;
    }

    CDialog::OnOK();
}

void CCaptureOpenDlg::OnDblclkListCapture(NMHDR* pNMHDR, LRESULT* pResult)
{
    OnOK();

    *pResult = 0;
}

BOOL CCaptureOpenDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    CWaitCursor wc;

    InitList() ;
    FillList() ;

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


HBRUSH CCaptureOpenDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem(IDC_CAPTURE_PRESENT)->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

//---------------------------------------------------------------------------

void CCaptureOpenDlg::FillList()
{
    theApp.m_RepositoryAPI.GetProjectCaptureFiles( m_arstrCaptures ) ;

    for( int nIndex = 0 ; nIndex < m_arstrCaptures.GetSize() ; nIndex ++ )
    {
		CString strFile(m_arstrCaptures[nIndex]);
		int nSlashPos = strFile.ReverseFind('\\');
		if(nSlashPos==-1)
			nSlashPos = 0;
		strFile = strFile.Mid(nSlashPos+1);

		int nDotPos = strFile.ReverseFind('.');
		if(nDotPos == -1)
			nDotPos = 0;
		strFile = strFile.Left(nDotPos);

		int nNdx = m_ctlList.InsertItem( nIndex , strFile) ;
		m_ctlList.SetItemData(nNdx,nIndex);
	}
}

//---------------------------------------------------------------------------

BOOL CCaptureOpenDlg::InitList()
{
    CBitmap bmp ;

    //Image List
    if( ! m_imlNormal.Create( 16, 16, ILC_COLOR32|ILC_MASK, 0, 1 ) )
        return FALSE;

    bmp.LoadBitmap(IDB_SCL);
    m_imlNormal.Add( &bmp, RGB(0,128,128));
    bmp.DeleteObject();

    m_ctlList.SetImageList( &m_imlNormal ,LVSIL_SMALL ) ;

    return TRUE ;

}

BOOL CCaptureOpenDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("OPENSTA_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
