/* Copyright (C) 2000 Cyrano SA

This file is part of OpenSTA

OpenSTA is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with OpenSTA; see the file COPYING.  If not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

For more information about the GNU Public license see:
http://www.gnu.org/copyleft/gpl.html

OpenSTA is at:
http://www.opensta.org/
http://opensta.sourceforge.net/
*/

// ModellerDoc.cpp : implementation of the CModellerDoc class
//
#include "stdafx.h"
#include <IECache.h>
#include <CSerialization.h>
#include <ConstantesLib.h>
#include <SCLParser.h>
#include "CyrExecuteLocalCommand.h"
#include "Modeller.h"
#include "Constantes.h"
#include "SCLCommentsDlg.h"
#include "resource.h"
#include "VarPSheet.h"
#include "SCLGenerator.h"
#include "URLListDlg.h"
#include <PhParse.hpp>
#include "VariousObject.h"
#include "ModellerDoc.h"
#include "ChildFrm.h"
#include "CaptureSaveDlg.h"
#include "OutputFindView.h"
#include "RegFunctions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//#pragma warning( disable : 4786 )

extern CModellerApp     theApp ;

typedef int  (*fnTELibMain) (HWND hwind, char *TofFile, char *OdlDir);
typedef void (*fnTELibStop)();

/////////////////////////////////////////////////////////////////////////////
// CModellerDoc
//--------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CModellerDoc, CBasedDocument)

//--------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CModellerDoc, CBasedDocument)
//{{AFX_MSG_MAP(CModellerDoc)
ON_COMMAND(ID_CAPTURE_CHECK, OnCaptureCheck)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_CHECK, OnUpdateCaptureCheck)
ON_COMMAND(ID_CAPTURE_PAUSE, OnCapturePause)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_PAUSE, OnUpdateCapturePause)
ON_COMMAND(ID_CAPTURE_RECORD, OnCaptureRecord)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_RECORD, OnUpdateCaptureRecord)
ON_COMMAND(ID_CAPTURE_REPLAY, OnCaptureReplay)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_REPLAY, OnUpdateCaptureReplay)
ON_COMMAND(ID_CAPTURE_STOP, OnCaptureStop)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_STOP, OnUpdateCaptureStop)
ON_COMMAND(ID_SCL_COMMENT, OnSclComment)
ON_UPDATE_COMMAND_UI(ID_SCL_COMMENT, OnUpdateSclComment)
ON_COMMAND(ID_FILE_SAVE, OnFileSave)
ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
ON_UPDATE_COMMAND_UI(IDM_VARIABLE_CREATE, OnUpdateVariableCreate)
ON_COMMAND(IDM_VARIABLE_CREATE, OnVariableCreate)
ON_UPDATE_COMMAND_UI(IDM_VARIABLE_MODIFY, OnUpdateVariableModify)
ON_COMMAND(IDM_VARIABLE_MODIFY, OnVariableModify)
ON_UPDATE_COMMAND_UI(ID_VIEW_REFRESH, OnUpdateViewRefresh)
ON_COMMAND(ID_VIEW_REFRESH, OnViewRefresh)
ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////
// CModellerDoc construction/destruction
//--------------------------------------------------------------------------

CModellerDoc::CModellerDoc()
        :CBasedDocument(),
        m_pCaptureView(NULL),
        m_Gateway(this),
        m_bPause( FALSE ) ,
        m_bLocalRecord( FALSE ) ,
        m_bLocalReplay( FALSE ) ,
        m_hOutputReplayWnd(0),
        m_hReplayLibrary(NULL),
		m_pHtmlRootTag(NULL)
{
	//Get the path to the compiler
	TCHAR  szModellerPath[_MAX_PATH];
    TCHAR  szShortSCLCompilerPath[_MAX_PATH];
    char szdrive[_MAX_DRIVE];
    char szdir[_MAX_DIR];

    VERIFY(::GetModuleFileName(AfxGetApp()->m_hInstance, szModellerPath, _MAX_PATH));
    _splitpath( szModellerPath, szdrive, szdir, NULL, NULL );

    strcpy(szModellerPath,szdrive);
    strcat(szModellerPath,szdir);

    m_strSCLCompilerPath = GetExtendedProfileString(HKEY_LOCAL_MACHINE, theApp.GetRegistryPath(),
                         STR_SCL_COMPILER_ENTRY, szModellerPath);

	GetShortPathName( (LPCSTR)m_strSCLCompilerPath,
                      szShortSCLCompilerPath,
                      _MAX_PATH);
	
	m_strSCLCompilerPath = szShortSCLCompilerPath;
}

//--------------------------------------------------------------------------


CModellerDoc::~CModellerDoc()
{
    if (m_hReplayLibrary != NULL)
        FreeLibrary(m_hReplayLibrary);

	if(m_pHtmlRootTag)
	{
		free(m_pHtmlRootTag);
		m_pHtmlRootTag = NULL;
	}
}


//--------------------------------------------------------------------------

void CModellerDoc::SetCaptureView(CSCLCaptureScriptView* _pCaptureView)
{
    m_pCaptureView = _pCaptureView;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::Parse(BOOL _bForceParse)
{
	CString strScript = m_pCaptureView->m_Editor.GetWindowText() ;
	m_sclParser.Parse(strScript);
	return TRUE;
}

//--------------------------------------------------------------------------
void CModellerDoc::OnUpdateGoToURL(long _lLine, CCmdUI* pCmdUI )
{
    CString strDataFile = GetSCLDocInfo()->GetFullALLFileName() ;
    CIECache cache( strDataFile ) ;
    if( !cache.Init() )
    {
		pCmdUI->Enable(FALSE);
		return;
	}

	const CScriptItemInfo *pItem = m_sclParser.GetItemOnLine(_lLine);
	if(!pItem || (pItem->GetItemType() != CScriptItemInfo::enRequest))
	{
		pCmdUI->Enable(FALSE);
		return;
	}

	std::list<URLKey>   UrlList ;

	const CRequestInfo *pRequest = (const CRequestInfo*)pItem;
    if(!pRequest->GetUrl().IsEmpty())
		cache.QueryHTMLURL(pRequest->GetUrl(),pRequest->GetConnectionId(), UrlList ) ;

    pCmdUI->Enable(!IsRecording() && !theApp.m_bReplay && UrlList.size()>0);
}

//--------------------------------------------------------------------------

void CModellerDoc::GoToURL(long _lLine)
{
	Parse(TRUE);

    theApp.SetGlobalOffLine( TRUE ) ;

	//----
    //Init Cache
	CString strDataFile = GetSCLDocInfo()->GetFullALLFileName() ;
    CIECache cache( strDataFile ) ;
    if( !cache.Init() )
        return;

	int nLine = m_pCaptureView->m_Editor.GetCurrentLine();
	const CScriptItemInfo *pItem = m_sclParser.GetItemOnLine(nLine);
	if(!pItem || pItem->GetItemType() != CScriptItemInfo::enRequest)
		return;

	const CRequestInfo *pUrl =  (CRequestInfo *)pItem;

    DWORD               dwLength ;

    std::list<URLKey>   UrlList ;
    cache.QueryHTMLURL(pUrl->GetUrl(),pUrl->GetConnectionId(),UrlList);

	long lRequestID = 1;
	int nNumUrls = UrlList.size();
	switch(nNumUrls)
	{
	case 0:
		AfxMessageBox("Unable to retrieve the url",MB_ICONWARNING);
	return;
	case 1:
		//Do Nothing
	break;
	default:
        URLKey          key ;
        CURLListDlg     urllistdlg( UrlList, key ) ;
        //---
        if( urllistdlg.DoModal() == IDCANCEL )
            return ;
		lRequestID = key.m_dwRId;
	break;
	}

	//mark new url
	m_pCaptureView->MarkLine(pUrl->GetStartLine(),TRUE);

    //HTML View
	cache.CacheAllForPrimaryURL(pUrl->GetUrl(),pUrl->GetConnectionId(),lRequestID);
    CStringObject objURL(pUrl->GetUrl());
    UpdateAllViews( NULL , ID_DISPLAY_PRIMARY_URL , &objURL );

    //ClientHeaderView
    LPCSTR pcClientValue = cache.GetClientHeader(pUrl->GetUrl(), pUrl->GetConnectionId(), &dwLength , lRequestID);
    CString strClientHeader( pcClientValue, dwLength );
    CStringObject objClient( strClientHeader ) ;
    UpdateAllViews( NULL , ID_DISPLAY_CLIENTHEADER_URL , &objClient );

    //ServerHeaderView
    LPCSTR pcServerValue = cache.GetServerHeader(pUrl->GetUrl(), pUrl->GetConnectionId(), &dwLength , lRequestID);
    CString strServerHeader( pcServerValue, dwLength );
    CStringObject objServer( strServerHeader ) ;
    UpdateAllViews( NULL , ID_DISPLAY_SERVERHEADER_URL , &objServer );

    //DomView
    LPBYTE pBody = cache.GetBody(pUrl->GetUrl() , pUrl->GetConnectionId() , &dwLength , lRequestID );
	
	if(cache.IsHTMLURL(pUrl->GetUrl() , pUrl->GetConnectionId(), lRequestID))
	{
		InitTagParsing();
		PhLexer<LPBYTE> lex(pBody);
		PhParser<unsigned char*> Parser(lex,FALSE) ;
    
		free(m_pHtmlRootTag);
		m_pHtmlRootTag = NULL;

		Parser.Parse(m_pHtmlRootTag);
	}
	else
	{
		free(m_pHtmlRootTag);
		m_pHtmlRootTag = NULL;
	}

	UpdateAllViews( NULL , ID_DISPLAY_DOM_URL , NULL );
}


//--------------------------------------------------------------------------

void CModellerDoc::SetOutputReplayWnd(HWND _hOutputReplayWnd)
{
    m_hOutputReplayWnd = _hOutputReplayWnd;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::OnNewDocument()
{
    if (!CBasedDocument::OnNewDocument())
        return FALSE;

    return TRUE;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
    if (!CBasedDocument::OnOpenDocument(lpszPathName))
        return FALSE;

    return TRUE;
}

//-------------------------------------------------------------------------

BOOL CModellerDoc::OnSaveDocument(LPCTSTR lpszPathName)
{
    return CBasedDocument::OnSaveDocument(lpszPathName);
}

//--------------------------------------------------------------------------

void CModellerDoc::Serialize(CArchive& ar)
{
    if (ar.IsStoring())
    {}

    else
    {}

}

/////////////////////////////////////////////////////////////////////////////
// CModellerDoc diagnostics
//--------------------------------------------------------------------------

#ifdef _DEBUG
void CModellerDoc::AssertValid() const
{
    CBasedDocument::AssertValid();
}

//--------------------------------------------------------------------------

void CModellerDoc::Dump(CDumpContext& dc) const
{
    CBasedDocument::Dump(dc);
}
#endif //_DEBUG



//--------------------------------------------------------------------------

void CModellerDoc::OnCaptureCheck()
{
	Compile(FALSE);
}

BOOL CModellerDoc::Compile(BOOL bCompileDependencies)
{
    CWaitCursor wc;

    if(m_pCaptureView->m_Editor.IsModified())
    {
        CString strScript= m_pCaptureView->m_Editor.GetWindowText() ;

        CFile file ;
        CString strSCLFile = GetSCLDocInfo()->GetFullSCLFileName() ;
        if( file.Open( strSCLFile , CFile::modeCreate|CFile::modeWrite|CFile::shareExclusive ) )
        {
            file.Write( strScript , strScript.GetLength() ) ;
            file.Close() ;
        }
    }

    POSITION pos = GetFirstViewPosition();
    CChildFrame *pFrame = static_cast<CChildFrame *>(GetNextView(pos)->GetParentFrame());
    pFrame->ShowCheckWnd();

    UpdateAllViews( NULL , ID_SCL_CHECKER , &CStringObject("") )  ;

	CString strWorkingPath = GetSCLDocInfo()->GetWorkingFolder();
	CString strSCLFile = GetSCLDocInfo()->GetSCLFileName();
    
	BOOL bResult = CompileScript(strSCLFile,bCompileDependencies);

	//Delete all dependencies...
	CFileFind ff;
    BOOL bWorking = ff.FindFile(strWorkingPath + "*" + STR_SCL_EXTENSION);
	while(bWorking)
    {
        bWorking = ff.FindNextFile();

		if(ff.GetFileName().CompareNoCase(strSCLFile))
			DeleteFile(ff.GetFilePath());
    }
	
	//...if failed to compile delete all tofs as well
	if(!bResult)
	{
		bWorking = ff.FindFile(strWorkingPath + "*" + STR_TOF_EXTENSION);
		while(bWorking)
		{
			bWorking = ff.FindNextFile();
			DeleteFile(ff.GetFilePath());
		}
	}

	return bResult;
}

BOOL CModellerDoc::CompileScript(const CString& strScript,BOOL bCompileDependencies)
{
	//Compile this script
	CString strWorkingPath = GetSCLDocInfo()->GetWorkingFolder();

	CString strIncludeDir = GetRepositoryPath() + strIncludeFolder;
	CString strSourcePath = GetRepositoryPath() + strScriptType + "\\";
        
	CString strFullSCLFileName = strWorkingPath + strScript;
    CString strFullTOFFileName = strWorkingPath + strScript.SpanExcluding(".") + STR_TOF_EXTENSION;

    CString strSCLExecute;
	CString strCmd((LPCSTR) IDS_SCL_CHECKER);
    strSCLExecute.Format(strCmd, m_strSCLCompilerPath, strIncludeDir, strFullTOFFileName, strFullSCLFileName);

    //Execution
    CyrExecuteLocalCommand Checker(GetSCLDocInfo()->GetSCLPath()) ;

    CString strOutput;
    if( Checker.ExecuteCommand  ((CHAR*)((LPCTSTR)strSCLExecute) , strOutput) != 0 || !strOutput.IsEmpty())
	{
	    if(strOutput.IsEmpty())
			strOutput = "Unknown error compiling " + strScript;
		else
		{

			char sz[_MAX_PATH];

			GetShortPathName( (LPCSTR)strWorkingPath,sz,_MAX_PATH);
			strOutput.Replace(sz,"");
		}
		strOutput = "Compiling... " + strScript + "\n" + strOutput;
        CStringObject objOutput((CString)strOutput);
        UpdateAllViews( NULL , ID_SCL_CHECKER , &objOutput ) ;
		return false;
	}

	if(bCompileDependencies)
	{
		//Parse dependency file
		int nPos = strFullSCLFileName.ReverseFind('.');
		CString strDepFile = strFullSCLFileName.Left(nPos) + ".SCD";

		try
		{
			CStdioFile fDep(strDepFile,CFile::modeRead);
		
			CString strDep;
			while(fDep.ReadString(strDep))
			{
				strDep = strDep.SpanExcluding("|");
		
				if(strDep.ReverseFind('.')==-1)
					strDep += STR_SCL_EXTENSION;
				else
				if(strDep.Right(strlen(STR_SCL_EXTENSION)) != STR_SCL_EXTENSION)
					continue;

				CFileStatus fs;
				if(CFile::GetStatus(strSourcePath + strDep,fs))
				{
					if(!CopyFile(strSourcePath + strDep,strWorkingPath+strDep,TRUE))
					{
						CString strMessage = "WARNING: Possible Cyclic Dependency " + strDep + " Is called multiple times.\n";
						CStringObject objOutput((CString)strMessage);
						UpdateAllViews( NULL , ID_SCL_CHECKER , &objOutput ) ;

						continue;
					}

					if(!CompileScript(strDep,bCompileDependencies))
						return false;
				}
				else
				{
					CString strMessage = "Compiling... " + strScript + "\n" + strScript + " Failed to compile...\tDependency script " + strDep + " could not be found.\n";
					CStringObject objOutput((CString)strMessage);
					UpdateAllViews( NULL , ID_SCL_CHECKER , &objOutput ) ;
					return false;
				}
			}

			fDep.Close();
		}
		catch(...)
		{
			return false;
		}
	}

    UpdateAllViews( NULL , ID_SCL_CHECKER , &CStringObject("Compiling... " + strScript + "\nSuccessfully Compiled " + strScript +"\n") )  ;
    
	return true;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCaptureCheck(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( !theApp.m_bReplay && !theApp.m_bRecording ) ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnCapturePause()
{
    m_Gateway.PauseRecord() ;
    m_bPause = !m_bPause ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCapturePause(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( theApp.m_bRecording && m_bLocalRecord) ;
    pCmdUI->SetCheck(m_bPause ? 1 : 0);
}

//--------------------------------------------------------------------------

void CModellerDoc::OnCaptureRecord()
{
    GetSCLDocInfo()->SetBrowserID(theApp.m_nBrowserID);
    if(!m_Gateway.GetDefaultParameters(&theApp.m_GatewayData.m_ProxyInfo.m_LocalProxy))
        return;

    CWaitCursor wc;
    
    if(GetSCLDocInfo()->GetSCLFileName() == UNTITLED_SCL_DOCUMENT)
    {
        CCaptureSaveDlg dlg(CCaptureSaveDlg::enScriptName);
        if(dlg.DoModal()!=IDOK)
            return;

		GetSCLDocInfo()->SetOriginalPath(dlg.m_strScriptName);

		CString strScript(dlg.m_strScriptName);
		int nSlashPos = strScript.ReverseFind('\\');
		strScript = strScript.Mid(nSlashPos + 1);
        GetSCLDocInfo()->SetFileInfo(strScript,GetSCLDocInfo()->GetBrowserID());

		SetTitle(strScript);
    }

    if(m_pCaptureView->m_Editor.IsModified())
    {
        int nResult = AfxMessageBox(IDS_SAVE_MODIFIED_SCRIPT,MB_YESNOCANCEL);
        if(nResult == IDCANCEL)
            return;
        else
	    if(nResult == IDYES)
            DoSave(NULL);
    }

	m_pCaptureView->SetText("",TRUE) ;

    theApp.SetGlobalOffLine( FALSE ) ;
    theApp.m_bRecording = m_Gateway.Record(theApp.m_bConsole,theApp.m_bTrace,theApp.m_bAutoCookieGeneration) ;
    m_bLocalRecord      = theApp.m_bRecording;

	AfxBeginThread(&WaitForBrowserThread,this);
}

//Sit here and wait for the browser to terminate
UINT CModellerDoc::WaitForBrowserThread(LPVOID pParam)
{
	WaitForSingleObject(theApp.m_ProcessInformation.hProcess,INFINITE);

	CWnd *pMainWnd = AfxGetMainWnd();
	if(pMainWnd)
		pMainWnd->SendMessage(WM_COMMAND,MAKEWPARAM(ID_CAPTURE_STOP,0),0);//Browser was stopped
	theApp.m_CaptureGateway.stopGateway();
	return 0;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCaptureRecord(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( !theApp.m_bRecording && !theApp.m_bReplay ) ;
}


//--------------------------------------------------------------------------

void CModellerDoc::OnCaptureReplay()
{
    CWaitCursor wc;
    //---
    if(m_pCaptureView->m_Editor.IsModified())
    {
        CString strScript= m_pCaptureView->m_Editor.GetWindowText() ;

        CFile file ;
        CString strSCLFile = GetSCLDocInfo()->GetFullSCLFileName() ;
        if( file.Open( strSCLFile , CFile::modeCreate|CFile::modeWrite|CFile::shareExclusive ) )
        {
            file.Write( strScript , strScript.GetLength() ) ;
            file.Close() ;
        }
    }

    POSITION pos = GetFirstViewPosition();
    CChildFrame *pFrame = static_cast<CChildFrame *>(GetNextView(pos)->GetParentFrame());
    pFrame->ShowReplayWnd();

    //clear replay output window
    UpdateAllViews( NULL , ID_REPLAY_SCL ,&CStringObject("")) ;

    //1) chemin du fichier SCL
    CString strFullTofFileName ;
    CString strFullSclFileName ;

    strFullTofFileName = GetSCLDocInfo()->GetFullTOFFileName();
    strFullSclFileName = GetSCLDocInfo()->GetFullSCLFileName();

    //Do the compile
    if(!Compile(TRUE))
        return;   //Compile failed

	//Select the replay window again
	pFrame->ShowReplayWnd();

    {
        TCHAR  szModellerPath[_MAX_PATH];
        TCHAR  szShortModellerPath[_MAX_PATH];
        char szdrive[_MAX_DRIVE];
        char szdir[_MAX_DIR];


        // ----
        VERIFY(::GetModuleFileName(AfxGetApp()->m_hInstance, szModellerPath, _MAX_PATH));

        _splitpath( szModellerPath, szdrive, szdir, NULL, NULL );

        strcpy(szModellerPath,szdrive);
        strcat(szModellerPath,szdir);

        GetShortPathName( szModellerPath,
                          szShortModellerPath,
                          _MAX_PATH);

        theApp.m_bReplay = TRUE;
        m_bLocalReplay   = TRUE;
        try
        {
            if ( m_hReplayLibrary == NULL )
            {
                m_hReplayLibrary = LoadLibrary(
#ifdef _DEBUG
                    "TEHttpLibd.dll"
#else
                    "TEHttpLib.dll"
#endif
                );
                fnTELibMain pfnTELibMain = (fnTELibMain) GetProcAddress(m_hReplayLibrary, "TELibMain");
                if(pfnTELibMain)
                    pfnTELibMain(m_hOutputReplayWnd, (CHAR*)((LPCTSTR)strFullTofFileName), szShortModellerPath);
            }
        }
        catch(CTException &)
        {
            FreeLibrary(m_hReplayLibrary);
            m_hReplayLibrary = NULL;
        }

    }
}

//--------------------------------------------------------------------------

void CModellerDoc::StopReplay()
{
    try
    {
        theApp.m_bReplay = FALSE;
        m_bLocalReplay   = FALSE;

        fnTELibStop pfnTELibStop = (fnTELibStop) GetProcAddress(m_hReplayLibrary, "TELibStop");
        if(pfnTELibStop)
            pfnTELibStop();
    }
    catch(const CTException &)
    {
        try
        {
            if(m_hReplayLibrary != NULL)
                FreeLibrary(m_hReplayLibrary);
        }
        catch(...)
        {
        }

        m_hReplayLibrary = NULL;
    }
}

//---------------------------------------------------------------------------

void CModellerDoc::OnReplayStopped()
{
    theApp.m_bReplay = FALSE;
    m_bLocalReplay   = FALSE;

    if(m_hReplayLibrary != NULL)
    {
        FreeLibrary(m_hReplayLibrary);
        m_hReplayLibrary = NULL;
    }
}


//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCaptureReplay(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(  !theApp.m_bRecording && !theApp.m_bReplay ) ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnCaptureStop()
{
    if ( theApp.m_bRecording)
    {
        CWaitCursor wc;
        //---
        theApp.m_bRecording = FALSE;
        m_bLocalRecord      = FALSE;
        if( m_Gateway.StopRecord() )
        {
            CString strScript;
            //----
            m_Gateway.GetSCLRecordedScript(strScript) ;

            m_pCaptureView->SetText(strScript,TRUE) ;
			Parse(TRUE);
        }
    }
    else
        StopReplay();

    m_bPause = false;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCaptureStop(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( (theApp.m_bRecording && m_bLocalRecord ) || ( theApp.m_bReplay && m_bLocalReplay)) ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateViewRefresh(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//--------------------------------------------------------------------------

void CModellerDoc::OnViewRefresh()
{
    Parse(TRUE);
}

//--------------------------------------------------------------------

void CModellerDoc::OnUpdateVariableCreate(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(!theApp.m_bRecording && !theApp.m_bReplay);
}

//--------------------------------------------------------------------

void CModellerDoc::OnVariableCreate()
{
    CVarInfo    VarInfo ;
    VarInfo.m_VarGeneral.m_strVarName = theApp.m_strVariablePrefix;
    CVarPSheet WizardDlg( &VarInfo, TRUE, TRUE) ;

    if( WizardDlg.DoModal() == IDCANCEL )
        return ;

	int nLine = m_sclParser.GetVariableInsertLine();

    CString     strLineToInsert ;
	GenerateVariable(VarInfo,strLineToInsert);
    m_pCaptureView->m_Editor.InsertString( nLine, strLineToInsert ) ;

	Parse();
}

//--------------------------------------------------------------------

void CModellerDoc::GenerateVariable(CVarInfo &VarInfo,CString& strLineToInsert)
{
    CString strFileNameInScript ;
    CString strFVRFileFullPath ;

    if( VarInfo.m_VarGeneral.m_lSourceInfo == VAR_SOURCE_FILE )
    {// FILE
        CString strValue ;
        int     nPos ;

        //
        nPos     = VarInfo.m_VarFile.m_strFilePath.ReverseFind( '\\' ) ;
		
        if( nPos >= 0 )
        {
            strValue = VarInfo.m_VarFile.m_strFilePath.Mid( nPos + 1 ) ;
        }
		else 
			strValue= VarInfo.m_VarFile.m_strFilePath;


        nPos     = strValue.ReverseFind( '.' );
        if( nPos >= 0 )
        {
            strValue = strValue.Left( nPos ) ;
        }

        strFileNameInScript = strValue ;
        strFVRFileFullPath  = GetSCLDocInfo()->GetFVRPath() + strValue + STR_FVR_EXTENSION;
		
        CopyFile( VarInfo.m_VarFile.m_strFilePath, strFVRFileFullPath, FALSE ) ;

        strValue = VarInfo.m_VarFile.m_strFilePath ;


        strLineToInsert = CSCLGenerator::GenVarFileDecl( VarInfo.m_VarGeneral.m_strVarName,
                          strFileNameInScript,
                          VarInfo.m_VarGeneral.m_lVarType,
                          VarInfo.m_VarGeneral.m_lVarScope ) ;

    }
    else if (VarInfo.m_VarValueList.m_bGenerateFile)
    {//GENERATE IN FILE
        try
        {
            strFVRFileFullPath  = GetSCLDocInfo()->GetFVRPath() + VarInfo.m_VarGeneral.m_strVarName + STR_FVR_EXTENSION;
            strFVRFileFullPath.MakeUpper() ;

            CStdioFile FVRFile( strFVRFileFullPath, CFile::modeCreate
                                | CFile::modeWrite | CFile::typeText ) ;

            for( int i=0; i < VarInfo.m_VarValueList.m_Values.GetSize(); i++ )
                FVRFile.WriteString( VarInfo.m_VarValueList.m_Values.GetAt(i) + "\n" ) ;

            FVRFile.Close() ;
            strFileNameInScript = VarInfo.m_VarGeneral.m_strVarName ;
        }
        catch( CFileException e )
        {}


        strLineToInsert = CSCLGenerator::GenVarFileDecl( VarInfo.m_VarGeneral.m_strVarName,
                          strFileNameInScript,
                          VarInfo.m_VarGeneral.m_lVarType,
                          VarInfo.m_VarGeneral.m_lVarScope ) ;
    }
    else
    {// VALUE LIST
        strLineToInsert = CSCLGenerator::GenVarValListDecl(  VarInfo.m_VarGeneral.m_strVarName,
                          &(VarInfo.m_VarValueList.m_Values),
                          VarInfo.m_VarGeneral.m_lVarOrder,
                          VarInfo.m_VarGeneral.m_lVarType,
                          VarInfo.m_VarGeneral.m_lVarScope ) ;
    }
}

//--------------------------------------------------------------------

void CModellerDoc::OnUpdateVariableModify(CCmdUI* pCmdUI)
{
    int nLine = m_pCaptureView->m_Editor.GetCurrentLine();
	const CScriptItemInfo *pItem = m_sclParser.GetItemOnLine(nLine);

	BOOL bVariableOnLine = (pItem && (pItem->GetItemType() == CScriptItemInfo::enVariable));

    pCmdUI->Enable(bVariableOnLine && !theApp.m_bRecording && !theApp.m_bReplay);
}

//--------------------------------------------------------------------

void CModellerDoc::OnVariableModify()
{
	Parse(TRUE);

    int nLine= m_pCaptureView->m_Editor.GetCurrentLine();
	const CScriptItemInfo *pItem = m_sclParser.GetItemOnLine(nLine);
	if(!pItem || (pItem->GetItemType() != CScriptItemInfo::enVariable))
		return;
	
	CVariableInfo *pVariable = (CVariableInfo*)pItem;
    
	int nStartLine = pVariable->GetStartLine();
	int nEndLine = pVariable->GetLastLine();

	CVarInfo VarInfo;
	GetVarInfoFromVariable(pVariable,VarInfo);
    
    CVarPSheet WizardDlg( &VarInfo, FALSE, VarInfo.m_VarGeneral.m_lSourceInfo == VAR_SOURCE_FILE ) ;

    if( WizardDlg.DoModal() == IDCANCEL )
        return ;

    m_pCaptureView->m_Editor.DeleteLines( nStartLine, nEndLine );

    CString     strLineToInsert ;
	GenerateVariable(VarInfo,strLineToInsert);
    m_pCaptureView->m_Editor.InsertString(nStartLine,strLineToInsert);
}

//--------------------------------------------------------------------------
void CModellerDoc::GetVarInfoFromVariable(CVariableInfo *pVariable,CVarInfo& VarInfo)
{
	CString strFileName;
	if(pVariable->GetFilename(strFileName))
	{
		VarInfo.m_VarGeneral.m_lSourceInfo = VAR_SOURCE_FILE;
		VarInfo.m_VarFile.m_strFilePath =strFileName;
	}
	else
		VarInfo.m_VarGeneral.m_lSourceInfo = VAR_SOURCE_LIST;

    VarInfo.m_VarFile.m_strFVRPath = GetSCLDocInfo()->GetFVRPath() ;
    CString &strFile = VarInfo.m_VarFile.m_strFilePath;
    if(strFile.Right(strlen(STR_FVR_EXTENSION)).CompareNoCase(STR_FVR_EXTENSION)!=0)
        strFile += STR_FVR_EXTENSION;

	VarInfo.m_VarGeneral.m_lVarOrder = (pVariable->IsRandom() ? VAR_ORDER_RANDOM : VAR_ORDER_SEQUENTIAL);
	switch(pVariable->GetScope())
	{
	case CVariableInfo::enLocal:	VarInfo.m_VarGeneral.m_lVarScope = VAR_SCOPE_LOCAL;		break;
	case CVariableInfo::enThread:	VarInfo.m_VarGeneral.m_lVarScope = VAR_SCOPE_THREAD;	break;	
	case CVariableInfo::enScript:	VarInfo.m_VarGeneral.m_lVarScope = VAR_SCOPE_SCRIPT;	break;
	case CVariableInfo::enGlobal:	VarInfo.m_VarGeneral.m_lVarScope = VAR_SCOPE_GLOBAL;	break;
	}
	
	switch(pVariable->GetType())
	{
	case CVariableInfo::enCharacter:	VarInfo.m_VarGeneral.m_lVarType = VAR_DATATYPE_CHAR;	break;
	case CVariableInfo::enInteger:		VarInfo.m_VarGeneral.m_lVarType = VAR_DATATYPE_INT;		break;
	}
	
	pVariable->GetName(VarInfo.m_VarGeneral.m_strVarName);
	pVariable->GetInitialisationValues(VarInfo.m_VarValueList.m_Values);
}

//--------------------------------------------------------------------------

void CModellerDoc::OnSclComment()
{
    CString strComments ;
    //----
    CSCLCommentsDlg dlg( &strComments ) ;
    //---
    int nRet = dlg.DoModal() ;
    if( nRet == IDOK )
        m_Gateway.InsertComments( strComments ) ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateSclComment(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( IsRecording() && !m_bPause && m_bLocalRecord) ;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::IsRecording()
{
    return theApp.m_bRecording ;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::RetrieveFromRepository()
{
    CWaitCursor wc;

    CString strSCLFile = GetSCLDocInfo()->GetSCLFileName();
    if(strSCLFile == UNTITLED_SCL_DOCUMENT)
        return true;

	CString strWorkingPath = GetSCLDocInfo()->GetWorkingFolder();

	CString strSCLPath = GetSCLDocInfo()->GetOriginalPath();
    CFile file ;
    if( file.Open( strSCLPath, CFile::modeRead|CFile::shareDenyWrite) )
    {
        //check browser
        CObjScript obscript;

        int nCount = file.GetLength() ;
        TCHAR *pczBuffer = new TCHAR[nCount + 1 ] ;
        file.Read( pczBuffer , nCount ) ;
        pczBuffer[nCount] = '\0' ;
        if( pczBuffer )
        {
            obscript.m_strScript = pczBuffer ;
            delete pczBuffer ;
        }
        file.Close() ;
        //----
        obscript.m_bModify = FALSE;

        InitBrowserFromScript( &obscript) ;

        //Copy capture to working folder
        CopyFile(strSCLPath,strWorkingPath + GetSCLDocInfo()->GetSCLFileName(),false);

        CString strAllFile = GetRepositoryPath() + strCaptureFolder + "\\" + GetSCLDocInfo()->GetSCLFileName();
        int nPos = strAllFile.ReverseFind('.');
        strAllFile.Delete(nPos,strlen(STR_SCL_EXTENSION));
        strAllFile += STR_ALL_EXTENSION;
		
        CopyFile(strAllFile,strWorkingPath + GetSCLDocInfo()->GetALLFileName(),false);

        m_pCaptureView->SetText(obscript.m_strScript, obscript.m_bModify) ;
        Parse(TRUE);
    }

	CString strDataPath = GetRepositoryPath() + strDataFolder +"\\";
    
    CFileFind FileFinder;
    BOOL bWorking = FileFinder.FindFile(strDataPath + "*" + STR_FVR_EXTENSION);
    while(bWorking)
    {
        bWorking = FileFinder.FindNextFile();
        CopyFile(FileFinder.GetFilePath(),strWorkingPath + FileFinder.GetFileName(),FALSE);
    }

    return TRUE;
}

//--------------------------------------------------------------------------

void CModellerDoc::InitBrowserFromScript( CObjScript *pObjScript )
{
	if(pObjScript->m_strScript.IsEmpty())
		return;

    CWaitCursor wc;

    long lID;

    lID = ID_BROWSER_NONE;

    //Browser Type
    if( pObjScript->m_strScript.Find( STR_BROWSER_IE4 ) >= 0 )
        lID = ID_BROWSER_IE4 ;
    else
    {
        if( pObjScript->m_strScript.Find( STR_BROWSER_IE5 ) >= 0 )
            lID = ID_BROWSER_IE5 ;
        else
        {
            if( pObjScript->m_strScript.Find( STR_BROWSER_NS47 ) >= 0 )
                lID = ID_BROWSER_NS47 ;
            else
            {
                SetDefaultBrowser(pObjScript) ;
                return ;
            }
        }
    }

    GetSCLDocInfo()->SetBrowserID(lID) ;
}


//--------------------------------------------------------------------------

void CModellerDoc::SetDefaultBrowser(CObjScript *pObjScript)
{
	CString strText ;
	CString strWarning ;
    //---
    strWarning.LoadString( IDS_WARNING ) ;
    strText.LoadString( IDS_NO_BROWSER_SELECTED ) ;
    AfxGetMainWnd()->MessageBox( strText , strWarning , MB_OK ) ;
    //---
    CFile File ;
    CString strSCLFile ;
    //---

    strSCLFile = GetSCLDocInfo()->GetFullSCLFileName() ;


    if( File.Open( strSCLFile , CFile::shareExclusive|CFile::modeReadWrite) )
    {
        TCHAR *pczBrowser = NULL ;
        int nCount = File.GetLength() ;
        TCHAR *pczBufferRead = new TCHAR[nCount + 1] ;
        //----
        File.SeekToBegin( );
        File.Read( pczBufferRead , nCount ) ;
        pczBufferRead[nCount] = '\0' ;

        //Write Browser : default IE5
        pczBrowser = new TCHAR[ strlen( STR_BROWSER_IE5 ) + 3] ;
        strcpy( pczBrowser , STR_BROWSER_IE5 ) ;
        strcat( pczBrowser , _T("\r\n") ) ;

        if( pczBrowser )
            nCount += strlen( pczBrowser ) ;

        //write new Buffer
        TCHAR *pczBufferWrite = new TCHAR[nCount + 1] ;
        //---
        if( pczBrowser )
            strcpy( pczBufferWrite , pczBrowser ) ;
        if( pczBufferRead )
            strcat( pczBufferWrite , pczBufferRead ) ;

        File.SeekToBegin( );
        File.Write( pczBufferWrite ,nCount ) ;

        File.Close() ;

        pObjScript->m_strScript = pczBufferWrite ;
        pObjScript->m_bModify = TRUE;
        GetSCLDocInfo()->SetBrowserID(ID_BROWSER_IE5) ;

        if(pczBufferWrite )
            delete pczBufferWrite ;
        if(pczBufferRead )
            delete pczBufferRead ;
        if(pczBrowser )
            delete pczBrowser ;

    }
}

//--------------------------------------------------------------------------

void CModellerDoc::OnFileSave()
{
    CWaitCursor wc;

    if(GetSCLDocInfo()->GetSCLFileName() == UNTITLED_SCL_DOCUMENT)
    {
        OnFileSaveAs();
        return;
    }
	Parse(TRUE);

    CBasedDocument::OnFileSave();
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateFileSave(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( !IsRecording() ) ;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::DoSave(LPCTSTR lpszPathName, BOOL bReplace)
{
    if(GetSCLDocInfo()->GetSCLFileName() == UNTITLED_SCL_DOCUMENT)
    {
        OnFileSaveAs();
        return true;
    }
	else
		return SaveToRepository();
}


//--------------------------------------------------------------------------

BOOL CModellerDoc::MoveFilesToRepositoryStorage()
{
    CWaitCursor wc;
    BOOL bResult = TRUE;
    
    //SCL File
    CString strSCLFromFullPath = GetSCLDocInfo()->GetFullSCLFileName();
	CString strSCLToFullPath = GetSCLDocInfo()->GetOriginalPath();
    bResult &= CopyFile(strSCLFromFullPath , strSCLToFullPath, FALSE ) ;

    //All File
	CString strAllFromFullPath = GetSCLDocInfo()->GetFullALLFileName();
    
	//If all file does not exist for any reason create an empty one
	CFileStatus fs;
	if(!CFile::GetStatus(strAllFromFullPath,fs))
		CFile(strAllFromFullPath,CFile::modeCreate);

    CString strAllToFullPath = GetRepositoryPath() + strCaptureFolder + "\\";
	strAllToFullPath += GetSCLDocInfo()->GetALLFileName();
    bResult &= CopyFile(  strAllFromFullPath , strAllToFullPath, FALSE);

    CString strDataPath = GetRepositoryPath() + strDataFolder +"\\";
    
	//FVR Files
    CFileFind FileFinder;
    BOOL bWorking = FileFinder.FindFile(GetSCLDocInfo()->GetWorkingFolder() + "*" + STR_FVR_EXTENSION);
    while(bWorking)
    {
        bWorking = FileFinder.FindNextFile();
        bResult &= CopyFile(FileFinder.GetFilePath(),strDataPath + FileFinder.GetFileName(),FALSE);
    }

    return bResult ;
}

//--------------------------------------------------------------------

BOOL CModellerDoc::SaveToRepository()
{
    CWaitCursor wc;
    CString strScript ;

    strScript = m_pCaptureView->m_Editor.GetWindowText() ;
    m_pCaptureView->m_Editor.SetModified( FALSE) ;

    Parse();

    //Save new script int scl file
    CFile file ;
    //---

    CString strSCLFile = GetSCLDocInfo()->GetSCLFileName() ;
    CString strFullPath = GetSCLDocInfo()->GetWorkingFolder() + strSCLFile ;
    if( file.Open( strFullPath , CFile::modeCreate|CFile::modeWrite|CFile::shareExclusive ) )
    {
        file.Write( strScript , strScript.GetLength() ) ;
        file.Close() ;
    }
  
    MoveFilesToRepositoryStorage();

    return TRUE;
}

//--------------------------------------------------------------------

void CModellerDoc::SetErrorLine( int lRow )
{
    UpdateAllViews( NULL , ID_CHECKER_ERROR , &CLongObject(lRow) ) ;
}

//--------------------------------------------------------------------

BOOL CModellerDoc::SaveModified()
{
    if(IsRecording())
    {
        AfxMessageBox("You cannot close the Modeller while a capture is in progress.Select the stop capture command before closing the Modeller.",MB_OK);
        return false;
    }

    CWaitCursor wc;

    SetModifiedFlag(m_pCaptureView->m_Editor.IsModified());

    if(IsModified())
    {
        int nResult = AfxMessageBox("You have made changes to this script\nDo you want to save them now?",MB_YESNOCANCEL);
        if(nResult == IDCANCEL)
            return 0;
        else
            if(nResult == IDNO)
                return 1;
            else
            {
                DoSave(NULL);
                return 1;
            }
    }
    else
        return CBasedDocument::SaveModified();
}


//--------------------------------------------------------------------

void CModellerDoc::OnCloseDocument()
{
    theApp.GetRepositoryAPI()->SaveProjectCaptureFiles( GetSCLDocInfo()->GetSCLFileName() , true  ) ;

    DeleteWorkingDirectory();
    theApp.UpdateWindowText();

    CBasedDocument::OnCloseDocument();
}

//--------------------------------------------------------------------

void CModellerDoc::Adressing( CVarResponse * _pVarResponse )
{
    Parse();

	const CString strMsg("Addressing has failed.\nCannot find original request");
	int nCurrentLine = m_pCaptureView->GetMarkedLine();
	if(nCurrentLine == -1)
	{
		AfxMessageBox(strMsg);
		return;
	}

	const CScriptItemInfo *pItem = m_sclParser.GetItemOnLine(nCurrentLine);
	if(!pItem || (pItem->GetItemType() != CScriptItemInfo::enRequest))
	{
		AfxMessageBox(strMsg);
		return;
	}

	const CRequestInfo *pUrl = (CRequestInfo*)pItem;
    _pVarResponse->m_lConId = pUrl->GetConnectionId();

    //statement
    CString strLineToInsert = CSCLGenerator::GenVarResponseInfoStat( _pVarResponse ) ;
	long lineToInsert = pUrl->GetLastLine() + 2;

    m_pCaptureView->m_Editor.InsertString( lineToInsert, strLineToInsert ) ;

    //declaration
    strLineToInsert = CSCLGenerator::GenVarResponseInfoDecl( _pVarResponse->m_strVarName ) ;
    lineToInsert    = m_sclParser.GetVariableInsertLine();
    m_pCaptureView->m_Editor.InsertString( lineToInsert, strLineToInsert) ;
}

//--------------------------------------------------------------------

void CModellerDoc::OnFileSaveAs()
{
    CCaptureSaveDlg dlg;
    if(dlg.DoModal() == IDOK)
    {
        CWaitCursor wc;

        CString strSCLFile = dlg.m_strScriptName;
		
        //Get the location of the existing all file and fvr files
        CString strWorkingFolder = GetSCLDocInfo()->GetWorkingFolder();

		int nSlashPos = strSCLFile.ReverseFind('\\');
		CString strNewFile = strSCLFile.Mid(nSlashPos+1);

		CString strNewSclFile = strWorkingFolder + strNewFile;
        CString strOldSclFile = strWorkingFolder + GetSCLDocInfo()->GetSCLFileName();

		CFileStatus fs;
		if(!CFile::GetStatus(strOldSclFile,fs))
			CFile(strOldSclFile,CFile::modeCreate);

		CFile::Rename(strOldSclFile,strNewSclFile);

        CString strOldAllFile = strWorkingFolder + GetSCLDocInfo()->GetALLFileName();

		//If all file does not exist for any reason create an empty one
		if(!CFile::GetStatus(strOldAllFile,fs))
			CFile(strOldAllFile,CFile::modeCreate);

		int nDotPos = strNewSclFile.ReverseFind('.');
		if(nDotPos == -1)
			nDotPos = 0;

        CString strNewAllFile = strNewSclFile.Left(nDotPos);
        strNewAllFile += STR_ALL_EXTENSION;
        CFile::Rename(strOldAllFile,strNewAllFile);

        //Setup new file information
        GetSCLDocInfo()->SetOriginalPath(strSCLFile) ;
		GetSCLDocInfo()->SetFileInfo(strNewFile);

        SetTitle(strNewFile);

        //This will create all directories we need
        MoveFilesToRepositoryStorage();

        SaveToRepository();
    }
	Parse(TRUE);
}

void CModellerDoc::OnUpdateFileSaveAs(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( !IsRecording() ) ;
}

void CModellerDoc::DeleteWorkingDirectory()
{
    CString strWorkingDir = GetSCLDocInfo()->GetWorkingFolder();

    CFileFind *pFileFind = new CFileFind;
    if(pFileFind->FindFile(strWorkingDir + "*"))
    {
        BOOL bQuit = false;
        do
        {
            bQuit = !pFileFind->FindNextFile();

            if(pFileFind->IsDots())
                continue;

            if(pFileFind->IsDirectory())
                continue;

            DeleteFile(pFileFind->GetFilePath());

        }
        while(!bQuit);
    }
    delete pFileFind;
    pFileFind = NULL;

    RemoveDirectory(strWorkingDir);
}

