/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"
#include "VarDBColumnList.h"

/////////////////////////////////////////////////////////////////////////////
// CVarDBColumnList implementation
//---------------------------------------------------------------------------

CVarDBColumnList::CVarDBColumnList(CDatabase* pDatabase)
        : CDBTable(pDatabase)
{}

//---------------------------------------------------------------------------


CVarDBColumnList::~CVarDBColumnList()
{
    Reset();
}

//---------------------------------------------------------------------------

void CVarDBColumnList::Reset()
{
    Free();

    for (int i = 0; i < m_aData.GetSize(); i++)
        delete m_aData.GetAt(i);

    m_aData.RemoveAll();

}

//---------------------------------------------------------------------------

BOOL CVarDBColumnList::Load(CString _strTableName, CString _strQualifierParam, CString _strOwnerParam)
{
    ASSERT(!_strTableName.IsEmpty());

    CWaitCursor crwait;

    SQLRETURN                   ReturnCode ;
    UDWORD                      RowsFetched ;
    UWORD*                      pRowStatus ;
    pFF_DB_COLUMN_SET_FIELDS    pHostData ;

    ASSERT(m_nRowSetSize > 0) ;

    pRowStatus = new UWORD[m_nRowSetSize] ;
    ASSERT(pRowStatus);

    pHostData = new FF_DB_COLUMN_SET_FIELDS[m_nRowSetSize] ;
    ASSERT(pHostData) ;

    OpenS() ;
    Reset() ; // Clear everything

    // Prepare the statement for bulk operation
    SQLSetStmtOption(m_hstmtS,SQL_BIND_TYPE,sizeof(FF_DB_COLUMN_SET_FIELDS));
    SQLSetStmtOption(m_hstmtS,SQL_CONCURRENCY,SQL_CONCUR_READ_ONLY);
    SQLSetStmtOption(m_hstmtS,SQL_ROWSET_SIZE,m_nRowSetSize) ;

    ReturnCode = ::SQLColumns(m_hstmtS,
                              (_strQualifierParam.IsEmpty()? (UCHAR FAR *)NULL: (UCHAR FAR *)(const char*)_strQualifierParam), SQL_NTS,
                              (_strOwnerParam.IsEmpty()? (UCHAR FAR *)NULL: (UCHAR FAR *)(const char*)_strOwnerParam), SQL_NTS,
                              (_strTableName.IsEmpty()? (UCHAR FAR *)NULL: (UCHAR FAR *)(const char*)_strTableName), SQL_NTS,
                              NULL, SQL_NTS);


    SQLBindCol(m_hstmtS,  1, SQL_C_CHAR,   pHostData[0].szCatalog, STR_LEN,&(pHostData[0].CatalogInd) );
    SQLBindCol(m_hstmtS,  2, SQL_C_CHAR,   pHostData[0].szSchema, STR_LEN, &(pHostData[0].SchemaInd) );
    SQLBindCol(m_hstmtS,  3, SQL_C_CHAR,   pHostData[0].szTableName, STR_LEN,&(pHostData[0].TableNameInd) );
    SQLBindCol(m_hstmtS,  4, SQL_C_CHAR,   pHostData[0].szColumnName, STR_LEN, &(pHostData[0].ColumnNameInd) );
    SQLBindCol(m_hstmtS,  5, SQL_C_SSHORT, &(pHostData[0].DataType), 0, &(pHostData[0].DataTypeInd) );
    SQLBindCol(m_hstmtS,  6, SQL_C_CHAR,   pHostData[0].szTypeName, STR_LEN, &(pHostData[0].TypeNameInd) );
    SQLBindCol(m_hstmtS,  7, SQL_C_SLONG,  &(pHostData[0].ColumnSize), 0, &(pHostData[0].ColumnSizeInd) );
    SQLBindCol(m_hstmtS,  8, SQL_C_SLONG,  &(pHostData[0].BufferLength), 0, &(pHostData[0].BufferLengthInd) );
    SQLBindCol(m_hstmtS,  9, SQL_C_SSHORT, &(pHostData[0].DecimalDigits), 0, &(pHostData[0].DecimalDigitsInd) );
    SQLBindCol(m_hstmtS, 10, SQL_C_SSHORT, &(pHostData[0].NumPrecRadix), 0, &(pHostData[0].NumPrecRadixInd) );
    SQLBindCol(m_hstmtS, 11, SQL_C_SSHORT, &(pHostData[0].Nullable), 0, &(pHostData[0].NullableInd) );


    // Fetch and store...
    for (UINT Count = 0;; Count ++)
    {
        ReturnCode = SQLExtendedFetch(m_hstmtS,SQL_FETCH_NEXT,1,&RowsFetched,pRowStatus);

        if (ReturnCode == SQL_ERROR)
        {
            //$$$DisplayDBError( m_pDatabase , m_hstmtS) ;
            return 0;
        }
        else if ( ReturnCode == SQL_NO_DATA_FOUND )
            break;

        for (UINT Count = 0; Count < RowsFetched; Count++)
        {
            if  (pRowStatus[Count] != SQL_ROW_DELETED && pRowStatus[Count] != SQL_ROW_ERROR )
            {
                pDB_COLUMN_SET_FIELDS  pData = new DB_COLUMN_SET_FIELDS();
                ASSERT(pData);

                pData->strCatalog           = GetNullValue( pHostData[Count].CatalogInd             , pHostData[Count].szCatalog          );
                pData->strSchema            = GetNullValue( pHostData[Count].SchemaInd              , pHostData[Count].szSchema           );
                pData->strTableName         = GetNullValue( pHostData[Count].TableNameInd           , pHostData[Count].szTableName        );
                pData->strColumnName        = GetNullValue( pHostData[Count].ColumnNameInd          , pHostData[Count].szColumnName       );
                pData->strTypeName          = GetNullValue( pHostData[Count].TypeNameInd            , pHostData[Count].szTypeName         );

                m_aData.Add(pData);
            }

        }

        if  (RowsFetched < (UINT)m_nRowSetSize)
            break;
    }

    delete [] pRowStatus;
    pRowStatus = NULL;
    delete [] pHostData;
    pHostData = NULL;

    Free();
    DropHstmt(&m_hstmtS);

    return m_aData.GetSize();
}

