/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// HttpStatsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "HttpStatsOcx.h"
#include "HttpStatsDlg.h"
#include "FilterDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CHttpStatsDlg dialog
enum SortField { enTime,enUser,enURL,enRespTime,enRespCode,enReplySize} g_nSortField = enTime;
BOOL g_bReverseSort = false;

CHttpStatsDlg::CHttpStatsDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CHttpStatsDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CHttpStatsDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_bLoading = false;

    m_FilterData.dwReplySizeEnd = 0;
    m_FilterData.dwReplySizeStart =0;
    m_FilterData.dwRespTimeEnd = 0;
    m_FilterData.dwRespTimeStart = 0;
    m_FilterData.dwTimeEnd = 0;
    m_FilterData.dwTimeStart = 0;
    m_FilterData.wRespCodeEnd = 0;
    m_FilterData.wRespCodeStart =0;
}

CHttpStatsDlg::~CHttpStatsDlg()
{
    if(m_arHttpData.GetSize())
    {
        delete []m_arHttpData[0];
        m_arHttpData.RemoveAll();
    }
}

void CHttpStatsDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CHttpStatsDlg)
    DDX_Control(pDX, IDC_STATS_LIST, m_ctlList);
    //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CHttpStatsDlg, CDialog)
//{{AFX_MSG_MAP(CHttpStatsDlg)
ON_WM_SIZE()
ON_NOTIFY(LVN_GETDISPINFO, IDC_STATS_LIST, OnGetdispinfoList)
ON_NOTIFY(LVN_COLUMNCLICK, IDC_STATS_LIST, OnColumnclickStatsList)
ON_COMMAND(ID_FILTER, OnFilter)
ON_WM_CONTEXTMENU()
//}}AFX_MSG_MAP
ON_COMMAND(ID_EXPORT, OnExport)
ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHttpStatsDlg message handlers

BOOL CHttpStatsDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    //These column headers should match those in onexport
    m_ctlList.InsertColumn(0,"#",LVCFMT_LEFT,80,0);
    m_ctlList.InsertColumn(1,"Time Stamp",LVCFMT_LEFT,80,1);
    m_ctlList.InsertColumn(2,"User ID",LVCFMT_LEFT,80,2);
    m_ctlList.InsertColumn(3,"URL",LVCFMT_LEFT,80,3);
    m_ctlList.InsertColumn(4,"Response Time",LVCFMT_LEFT,80,4);
    m_ctlList.InsertColumn(5,"Response Code",LVCFMT_LEFT,80,5);
    m_ctlList.InsertColumn(6,"Reply Size",LVCFMT_LEFT,80,6);

    m_ctlList.SetExtendedStyle(LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES);

    return TRUE;  // return TRUE  unless you set the focus to a control
}


UINT CHttpStatsDlg::LoadHttpThread(LPVOID pParam)
{
    CHttpStatsDlg *pParent = (CHttpStatsDlg*)pParam;

    pParent->EnableWindow(FALSE);
    pParent->m_bLoading = true;

    BOOL bResult = pParent->LoadHttpData();

    pParent->EnableWindow();
    pParent->m_bLoading = false;
    return bResult;
}

BOOL CHttpStatsDlg::LoadHttpData()
{
    CWaitCursor wc;

    if(m_arHttpData.GetSize())
    {
        delete []m_arHttpData[0];
        m_arHttpData.RemoveAll();
    }

    m_ctlList.SetRedraw(FALSE);
    m_ctlList.DeleteAllItems();

    //Check the stat file exists
    CFileStatus fsStats;
    if(!CFile::GetStatus(m_strHttpPath,fsStats))
        return FALSE;

    //Get the URL file path and check it exists
    CString strURLPath(m_strHttpPath);
    strURLPath.Replace(".stat",".urls");

    CFileStatus fsURLs;
    if(!CFile::GetStatus(strURLPath,fsURLs))
        return FALSE;

    //Calculate the number of stats
    int nStatSize = sizeof(HttpFileStat);
    int nNumStats = fsStats.m_size/nStatSize;

	if(!nNumStats)
		return true;
    //////////////////////////
    //Memory map the two files

    //Open Stat file
    HANDLE hStatFile = CreateFile(m_strHttpPath,GENERIC_READ,0,NULL,
                                  OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);

    if(hStatFile == INVALID_HANDLE_VALUE)
        return FALSE;

    //Open URL file
    HANDLE hURLFile = CreateFile(strURLPath,GENERIC_READ,0,NULL,
                                 OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);

    if(hURLFile == INVALID_HANDLE_VALUE)
        return FALSE;

    //Create a mapping for the stat file
    HANDLE hStatMap = CreateFileMapping(hStatFile,NULL,PAGE_READONLY,0,0,"StatDat");

    if(hStatMap == NULL)
        return FALSE;

    //Create a mapping for the url file
    HANDLE hURLMap = CreateFileMapping(hURLFile,NULL,PAGE_READONLY,0,0,"URLDat");

    if(hURLMap == NULL)
        return FALSE;

    //Map the stat file to memory
    LPBYTE pStatBuffer = (LPBYTE)MapViewOfFile(hStatMap,FILE_MAP_READ,0,0,0);

    //Map the URL file to memory
    LPBYTE pURLBuffer = (LPBYTE)MapViewOfFile(hURLMap,FILE_MAP_READ,0,0,0);

    //Show the user that something is happening
    ShowStatus(TRUE);

    CWnd *pCaption = GetDlgItem(IDC_CAPTION);
    pCaption->SetWindowText("Loading Http Data...");

    CProgressCtrl *pProgress = (CProgressCtrl*)GetDlgItem(IDC_PROGRESS);
    pProgress->SetRange32(0,nNumStats);
    pProgress->SetPos(0);

    ///////////////////////////
    //Perform the load...
    LPBYTE pStatBufPos = pStatBuffer;
    LPBYTE pUrlBufPos = pURLBuffer;

    CMap <CString,LPCTSTR,int,int> mapUrlToNdx;
    mapUrlToNdx.InitHashTable(nNumStats);
    m_arHttpData.SetSize(nNumStats);

    HttpData *pData = new HttpData[nNumStats];

    for(int nStat=0;nStat<nNumStats;nStat++)
    {
        HttpData *pStat = &pData[nStat];

        if(!mapUrlToNdx.Lookup((LPCTSTR)pUrlBufPos,pStat->nURLNdx))
        {
            pStat->nURLNdx = mapUrlToNdx.GetCount();
            mapUrlToNdx.SetAt((LPCTSTR)pUrlBufPos,pStat->nURLNdx);
        }

        m_arHttpData[nStat] = pStat;

        memcpy(&pStat->HttpStat,pStatBufPos,nStatSize);
        pStatBufPos += nStatSize;

        pUrlBufPos += strlen((LPCTSTR)pUrlBufPos) + 1;

        int nPos = pProgress->GetPos();
        if(nStat - nPos > 2000)
            pProgress->SetPos(nStat);
    }

    pCaption->SetWindowText("Processing URLs...");
    pProgress->SetRange32(0,mapUrlToNdx.GetCount());

    m_arstrURLs.SetSize(mapUrlToNdx.GetCount());

    int nCurrentIndex=0;
    int nIndex=1;
    CString szURL;
    POSITION pos = mapUrlToNdx.GetStartPosition();
    while(pos)
    {
        pProgress->SetPos(nIndex);

        mapUrlToNdx.GetNextAssoc(pos,szURL,nCurrentIndex);
        m_arstrURLs[nCurrentIndex] = szURL;
    }

    for(nStat=0;nStat<nNumStats;nStat++)
    {
        HttpFileStat *pData = &m_arHttpData[nStat]->HttpStat;

		COleDateTime t((long)(double)pData->dwTimeStamp);
        m_arHttpData[nStat]->strTimeStamp = t.Format();
        m_arHttpData[nStat]->strUserId.Format("%d_%d_%d_%d-%d-%d",pData->UserId.IP1,pData->UserId.IP2,
                                              pData->UserId.IP3,pData->UserId.IP4,pData->UserId.dwPID,pData->UserId.dwUser);
        m_arHttpData[nStat]->strResponseTime.Format("%d",pData->dwResponseTime);
        m_arHttpData[nStat]->strResponseCode.Format("%d",pData->wResponseCode);
        m_arHttpData[nStat]->strReplySize.Format("%d",pData->dwReplySize);
        m_arHttpData[nStat]->pstrURL = &(m_arstrURLs[m_arHttpData[nStat]->nURLNdx]);
    }

    SetupDefaultFilters();

    pProgress->SetPos(nStat);

    ShowStatus(FALSE);

    ApplyFilters();

    //Clean up
    UnmapViewOfFile(pStatBuffer);
    UnmapViewOfFile(pURLBuffer);
    CloseHandle(hStatMap);
    CloseHandle(hURLMap);
    CloseHandle(hStatFile);
    CloseHandle(hURLFile);

    return true;
}


int CHttpStatsDlg::SortHttpData( const void *arg1, const void *arg2 )
{
    HttpData *p1 = *(HttpData**)arg1;
    HttpData *p2 = *(HttpData**)arg2;

    switch(g_nSortField)
    {
    case enTime:
        if(p1->HttpStat.dwTimeStamp > p2->HttpStat.dwTimeStamp)
            return 1;
        return (p1->HttpStat.dwTimeStamp < p2->HttpStat.dwTimeStamp ? -1 : 0);
        break;
    case enUser:
        return (p1->strUserId.Compare(p2->strUserId));
        break;
    case enURL:
        return (p1->pstrURL->Compare(*(p2->pstrURL)));
        break;
    case enRespTime:
        if(p1->HttpStat.dwResponseTime > p2->HttpStat.dwResponseTime)
            return 1;
        return (p1->HttpStat.dwResponseTime < p2->HttpStat.dwResponseTime ? -1 : 0);
        break;
    case enRespCode:
        if(p1->HttpStat.wResponseCode > p2->HttpStat.wResponseCode)
            return 1;
        return (p1->HttpStat.wResponseCode < p2->HttpStat.wResponseCode ? -1 : 0);
        break;
    case enReplySize:
        if(p1->HttpStat.dwReplySize > p2->HttpStat.dwReplySize)
            return 1;
        return (p1->HttpStat.dwReplySize < p2->HttpStat.dwReplySize ? -1 : 0);
        break;
    default:
        ASSERT(FALSE);
        return 0;
        break;
    }
}

void CHttpStatsDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    if(!m_bLoading && m_ctlList.GetSafeHwnd())
    {
        CRect rc;
        m_ctlList.GetWindowRect(&rc);
        ScreenToClient(&rc);

        m_ctlList.SetWindowPos(NULL,0,0,cx-rc.left*2 ,cy -rc.top*2 ,SWP_NOZORDER|SWP_NOMOVE);
    }
}

BOOL CHttpStatsDlg::LoadFile(const CString& strPath)
{
    CWaitCursor wc;
    m_strHttpPath = strPath;
    m_FilterData.mapUrls.RemoveAll();
    m_FilterData.mapUserIds.RemoveAll();

    m_bLoading = true;
    CWinThread *pThread = AfxBeginThread(&LoadHttpThread,this);

    MSG msg;
    while (m_bLoading)
    {
        while (::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            if (!AfxGetApp()->PreTranslateMessage(&msg))
            {
                ::TranslateMessage(&msg);
                ::DispatchMessage(&msg);
            }
        }
        AfxGetApp()->OnIdle(0);   // updates user interface
        AfxGetApp()->OnIdle(1);   // frees temporary objects
    }


    DWORD dwCode=0;
    GetExitCodeThread(pThread->m_hThread,&dwCode);
    m_ctlList.RedrawWindow(NULL,NULL,RDW_INVALIDATE|RDW_FRAME);
    return (dwCode == 1 ? TRUE : FALSE);
}

void CHttpStatsDlg::OnGetdispinfoList(NMHDR* pNMHDR, LRESULT* pResult)
{
    LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;

    LV_ITEM &lv = pDispInfo->item;

    if(lv.mask | LVIF_TEXT )
    {
        int nNdx = (g_bReverseSort ? (m_arVisibleStats.GetSize()-lv.iItem-1): lv.iItem);
        HttpData *pHttpData = m_arVisibleStats[nNdx];

        static CString strNum;
        switch(lv.iSubItem)
        {
        case 0:
            strNum.Format("%d",lv.iItem+1);
            lv.pszText = strNum.GetBuffer(0);
            break;
        case 1:
            lv.pszText = pHttpData->strTimeStamp.GetBuffer(0);
            break;
        case 2:
            lv.pszText = pHttpData->strUserId.GetBuffer(0);
            break;
        case 3:
            lv.pszText = pHttpData->pstrURL->GetBuffer(0);
            break;
        case 4:
            lv.pszText = pHttpData->strResponseTime.GetBuffer(0);
            break;
        case 5:
            lv.pszText = pHttpData->strResponseCode.GetBuffer(0);
            break;
        case 6:
            lv.pszText = pHttpData->strReplySize.GetBuffer(0);
            break;
        }
    }

    *pResult = 0;
}

void CHttpStatsDlg::OnColumnclickStatsList(NMHDR* pNMHDR, LRESULT* pResult)
{
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

    *pResult = 0;

    if(!pNMListView->iSubItem)
        return;

    if(pNMListView->iSubItem-1 != g_nSortField)
    {
        g_bReverseSort = false;

        CWaitCursor wc;

        g_nSortField = (SortField)(pNMListView->iSubItem-1);
        qsort(m_arVisibleStats.GetData(),(size_t)m_arVisibleStats.GetSize(),sizeof(HttpData**),SortHttpData);
    }
    else
        g_bReverseSort = !g_bReverseSort;

    m_ctlList.Invalidate();
}

void CHttpStatsDlg::ShowStatus(BOOL bShow)
{
    CWnd *pCaption = GetDlgItem(IDC_CAPTION);
    CWnd *pProgress = GetDlgItem(IDC_PROGRESS);

    CRect rcClient;
    GetClientRect(&rcClient);

    CRect rcList;
    m_ctlList.GetWindowRect(&rcList);
    ScreenToClient(&rcList);

    if(bShow)
    {
        if(pCaption->IsWindowVisible())
            return;

        CRect rcCaption;
        pCaption->GetWindowRect(&rcCaption);
        ScreenToClient(&rcCaption);

        CRect rcProgress;
        pProgress->GetWindowRect(&rcProgress);
        ScreenToClient(&rcProgress);

        //reposition windows
        pCaption->SetWindowPos(NULL,rcCaption.left,rcClient.bottom-rcCaption.Height()-10,0,0,SWP_NOZORDER|SWP_NOSIZE);
        m_ctlList.SetWindowPos(NULL,0,0,rcClient.Width(),rcClient.bottom-rcCaption.Height()-20,SWP_NOZORDER|SWP_NOMOVE);
        pProgress->SetWindowPos(NULL,rcCaption.right+10,rcClient.bottom-rcCaption.Height()-10,rcList.right-10-rcCaption.right,rcProgress.Height(),SWP_NOZORDER);

        pCaption->ShowWindow(SW_SHOW);
        pProgress->ShowWindow(SW_SHOW);
    }
    else
    {
        pCaption->ShowWindow(SW_HIDE);
        pProgress->ShowWindow(SW_HIDE);
        m_ctlList.SetWindowPos(NULL,0,0,rcClient.Width(),rcClient.Height()-rcList.top*2,SWP_NOZORDER|SWP_NOMOVE);
    }
}

void CHttpStatsDlg::ApplyFilters()
{
    ShowStatus(TRUE);

    CWnd *pCaption = GetDlgItem(IDC_CAPTION);
    CProgressCtrl *pProgress = (CProgressCtrl*)GetDlgItem(IDC_PROGRESS);
    pCaption->SetWindowText("Applying Filters...");

    int nNumStats = m_arHttpData.GetSize();
    pProgress->SetRange32(0,nNumStats);
    pProgress->SetPos(0);

    //Copy the array
    int nCount=0;
    m_arVisibleStats.SetSize(nNumStats);

    for(int nNdx=0;nNdx<nNumStats;nNdx++)
    {
        if(IsStatInFilter(m_arHttpData[nNdx]))
            m_arVisibleStats[nCount++] = m_arHttpData[nNdx];

        if(!(nNdx % 200))
            pProgress->SetPos(nNdx);
    }

    m_arVisibleStats.SetSize(nCount);

    pProgress->SetRange32(0,nCount);
    pCaption->SetWindowText("Sorting...");
    pProgress->SetPos(0);

    //Sort data
    qsort(m_arVisibleStats.GetData(),(size_t)nCount,sizeof(HttpData**),SortHttpData);

    m_ctlList.SetRedraw(FALSE);

    pCaption->SetWindowText("Clearing List...");

    //remove unneeded items
    m_ctlList.DeleteAllItems();

    m_ctlList.SetItemCount(nCount);
    pCaption->SetWindowText("Filling List...");

    //Add extra items to  list
    for(nNdx=0;nNdx<m_arVisibleStats.GetSize();nNdx++)
    {
        m_ctlList.InsertItem(nNdx,LPSTR_TEXTCALLBACK);

        if(!(nNdx % 200))
            pProgress->SetPos(nNdx);
    }
    pProgress->SetPos(nNdx);

    m_ctlList.SetRedraw();

    ShowStatus(FALSE);
}

void CHttpStatsDlg::SetupDefaultFilters()
{
    int nNumStats = m_arHttpData.GetSize();
    if(!nNumStats)
        return;

    m_FilterData.dwReplySizeStart = m_arHttpData[0]->HttpStat.dwReplySize;
    m_FilterData.dwRespTimeStart = m_arHttpData[0]->HttpStat.dwResponseTime;
    m_FilterData.dwTimeStart = m_arHttpData[0]->HttpStat.dwTimeStamp;
    m_FilterData.wRespCodeStart = m_arHttpData[0]->HttpStat.wResponseCode;

    m_FilterData.dwReplySizeEnd = 0;
    m_FilterData.dwRespTimeEnd = 0;
    m_FilterData.dwTimeEnd = 0;
    m_FilterData.wRespCodeEnd = 0;

    bool bSelected=true;
    m_FilterData.mapUserIds.InitHashTable(nNumStats);
    m_FilterData.mapUrls.InitHashTable(nNumStats);
    for(int nStat=0;nStat<nNumStats;nStat++)
    {
        if(m_arHttpData[nStat]->HttpStat.dwReplySize > m_FilterData.dwReplySizeEnd)
            m_FilterData.dwReplySizeEnd = m_arHttpData[nStat]->HttpStat.dwReplySize;

        if(m_arHttpData[nStat]->HttpStat.dwReplySize < m_FilterData.dwReplySizeStart)
            m_FilterData.dwReplySizeStart = m_arHttpData[nStat]->HttpStat.dwReplySize;

        if(m_arHttpData[nStat]->HttpStat.dwResponseTime < m_FilterData.dwRespTimeStart)
            m_FilterData.dwRespTimeStart = m_arHttpData[nStat]->HttpStat.dwResponseTime;

        if(m_arHttpData[nStat]->HttpStat.dwResponseTime > m_FilterData.dwRespTimeEnd)
            m_FilterData.dwRespTimeEnd = m_arHttpData[nStat]->HttpStat.dwResponseTime;

        if(m_arHttpData[nStat]->HttpStat.dwTimeStamp < m_FilterData.dwTimeStart)
            m_FilterData.dwTimeStart = m_arHttpData[nStat]->HttpStat.dwTimeStamp;

        if(m_arHttpData[nStat]->HttpStat.dwTimeStamp > m_FilterData.dwTimeEnd)
            m_FilterData.dwTimeEnd  = m_arHttpData[nStat]->HttpStat.dwTimeStamp;

        if(m_arHttpData[nStat]->HttpStat.wResponseCode < m_FilterData.wRespCodeStart)
            m_FilterData.wRespCodeStart = m_arHttpData[nStat]->HttpStat.wResponseCode;

        if(m_arHttpData[nStat]->HttpStat.wResponseCode > m_FilterData.wRespCodeEnd)
            m_FilterData.wRespCodeEnd = m_arHttpData[nStat]->HttpStat.wResponseCode;

        m_FilterData.mapUserIds.SetAt(m_arHttpData[nStat]->strUserId,bSelected);
        m_FilterData.mapUrls.SetAt(*(m_arHttpData[nStat]->pstrURL),bSelected);
    }

    //setup default fields
    m_FilterData.dwDefReplySizeStart = m_FilterData.dwReplySizeStart;
    m_FilterData.dwDefReplySizeEnd = m_FilterData.dwReplySizeEnd;

    m_FilterData.dwDefRespTimeStart = m_FilterData.dwRespTimeStart;
    m_FilterData.dwDefRespTimeEnd = m_FilterData.dwRespTimeEnd;

    m_FilterData.dwDefTimeStart = m_FilterData.dwTimeStart;
    m_FilterData.dwDefTimeEnd = m_FilterData.dwTimeEnd;

    m_FilterData.wDefRespCodeStart = m_FilterData.wRespCodeStart;
    m_FilterData.wDefRespCodeEnd = m_FilterData.wRespCodeEnd;
}

BOOL CHttpStatsDlg::IsStatInFilter(HttpData *pHttpData)
{
    bool bSelected;

    if((pHttpData->HttpStat.dwTimeStamp >= m_FilterData.dwTimeStart) &&
            (pHttpData->HttpStat.dwTimeStamp <= m_FilterData.dwTimeEnd) &&

            (pHttpData->HttpStat.dwResponseTime >= m_FilterData.dwRespTimeStart) &&
            (pHttpData->HttpStat.dwResponseTime <= m_FilterData.dwRespTimeEnd) &&

            (pHttpData->HttpStat.dwReplySize >= m_FilterData.dwReplySizeStart) &&
            (pHttpData->HttpStat.dwReplySize <= m_FilterData.dwReplySizeEnd) &&

            (pHttpData->HttpStat.wResponseCode >= m_FilterData.wRespCodeStart) &&
            (pHttpData->HttpStat.wResponseCode <= m_FilterData.wRespCodeEnd))
    {
        if(!m_FilterData.mapUserIds.Lookup(pHttpData->strUserId,bSelected) || !bSelected)
		   return false;

        if(!m_FilterData.mapUrls.Lookup(*(pHttpData->pstrURL),bSelected) || !bSelected)
            return false;

        return true;
    }
    return false;
}


void CHttpStatsDlg::OnFilter()
{
    ShowFilter();
}

void CHttpStatsDlg::OnContextMenu(CWnd* pWnd, CPoint point)
{
    if(!m_arHttpData.GetSize())
        return ;

    CMenu menu;
    if (menu.LoadMenu(IDR_POPUP_MENU))
    {
        CMenu* pPopup = menu.GetSubMenu(0);
        ASSERT(pPopup != NULL);

        pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON,
                               point.x, point.y,
                               this);
    }
}

void CHttpStatsDlg::ShowFilter()
{
    CFilterDlg dlg(&m_FilterData);
    if(dlg.DoModal() == IDOK)
    {
        ApplyFilters();
        m_ctlList.Invalidate();
    }
}

BOOL CHttpStatsDlg::OnEraseBkgnd(CDC *pDC)
{
    if(m_bLoading)
    {
        CRect rc;
        GetClientRect(&rc);

        CRect rcList;
        m_ctlList.GetWindowRect(&rcList);
        ScreenToClient(&rcList);

        rc.top = rcList.bottom;

        pDC->FillSolidRect(rc,GetSysColor(COLOR_BTNFACE));

        rc.left = rcList.left;
        rc.top = 0;
        rc.bottom = rcList.bottom;

        pDC->FillSolidRect(rc,GetSysColor(COLOR_BTNFACE));
    }
    return true;
}

void CHttpStatsDlg::OnCancel()
{}


void CHttpStatsDlg::OnOK()
{}


void CHttpStatsDlg::OnExport()
{
    CFileDialog dlg(FALSE,"csv","HttpStat.csv",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"Comma Separated Values (*.csv)|*.csv||",this);
    if(dlg.DoModal() != IDOK)
        return;

    CString strPath(dlg.GetPathName());

    CStdioFile f;
    if(!f.Open(strPath,CFile::modeWrite|CFile::modeCreate))
    {
        CString strMsg;
        strMsg.Format("Failed to open the export file \"%s\"",strPath);
        MessageBox(strMsg,"Http Stats",MB_ICONEXCLAMATION|MB_OK);
        return;
    }

    //Create a header line in the csv file
    CHeaderCtrl* pHeader = m_ctlList.GetHeaderCtrl();
    int nNumColumns = pHeader->GetItemCount();

    char szText[256];
    HDITEM hdItem;
    hdItem.mask = HDI_TEXT;
    hdItem.pszText = szText;
    hdItem.cchTextMax = 256;

    CString strLine;
    for(int nCol=1;nCol<nNumColumns;nCol++)
    {
        pHeader->GetItem(nCol,&hdItem);
        strLine += hdItem.pszText;
        strLine += ",";
    }
    strLine += "\n";
	f.WriteString(strLine);
	strLine.Empty();

    int nNumItems = m_ctlList.GetItemCount();
    for(int nItem=0;nItem<nNumItems;nItem++)
    {
        for(int nColumn=1;nColumn<nNumColumns;nColumn++)
        {
            strLine += m_ctlList.GetItemText(nItem,nColumn);
            strLine += ",";
        }
        strLine += "\n";
	    f.WriteString(strLine);
		strLine.Empty();
    }
    f.Close();
    MessageBox("Export complete","Http Stats",MB_ICONINFORMATION|MB_OK);
}
