/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     SNMP Test Executer/Test Monitor
///////////////////////////////////////////////////////////////////////////////

#ifndef _OtaExecuter_I_H_
#define _OtaExecuter_I_H_

#include <string>
#include <list>

#ifndef VMS
using     std::string;
using     std::list;
#endif

#include "TestExecuter.h"
#include <OSTAORB/OSTAORB.h>
#include "mdbmtx.h"
#include <TScript.h>
#include <OtaTarget.h>
#include <OtaQuery.h>

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

///////////////////////////////////////////////////////////////////////////////
// @type EnvironmentArray | This list contains a collection of SMP environment objects.
typedef   vector< string > EnvironmentArray;

///////////////////////////////////////////////////////////////////////////////
// @type TargetArray | This list contains a collection of SMP target objects.
typedef   vector< OtaTarget * >TargetArray;

///////////////////////////////////////////////////////////////////////////////
// @type QueryArray | This list contains a collection of SMP query objects.
typedef   vector< OtaQuery * >QueryArray;

// @class SmpExecuterImpl
// @base public | <c _sk_TestExecuter>
class     SmpExecuterImpl : public POA_TestExecuter, public PortableServer::RefCountServantBase  
{
public:
    SmpExecuterImpl(CORBAServer * pserver, string & regname, string node,
                    int pid, string exename);
    virtual ~ SmpExecuterImpl();
    virtual   CORBA::Long Initialise(const char *node, const char *user,
                                     const char *password,
                                     const char *TestRunString);
    virtual   CORBA::Long ExecuteThread(const char *ProjectName,
                                        const char *ScriptName,
                                        TestLevel_ptr & parent,
                                        const ThreadDef & thrdef);
    virtual void Stop();
    virtual void Kill();
    //Dummy bodies to functions that are required for other test executers with thread information,
    //Maybe should split the TestExecuter class hierarchy?
    virtual char *GetNode()
    {
        return CORBA::string_dup(m_nodename.c_str());
    }
    virtual char *GetNodeName();
    virtual char *Name()
    {
        return CORBA::string_dup(m_RegisterName.c_str());
    }
    virtual char *GetThreadIdents()
    {
        return CORBA::string_dup((const char *) "");
    }
    virtual   CORBA::ULong GetNoThreads(char *&ScriptName)
    {
        return 0;
    }
    virtual   CORBA::ULong GetNoActiveThreads()
    {
        return 0;
    }
    virtual ThreadStatus GetStatus(char *&ident)
    {
        return (ThreadStatus) 0;
    }
    virtual char *GetTarget()
    {
        return CORBA::string_dup((const char *) "SMP");
    }
    virtual char *TestName()
    {
        return CORBA::string_dup((const char *) "");
    }
    virtual   CORBA::Boolean Suspend(char *&ident)
    {
        return false;
    }
    virtual   CORBA::Boolean Activate(char *&ident)
    {
        return false;
    }
    virtual   CORBA::Boolean Detatch(char *&ident)
    {
        return false;
    }
    virtual char *ScriptName(char *&ident)
    {
        return CORBA::string_dup((const char *) "");
    }
    virtual   CORBA::ULong StartTime(char *&ident)
    {
        return 0;
    }
    virtual char *GetNoteText(char *&ident)
    {
        return CORBA::string_dup((const char *) "");
    }
    virtual   CORBA::Long GetTestCaseResults(TestCaseStatus Status)
    {
        return 0;
    }
    virtual   CORBA::ULong GetThreadInformation(ThreadInfoList & TList)
    {
        return 0;
    }

    void      ExecutionThread();    // @cmember Main execution thread method
    void      ExecuteQuery(int index);

    // @access Protected Members
protected:


private:
    void      ParseTof(const char *TofFile);
    void      ReadOptions(TRecord * record);
    void      ReadEnvironment(TRecord * record);
    void      ReadQuery(TRecord * record, bool detail);
    void      SetEnvironment();

    void      LogConfig();

    string    m_TestRunString;
    CECOrbTestRun *m_testRun;
    string    m_nodename;
    int       m_pid;
    string    m_exename;
    CORBAServer *m_pServer;
    string    m_RegisterName;
    Cmdbmutex m_OtaExecuterMTX;
    QueryArray m_query;
    int       m_tracing;
    bool      m_details;
    bool      m_threadstarted;
    TargetArray m_targets;
    EnvironmentArray m_environments;
    CTThread* m_executor;   // @cmember Thread controller
    long      m_threads;    // @cmember Active thread count
    enum TestStatus
    { UNKNOWN, NOT_STARTED, ACTIVE, FAILED, SUCCESS };
    TestStatus m_status;
    CTMutex m_mutex;    // @cmember Thread count guard
    TestLevel_var m_ParentTestLevel;    // @cmember TestLevel that kicked off this SMP

}
;

#endif
