/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// SNMPPluginView.cpp : implementation of the CSNMPPluginView class
//

#include "stdafx.h"
#include "SNMPPlugin.h"

#include "SNMPPluginDoc.h"
#include "SNMPPluginView.h"
#include "SNMPScanDlg.h"
#include "QueryEditDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView

IMPLEMENT_DYNCREATE(CSNMPPluginView, CFormView)

BEGIN_MESSAGE_MAP(CSNMPPluginView, CFormView)
	//{{AFX_MSG_MAP(CSNMPPluginView)
	ON_COMMAND(ID_CANCEL_EDIT_SRVR, OnCancelEditSrvr)
	ON_BN_CLICKED(IDC_ADD_ROW, OnAddRow)
	ON_BN_CLICKED(IDC_REMOVE_ROW, OnRemoveRow)
	ON_BN_CLICKED(IDC_SCAN, OnScan)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WMU_CELL_BUTTON_PRESSED, OnButtonPressed)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView construction/destruction

CSNMPPluginView::CSNMPPluginView()
	: CFormView(CSNMPPluginView::IDD)
{
	//{{AFX_DATA_INIT(CSNMPPluginView)
	//}}AFX_DATA_INIT
	m_bLoaded = false;
}

CSNMPPluginView::~CSNMPPluginView()
{
}

void CSNMPPluginView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSNMPPluginView)
	DDX_Control(pDX, IDC_GROUP, m_ctlGroup);
	//}}AFX_DATA_MAP
    DDX_Control(pDX, IDC_GRID, m_wndGrid);
}

BOOL CSNMPPluginView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

void CSNMPPluginView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();

	GetDocument()->SetModifiedFlag();	

	CString strTofPath = GetDocument()->GetProfilePath();

	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();
	
    const int nNumColumns = 5;
    CString headers = "Description\tSNMP Server\tQuery\tInterval\tDelta Value";
	CRect r;
	m_wndGrid.GetWindowRect(r);

    m_wndGrid.SetColumnCount(nNumColumns);
    m_wndGrid.SetRowCount(1);
    m_wndGrid.SetFixedRowCount();
	
	m_wndGrid.SetColumnWidth(0,r.Width()/5);
	m_wndGrid.SetColumnWidth(1,r.Width()/5);
	m_wndGrid.SetColumnWidth(2,r.Width()*2/5);
	m_wndGrid.SetColumnWidth(3,r.Width()/10);
	m_wndGrid.SetColumnWidth(4,r.Width()/10);

    m_wndGrid.PasteTextFromBuffer(0,0, headers);

	if (!ReadProfile(strTofPath))
	{
		OnAddRow();
		m_bLoaded = true;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView printing

BOOL CSNMPPluginView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CSNMPPluginView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CSNMPPluginView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CSNMPPluginView::OnPrint(CDC* pDC, CPrintInfo* /*pInfo*/)
{
	// TODO: add customized printing code here
}

/////////////////////////////////////////////////////////////////////////////
// OLE Server support

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the server (not the container) causes the deactivation.
void CSNMPPluginView::OnCancelEditSrvr()
{
	GetDocument()->OnDeactivateUI(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView diagnostics

#ifdef _DEBUG
void CSNMPPluginView::AssertValid() const
{
	CFormView::AssertValid();
}

void CSNMPPluginView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CSNMPPluginDoc* CSNMPPluginView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSNMPPluginDoc)));
	return (CSNMPPluginDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView message handlers
void CSNMPPluginView::OnAddRow() 
{
	int nRow = m_wndGrid.InsertRow("");

	// Setup Row Ctrls
	m_wndGrid.CreateListCell(nRow, enServer, m_arrServers);
	m_wndGrid.CreateButtonCell(nRow, enQuery, "...", CECGridCtrl::ALIGN_RIGHT);
	m_wndGrid.CreateSpinCell(nRow, enInterval, TRUE);	
	m_wndGrid.CreateCheckCell(nRow, enDelta);

	// Setup Default values
	m_wndGrid.SetItemText(nRow, enDesc, "Description");
	m_wndGrid.SetItemText(nRow, enServer, "Address");
	m_wndGrid.SetItemText(nRow, enQuery, "");
	m_wndGrid.SetItemText(nRow, enInterval, "5");

	m_wndGrid.Invalidate();		
}

void CSNMPPluginView::OnRemoveRow() 
{
	CCellRange range = m_wndGrid.GetSelectedCellRange();
	if (range.GetRowSpan() != 1)
	{
		MessageBox("Please select a single row to remove.", "OpenSTA", MB_OK);
		return;
	}
	m_wndGrid.DeleteRow(range.GetMinRow());
	m_wndGrid.Invalidate();	
}

void CSNMPPluginView::EditQuery(int nRow)
{
	CString strServer = m_wndGrid.GetItemText(nRow, enServer);
	if (strServer.IsEmpty())
	{
		MessageBox("Please select a server.", "OpenSTA", MB_ICONEXCLAMATION);
		return;
	}

	CString strQuery = m_wndGrid.GetItemText(nRow, enQuery);

	CQueryEditDlg dlg(strServer, strQuery);
	if(dlg.DoModal())
	{
		m_wndGrid.SetItemText(nRow, enQuery, dlg.GetQuery());
	}

}

LRESULT CSNMPPluginView::OnButtonPressed(WPARAM nRow,LPARAM nColumn)
{
	if (nColumn == enQuery)
	{
		EditQuery(nRow);
	}

	return 0;
}

void CSNMPPluginView::OnScan() 
{
	CSNMPScanDlg dlg;
	if(dlg.DoModal())
	{
		dlg.GetServers(m_arrServers);
	}

	for (int i = 1; i<m_wndGrid.GetRowCount(); i++)
	{
		m_wndGrid.CreateListCell(i, enServer, m_arrServers);
	}
}

BOOL CSNMPPluginView::ReadProfile(CString strFilePath) 
{
	// Check if file exists
	CFileStatus fs;
	if (!CFile::GetStatus(strFilePath, fs))
		return FALSE;

	int nAdded = 0;
	try
	{
		CStdioFile f(strFilePath, CFile::modeRead | CFile::typeText);

		CString strLine ="";
		f.ReadString(strLine);

		for (; f.ReadString(strLine); )
		{
			if (strLine == "SimpleQuery")
			{
				// ignore enabled line
				f.ReadString(strLine);
				// ignore type line
				f.ReadString(strLine);

				CString strDelta = "";
				f.ReadString(strDelta);
				BOOL bDelta = atoi(strDelta.Right(1));

				CString strInterval = "";
				f.ReadString(strInterval);
				int nPos = strInterval.Find(" ", 0);
				int nLen = strInterval.GetLength();
				strInterval = strInterval.Right(nLen-nPos-1);

				CString strHeading = "";
				f.ReadString(strHeading);
				nPos = strHeading.Find(" ", 0);
				nLen = strHeading.GetLength();
				strHeading = strHeading.Right(nLen-nPos-1);

				// ignore text line
				f.ReadString(strLine);

				CString strQuery = "";
				f.ReadString(strQuery);
				strQuery.Remove('\"');
				nPos = strQuery.Find(" ", 0);
				nLen = strQuery.GetLength();

				CString strServer = strQuery.Left(nPos);
				strQuery = strQuery.Right(nLen-nPos-1);

				OnAddRow();
				nAdded++;
				int nRow = m_wndGrid.GetRowCount()-1;
				m_wndGrid.SetItemText(nRow, enDesc, strHeading);
				m_wndGrid.SetItemText(nRow, enServer, strServer);
				m_wndGrid.SetItemText(nRow, enQuery, strQuery);
				m_wndGrid.SetItemText(nRow, enInterval, strInterval);
				m_wndGrid.SetCheckBoxCellState(nRow, enDelta, bDelta);
			}
		}

		f.Close();
		m_bLoaded = true;
	}
	catch(...)
	{
		return FALSE;
	}

	if (nAdded == 0)
		return FALSE;

	return TRUE;
}

BOOL CSNMPPluginView::WriteProfile(CString strFilePath) 
{
	if (!m_bLoaded)
		return true;

	try
	{
		CStdioFile f(strFilePath, CFile::modeCreate | CFile::modeWrite | CFile::typeText);
		f.WriteString("tof.6\n");
		f.WriteString("date\n");
		f.WriteString("lastedit\n");
		f.WriteString("name\n");
		f.WriteString("description\n");
		f.WriteString("\n");

		for (int i =1; i<m_wndGrid.GetRowCount(); i++)
		{
			CString szDesc = m_wndGrid.GetItemText(i,enDesc);
			CString szServer = m_wndGrid.GetItemText(i,enServer);
			CString szQuery = m_wndGrid.GetItemText(i,enQuery);
			CString szInterval = m_wndGrid.GetItemText(i,enInterval);
			BOOL bDelta = m_wndGrid.GetCheckBoxCellState(i, enDelta);
			CString szDelta = "";

			if (bDelta)
				szDelta="1";
			else
				szDelta="0";

			if (!szServer.IsEmpty() && !szQuery.IsEmpty())
			{
				f.WriteString("SimpleQuery\n");
				f.WriteString("enabled 1\n");
				f.WriteString("type 3\n");
				f.WriteString("delta "+szDelta+"\n");
				f.WriteString("frequency "+szInterval+"\n");
				f.WriteString("heading "+szDesc+"\n");
				f.WriteString("text \"\n");
				f.WriteString(szServer+" "+szQuery+"\"\n");
				f.WriteString("\n");
			}
		}

		f.Close();
		return true;
	}
	catch(...)
	{
		MessageBox("Unable to write profile to disk!", "OpenSTA", MB_ICONEXCLAMATION);
	}

	return false;
}

void CSNMPPluginView::OnSize(UINT nType, int cx, int cy) 
{
	CFormView::OnSize(nType, cx, cy);
	
	if (m_ctlGroup.m_hWnd != NULL)
	{
		m_ctlGroup.MoveWindow(5,5,cx-10,cy-10);
		m_wndGrid.MoveWindow(20,25,cx-40,cy-40);
	    m_wndGrid.ExpandColumnsToFit(); 
	}	
}
