// GraphWnd.cpp : implementation file
//

#include "stdafx.h"
#include "GraphWnd.h"
#include "XChartCtrl.h"
#include "GraphTypes.h"
#include "resource.h"
#include "ExcelExport.h"
#include "GraphCustomiseDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern HINSTANCE m_hInstance;
/////////////////////////////////////////////////////////////////////////////
// CGraphWnd
IMPLEMENT_DYNAMIC(CGraphWnd,CWnd)

CGraphWnd::CGraphWnd()
{
	m_pGraph = NULL;
	m_pChart = NULL;
	
	m_bShowTitle = FALSE;
	m_strTitleFont = "Times New Roman";
	m_colTitleText = RGB(210,40,50);
	m_colTitleBk = RGB(0,128,192);
	m_colPlotBk = RGB(0xff,0xff,0xff);
    m_strLabelFont = "Verdana";
    m_colGrid = RGB(0xf0,0xca,0xa6);
    m_colLabel = RGB(0,0,0);
    m_colAxisText = RGB(0,0,0);

	m_arDefLineColours.Add(RGB(255,0,0));
	m_arDefLineColours.Add(RGB(0,0,255));
	m_arDefLineColours.Add(RGB(0,255,0));
	m_arDefLineColours.Add(RGB(255,255,0));
	m_arDefLineColours.Add(RGB(255,0,255));
	m_arDefLineColours.Add(RGB(0,255,255));
	m_arDefLineColours.Add(RGB(0,0,0));
	m_arDefLineColours.Add(RGB(255,255,255));
	m_arDefLineColours.Add(RGB(190,190,190));
	m_arDefLineColours.Add(RGB(70,70,70));

	LoadGraphStyles();
}

CGraphWnd::~CGraphWnd()
{
	SaveGraphStyles();
	delete m_pChart;
	delete m_pGraph;
}


BEGIN_MESSAGE_MAP(CGraphWnd, CWnd)
	//{{AFX_MSG_MAP(CGraphWnd)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_EXPORT, OnExport)
	ON_COMMAND(ID_GRAPH_CUSTOMIZE, OnGraphCustomize)
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(GRAPH_MENU_ITEM_ID_START,GRAPH_MENU_ITEM_ID_END,OnGraphMenuCommand)

END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CGraphWnd message handlers

int CGraphWnd::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_pChart = new CXChartCtrl;
	m_pChart->Create("",WS_CHILD|WS_VISIBLE,CRect(0,0,0,0),this,0);

	return 0;
}

void CGraphWnd::SetGraphStyles()
{
	//Setup default styles
	if(m_bShowTitle)
	{
		CString str;
		GetWindowText(str);
		m_pChart->SetTitleText(str);
		m_pChart->SetTitleTextColor(m_colTitleText);
		m_pChart->SetTitleFont(m_strTitleFont);
		m_pChart->SetTitleBkColor(m_colTitleBk);
	}
	else
        m_pChart->SetTitleText("");

	// Set the grid and text colours/styles
    m_pChart->SetPlotBkColor(m_colPlotBk);
    m_pChart->SetLabelFont(m_strLabelFont);
    m_pChart->SetGridColor(m_colGrid);
    m_pChart->SetLabelColor(m_colLabel);
    m_pChart->SetAxisTextColor(m_colAxisText);

	//Setup data colors
	COleSafeArray saData;
	saData.CreateOneDim(VT_I4,m_arDefLineColours.GetSize(),m_arDefLineColours.GetData());
    m_pChart->SetDataColors(saData);
}

void CGraphWnd::LoadGraphStyles()
{
	CWinApp *pApp = AfxGetApp();

	m_bShowTitle = pApp->GetProfileInt("GraphStyles","ShowTitle",m_bShowTitle);
	m_strTitleFont = pApp->GetProfileString("GraphTitle","TitleFont", m_strTitleFont);
	m_colTitleText = pApp->GetProfileInt("GraphTitle","TextClr",(int)m_colTitleText);
	m_colTitleBk = pApp->GetProfileInt("GraphTitle","BG Color",(int)m_colTitleBk);
    m_colPlotBk = pApp->GetProfileInt("GraphStyles","BGCol",(int)m_colPlotBk);
    m_strLabelFont = pApp->GetProfileString("GraphStyles","LabelFont", m_strLabelFont);
    m_colGrid = pApp->GetProfileInt("GraphStyles","GridCol",(int)m_colGrid);
    m_colLabel = pApp->GetProfileInt("GraphStyles","LabelCol",(int)m_colLabel);
    m_colAxisText = pApp->GetProfileInt("GraphStyles","ATCol",(int)m_colAxisText);

	CString strKey;
	int nNumLineColors = m_arDefLineColours.GetSize();
	for(int nCol=0;nCol<nNumLineColors;nCol++)
	{
		strKey.Format("LineCol%d",nCol);
		m_arDefLineColours[nCol] = pApp->GetProfileInt("GraphStyles",strKey,(int)m_arDefLineColours[nCol]);
	}
}

void CGraphWnd::SaveGraphStyles()
{
	CWinApp *pApp = AfxGetApp();

	pApp->WriteProfileInt("GraphStyles","ShowTitle",m_bShowTitle);
	pApp->WriteProfileString("GraphTitle","TitleFont", m_strTitleFont);
	pApp->WriteProfileInt("GraphTitle","TextClr",(int)m_colTitleText);
	pApp->WriteProfileInt("GraphTitle","BG Color",(int)m_colTitleBk);
    pApp->WriteProfileInt("GraphStyles","BGCol",(int)m_colPlotBk);
    pApp->WriteProfileString("GraphStyles","LabelFont", m_strLabelFont);
    pApp->WriteProfileInt("GraphStyles","GridCol",(int)m_colGrid);
    pApp->WriteProfileInt("GraphStyles","LabelCol",(int)m_colLabel);
    pApp->WriteProfileInt("GraphStyles","ATCol",(int)m_colAxisText);

	CString strKey;
	int nNumLineColors = m_arDefLineColours.GetSize();
	for(int nCol=0;nCol<nNumLineColors;nCol++)
	{
		strKey.Format("LineCol%d",nCol);
		pApp->WriteProfileInt("GraphStyles",strKey,(int)m_arDefLineColours[nCol]);
	}
}

void CGraphWnd::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
	if(m_pChart)
		m_pChart->MoveWindow(0,0,cx,cy);
}

void CGraphWnd::OnContextMenu(CWnd* pWnd, CPoint point) 
{
    //This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

	AfxSetResourceHandle(m_hInstance);
    CMenu menu;
    VERIFY(menu.LoadMenu(IDR_GRAPHS));

    //hide the export button if Excel is not available
    if(!CExcelExport::IsExcelRegistered())
        menu.RemoveMenu(ID_EXPORT,MF_BYCOMMAND);

    //Add any graph specific menu items
    CGraphTypeBase::MenuItemArray arMenuItems;

    m_pGraph->GetMenuItems(arMenuItems);

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    for(int nNdx=0;nNdx<arMenuItems.GetSize();nNdx++)
    {
        pPopup->InsertMenu(-1,MF_BYPOSITION,arMenuItems[nNdx].uiCommandID,arMenuItems[nNdx].strItem);
        pPopup->EnableMenuItem(arMenuItems[nNdx].uiCommandID,MF_ENABLED|MF_BYCOMMAND);
    }

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);

	AfxSetResourceHandle(AfxGetInstanceHandle());
}

void CGraphWnd::OnExport() 
{
    m_pGraph->ExportToExcel();
}

void CGraphWnd::OnGraphCustomize() 
{
	AfxSetResourceHandle(m_hInstance);
 
    CGraphCustomiseDlg dlg("Customize Graph",this);
    dlg.SetChartStyle(m_pGraph->GetGraphType());
    if(dlg.DoModal()==IDOK)
    {
		SetGraphStyles();
		m_pGraph->SetGraphType(dlg.GetChartStyle());
	}

	AfxSetResourceHandle(AfxGetInstanceHandle());
}

void CGraphWnd::OnGraphMenuCommand(UINT nID)
{
	m_pGraph->OnCommand(nID,this);
}